from typing import *
from itertools import product
from PyTASTileSystem import Glue, TileType, Tile, PyTASTileSystem

# assumptions:
# all {GS_PRE} are followed by at least 1 bit

N = 0
E = 1
S = 2
W = 3

SEED_GLUE = 'SEED'

IN = True
OUT = False

GS_PRE = '#'
GTE_SEP1 = '|1'
GTE_SEP2 = '|2'
GT_PRE = '$'

BLANK = '_'
BLANK2 = '-'
B = ['B0', 'B1', 'B2', 'B3', 'B4', 'B5', 'B6', 'B7']

arrow_map  = { '^': N, '>': E, 'v': S, '<': W }
arrow_name = { '^': 'N', '>': 'E', 'v': 'S', '<': 'W' }
arrow_cw   = { '^':'>', '>':'v', 'v':'<', '<':'^' }
arrow_ccw  = { '^':'<', '>':'^', 'v':'>', '<':'v' }
arrow_opp  = { '^':'v', '>':'<', 'v':'^', '<':'>' }

WHITE = 'rgb(255,255,255)'
GRAY = 'rgb(150,150,150)'
RED = 'rgb(255,50,50)'
GREEN = 'rgb(50,255,50)'
BLUE = 'rgb(50,120,255)'
MAGENTA = 'rgb(255,50,255)'
CYAN = 'rgb(50,255,255)'
YELLOW = 'rgb(255,255,50)'
PURPLE = 'rgb(150,100,255)'

def createType(inputs, outputs, color, prefix):
	sorted_inputs = sorted(inputs)
	name = f'{prefix}({",".join(inputs)})'
	
	seen = [False, False, False, False]

	ttype = TileType(name)
	for g in inputs:
		arrow_dir = arrow_map[g[-1]]
		side = (arrow_dir + 2) % 4

		if seen[side]:
			print('two glues for same direction on a tile')
			exit()
		seen[side] = True

		if len(inputs) == 2:
			strength = 1
		elif len(inputs) == 1:
			strength = 2
		else:
			print('there can only be one or two inputs')
			exit()

		ttype.sides[side] = Glue(strength, g)

	for (g, strength) in outputs:
		arrow_dir = arrow_map[g[-1]]
		side = arrow_dir

		if seen[side]:
			print('two glues for same direction on a tile')
			exit()
		seen[side] = True

		ttype.sides[side] = Glue(strength, g)

	ttype.color = color
	return ttype

def symbcol(symb):
	if symb == '0' or symb == '0$':
		return YELLOW
	elif symb == '1' or symb  == '1$':
		return BLUE
	elif symb in B:
		return PURPLE
	elif symb in [GS_PRE, GTE_SEP1, GTE_SEP2, GT_PRE]:
		return GRAY
	else:
		return WHITE 

def symbxor(sa, sb):
	if sa == '0' and sb == '0':
		return '0'
	elif sa == '1' and sb == '1':
		return '0'
	elif sa == '0' and sb == '1':
		return '1'
	elif sa == '1' and sb == '0':
		return '1'
	elif sa == GS_PRE or sb == GS_PRE:
		return GS_PRE
	elif sa == BLANK:
		return sb
	else:
		return sa

def flip(b):
	if b == '0':
		return '1'
	elif b == '1':
		return '0'
	else:
		return b

def propagate(d='^'):
	na = d
	ea = arrow_cw[na]
	wa = arrow_ccw[na]
	sa = arrow_opp[na]

	nn = arrow_name[na]
	en = arrow_name[ea]
	wn = arrow_name[wa]
	sn = arrow_name[sa]

	ttypes = []
	prefix = ''

	for s_in, w_in in product(['0', '1', GS_PRE, GTE_SEP1, GTE_SEP2, GT_PRE, BLANK], repeat=2):
		col = symbcol(symbxor(s_in, w_in))#symbcol(s_in) if s_in != BLANK else symbcol(w_in)
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'{w_in}_p{en}{ea}'],[(f'{s_in}_p{nn}{na}',1), (f'{w_in}_p{en}{ea}',1)], col, prefix))

	return ttypes

def generic_counter_tiles(d='^'):
	na = d
	ea = arrow_cw[na]
	wa = arrow_ccw[na]
	sa = arrow_opp[na]

	nn = arrow_name[na]
	en = arrow_name[ea]
	wn = arrow_name[wa]
	sn = arrow_name[sa]

	ttypes = []

	ttypes.append(createType([f'lb1*{na}'], [(f'lb0{na}',1), (f'carry{wa}',1), (f'SFO{nn}{ea}',1)], symbcol('0'), ''))
	
	ttypes.append(createType([f'b0{na}', f'no_carry{wa}'], [(f'b0{na}',1), (f'no_carry{wa}',1)], symbcol('0'), ''))
	ttypes.append(createType([f'b1{na}', f'no_carry{wa}'], [(f'b1{na}',1), (f'no_carry{wa}',1)], symbcol('1'), ''))
	ttypes.append(createType([f'b0{na}', f'carry{wa}'], [(f'b1{na}',1), (f'no_carry{wa}',1)], symbcol('1'), ''))
	ttypes.append(createType([f'b1{na}', f'carry{wa}'], [(f'b0{na}',1), (f'carry{wa}',1)], symbcol('0'), ''))

	ttypes.append(createType([f'b0{na}', f'parity{ea}'], [(f'b0{na}',1), (f'parity{ea}',1)], symbcol('0'), ''))
	ttypes.append(createType([f'b1{na}', f'parity{ea}'], [(f'b1{na}',1), (f'parity{ea}',1)], symbcol('1'), ''))
	ttypes.append(createType([f'lb0{na}', f'parity{ea}'], [(f'lb1*{na}',2), (f'SFO{nn}{ea}',1)], symbcol('1'), ''))
	


	return ttypes

# counter_start lsb=index 0 and msb=index -1
def sparse_frame_tiles(d='^', X_binary='0101010', Y_binary='0101'):
	na = d
	ea = arrow_cw[na]
	wa = arrow_ccw[na]
	sa = arrow_opp[na]

	nn = arrow_name[na]
	en = arrow_name[ea]
	wn = arrow_name[wa]
	sn = arrow_name[sa]

	ttypes = []

	X_bits = len(X_binary)

	for i, b in enumerate(X_binary):
		if i == 0:
			if b == '0':
				ttypes.append(createType([f'SFI{nn}_bit{i}{na}'], [(f'lb{b}{na}',1), (f'SFI{nn}_bit{i+1}{wa}',2), (f'SFO{nn}{ea}',1)], symbcol(b), ''))
			else:
				ttypes.append(createType([f'SFI{nn}_bit{i}{na}'], [(f'lb{b}*{na}',2), (f'SFI{nn}_bit{i+1}{wa}',2), (f'SFO{nn}{ea}',1)], symbcol(b), ''))
		else:
			ttypes.append(createType([f'SFI{nn}_bit{i}{wa}'], [(f'b{b}{na}',1), (f'SFI{nn}_bit{i+1}{wa}',2)], symbcol(b), ''))

	if X_binary[0] == '0':
		ttypes.append(createType([f'SFI{nn}_bit{X_bits}{wa}'], [(f'MSB*_{nn}{na}',2), (f'SFI2{nn}_bit0{wa}',2)], RED, ''))
	else:
		ttypes.append(createType([f'SFI{nn}_bit{X_bits}{wa}'], [(f'MSB_{nn}{na}',1), (f'SFI2{nn}_bit0{wa}',2)], RED, ''))

	ttypes.append(createType([f'MSB_{nn}{na}', f'no_carry{wa}'], [(f'MSB*_{nn}{na}',2)], RED, ''))
	ttypes.append(createType([f'MSB_{nn}{na}', f'carry{wa}'], [(f'SFO{nn}*{na}',2), (f'SFI{wn}_bit0{wa}',2)], RED, ''))
	ttypes.append(createType([f'MSB*_{nn}{na}'], [(f'MSB_{nn}{na}',1), (f'parity{ea}',1)], RED, ''))

	Y_bits = len(Y_binary)

	for i, b in enumerate(Y_binary):
		if i == 0:
			if b == '0':
				ttypes.append(createType([f'SFI2{nn}_bit{i}{wa}'], [(f'lb{b}{na}',1), (f'SFI2{nn}_bit{i+1}{wa}',2)], symbcol(b), ''))
			else:
				ttypes.append(createType([f'SFI2{nn}_bit{i}{wa}'], [(f'lb{b}*{na}',2), (f'SFI2{nn}_bit{i+1}{wa}',2)], symbcol(b), ''))
		else:
			ttypes.append(createType([f'SFI2{nn}_bit{i}{wa}'], [(f'b{b}{na}',1), (f'SFI2{nn}_bit{i+1}{wa}',2)], symbcol(b), ''))

	if Y_binary[0] == '0':
		ttypes.append(createType([f'SFI2{nn}_bit{Y_bits}{wa}'], [(f'MSB2*_{nn}{na}',2), (f'{BLANK}_p{wn}{wa}',1)], RED, ''))
	else:
		ttypes.append(createType([f'SFI2{nn}_bit{Y_bits}{wa}'], [(f'MSB2_{nn}{na}',1), (f'{BLANK}_p{wn}{wa}',1)], RED, ''))

	ttypes.append(createType([f'MSB2_{nn}{na}', f'no_carry{wa}'], [(f'MSB2*_{nn}{na}',2), (f'{BLANK}_p{wn}{wa}',1)], RED, ''))
	ttypes.append(createType([f'MSB2_{nn}{na}', f'carry{wa}'], [(f'fill*{wa}',2)], RED, ''))
	ttypes.append(createType([f'MSB2*_{nn}{na}'], [(f'MSB2_{nn}{na}',1), (f'parity{ea}',1), (f'{BLANK}_p{wn}{wa}',1)], RED, ''))

	ttypes.append(createType([f'SFO{nn}*{na}'], [(f'out{nn}{na}',1),(f'circ{nn}{ea}',1)], RED, ''))
	ttypes.append(createType([f'b0{na}', f'circ{nn}{ea}'], [(f'out{nn}{na}',1),(f'circ{nn}{ea}',1)], RED, ''))
	ttypes.append(createType([f'lb0{na}', f'circ{nn}{ea}'], [(f'out{nn}{na}',1),(f'circ*{nn}{ea}',2)], RED, ''))

	ttypes.append(createType([f'circ*{nn}{ea}'], [(f'out{nn}a{na}',1),(f'out{en}a{ea}',1),(f'circ{en}{sa}',1)], RED, ''))
	ttypes.append(createType([f'circ{en}{sa}', f'SFO{nn}{ea}'], [(f'out{en}{ea}',1),(f'circ{en}{sa}',1)], RED, ''))

	ttypes.append(createType([f'fill*{wa}'], [(f'fill**{sa}',1),(f'{BLANK}_p{nn}{na}',1),(f'{BLANK}_p{wn}{wa}',1)], GRAY, ''))
	ttypes.append(createType([f'fill**{sa}', f'{BLANK}_p{wn}{wa}'], [(f'fill*{wa}',2),(f'{BLANK}_p{sn}{sa}',1)], GRAY, ''))

	return ttypes


def gen_square_tile_set(X=1000, Y=300):

	X_binary='0000000000'
	Y_binary='00000000'

	ttypes = []

	ttypes.append(createType([f'{SEED_GLUE}<'], [(f'FA^',1)], MAGENTA, ''))

	ttypes.append(createType([f'FA^', f'west<'], [(f'FA^',1)], MAGENTA, ''))
	ttypes.append(createType([f'FA^', f'west*<'], [(f'FA*^',2)], MAGENTA, ''))
	ttypes.append(createType([f'FA*^'], [(f'FB>', 1),(f'FC^',1), (f'FF0<',2)], MAGENTA, ''))

	ttypes.append(createType([f'{B[0]}^', f'FB>'], [(f'FB>', 1),(f'{BLANK}_pN^',1)], MAGENTA, ''))
	for s_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE, BLANK] + B[1:7]:
		ttypes.append(createType([f'{s_in}^', f'FB>'], [(f'FB>', 1),(f'{s_in}_pN^',1)], MAGENTA, ''))
	ttypes.append(createType([f'{B[7]}^', f'FB>'], [(f'FB*>', 2),(f'rot*_{B[7]}^',2)], MAGENTA, ''))

	for s_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE, BLANK] + B[1:7]:
		ttypes.append(createType([f'rot*_{s_in}^'], [(f'{s_in}_pN^', 1),(f'{s_in}_pE>',1),(f'rot*<',1)], symbcol(s_in), ''))
		ttypes.append(createType([f'{s_in}_pN^', f'rot*<'], [(f'rot*_{s_in}^', 2),(f'{BLANK}_pW<',1)], symbcol(s_in), ''))

	ttypes.append(createType([f'rot*_{B[7]}^'], [(f'{B[7]}*_pN^', 1),(f'{B[7]}_pE>',1),(f'rot*<',1)], symbcol(B[7]), ''))

	for s_in in B[1:7]:
		ttypes.append(createType([f'{s_in}_pN^', f'{BLANK}_pW<'], [(f'{s_in}_pN^', 1),(f'{BLANK}_pW<',1)], symbcol(s_in), ''))
		ttypes.append(createType([f'{s_in}_pE>', f'{BLANK}_pSv'], [(f'{s_in}_pE>', 1),(f'{BLANK}_pSv',1)], symbcol(s_in), ''))

		for w_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE, BLANK] + B[1:7]:
			ttypes.append(createType([f'{s_in}_pN^', f'{w_in}_pE>'], [(f'{s_in}_pN^', 1),(f'{w_in}_pE>',1)], symbcol(symbxor(w_in, s_in)), ''))

	ttypes.append(createType([f'{B[7]}_pN^', f'{BLANK}_pW<'], [(f'{B[7]}_pN^', 1),(f'{BLANK}_pW<',1)], symbcol(B[7]), ''))
	ttypes.append(createType([f'{B[7]}_pE>', f'{BLANK}_pSv'], [(f'{B[7]}_pE>', 1)], symbcol(B[7]), ''))

	for w_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE, BLANK] + B[1:7]:
		ttypes.append(createType([f'{B[7]}*_pN^', f'{w_in}_pE>'], [(f'{B[7]}*_pN^', 1),(f'{w_in}_pE>',1)], symbcol(symbxor(w_in, B[7])), ''))

	for s_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE, BLANK]:
		for w_in in B[1:7]:
			ttypes.append(createType([f'{s_in}_pN^', f'{w_in}_pE>'], [(f'{s_in}_pN^', 1),(f'{w_in}_pE>',1)], symbcol(symbxor(w_in, s_in)), ''))

	ttypes.append(createType([f'{B[7]}_pN^', f'{BLANK}_pE>'], [(f'{B[7]}_pN^', 1)], symbcol(B[7]), ''))

	ttypes.append(createType([f'FC^', f'{BLANK}_pW<'], [(f'FC^', 1)], MAGENTA, ''))
	ttypes.append(createType([f'FC^', f'rot*<'], [(f'FC*^', 2)], MAGENTA, ''))
	ttypes.append(createType([f'FC*^'], [(f'FD0^', 2), (f'FE>',1)], MAGENTA, ''))

	for i in range(len(X_binary)+1):
		ttypes.append(createType([f'FD{i}^'], [(f'FD{i+1}^', 2), (f'{BLANK}_pE>',1)], MAGENTA, ''))
		ttypes.append(createType([f'FF{i}<'], [(f'FF{i+1}<', 2)], MAGENTA, ''))
		ttypes.append(createType([f'FH{i}v'], [(f'FH{i+1}v', 2)], MAGENTA, ''))
		ttypes.append(createType([f'FI{i}>'], [(f'FI{i+1}>', 2), (f'{BLANK}_pSv',1)], MAGENTA, ''))

	ttypes.append(createType([f'FD{len(X_binary)+1}^'], [(f'FD^',1), (f'SFIN_pre1<', 2), (f'{BLANK}_pE>',1)], MAGENTA, ''))
	ttypes.append(createType([f'SFIN_pre1<'], [(f'SFIN_pre2<',2),(f'circEv',1)], RED, ''))
	ttypes.append(createType([f'SFIN_pre2<'], [(f'SFIN_bit0^',2)], RED, ''))

	ttypes.append(createType([f'FF{len(X_binary)+1}<'], [(f'FF<',1), (f'SFIW_pre1v', 2)], MAGENTA, ''))
	ttypes.append(createType([f'SFIW_pre1v'], [(f'SFIW_pre2v',2),(f'circN>',1)], RED, ''))
	ttypes.append(createType([f'SFIW_pre2v'], [(f'SFIW_bit0<',2)], RED, ''))

	ttypes.append(createType([f'FH{len(X_binary)+1}v'], [(f'FHv',1), (f'SFIS_pre1>', 2)], MAGENTA, ''))
	ttypes.append(createType([f'SFIS_pre1>'], [(f'SFIS_pre2>',2),(f'circW^',1)], RED, ''))
	ttypes.append(createType([f'SFIS_pre2>'], [(f'SFIS_bit0v',2)], RED, ''))

	ttypes.append(createType([f'FI{len(X_binary)+1}>'], [(f'FI>',1), (f'SFIE_pre1^', 2), (f'{BLANK}_pSv',1)], MAGENTA, ''))
	ttypes.append(createType([f'SFIE_pre1^'], [(f'SFIE_pre2^',2),(f'circS<',1)], RED, ''))
	ttypes.append(createType([f'SFIE_pre2^'], [(f'SFIE_bit0>',2)], RED, ''))

	for s_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE, BLANK] + B[1:7]:
		ttypes.append(createType([f'{s_in}_pN^', f'FE>'], [(f'{s_in}_pN^', 1), (f'FE>',1)], MAGENTA, ''))
		ttypes.append(createType([f'{s_in}_pE>', f'FG^'], [(f'{s_in}_pE>', 1), (f'FG^',1)], MAGENTA, ''))
	ttypes.append(createType([f'{B[7]}*_pN^', f'FE>'], [(f'{B[7]}_pN^', 1), (f'FE>',1)], MAGENTA, ''))
	ttypes.append(createType([f'{B[7]}_pE>', f'FG^'], [(f'{B[7]}_pE>', 1), (f'FG^',1)], MAGENTA, ''))

	ttypes.append(createType([f'FD^', f'outE>'], [(f'FD^', 1), (f'{BLANK}_pE>',1)], MAGENTA, ''))
	ttypes.append(createType([f'FI>', f'outSv'], [(f'FI>', 1), (f'{BLANK}_pSv',1)], MAGENTA, ''))
	ttypes.append(createType([f'FB*>'], [(f'FG^', 1), (f'FH0v', 2), (f'FJ>', 1)], MAGENTA, ''))
	ttypes.append(createType([f'FG^', f'FE>'], [(f'FI0>', 2), (f'FK^', 1)], MAGENTA, ''))

	ttypes.append(createType([f'FD^', f'outEa>'], [(f'Finalize*^', 2), (f'{BLANK}_pE>',1)], MAGENTA, ''))
	ttypes.append(createType([f'FI>', f'outSav'], [(f'Finalize*>', 2), (f'{BLANK}_pSv',1)], MAGENTA, ''))

	ttypes.append(createType([f'FJ>', f'outNa^'], [(f'FJ>', 1)], MAGENTA, ''))
	ttypes.append(createType([f'FK^', f'outWa<'], [(f'FK^', 1)], MAGENTA, ''))
	ttypes.append(createType([f'FJ>', f'outN^'], [(f'FJ>', 1)], MAGENTA, ''))
	ttypes.append(createType([f'FK^', f'outW<'], [(f'FK^', 1)], MAGENTA, ''))

	ttypes.append(createType([f'Finalize*^'], [(f'FinalizeN>', 1), (f'FinalizeNa<', 1), (f'{BLANK}^', 1)], symbcol(BLANK), ''))
	ttypes.append(createType([f'FinalizeNa<', f'outNa^'], [(f'FinalizeN<', 1), (f'{BLANK}^', 1)], symbcol(BLANK), ''))
	ttypes.append(createType([f'FinalizeN<', f'outN^'], [(f'FinalizeN<', 1), (f'{BLANK}^', 1)], symbcol(BLANK), ''))
	ttypes.append(createType([f'FinalizeN<', f'outNa^'], [(f'FinalizeN*<', 2), (f'{BLANK2}^', 1)], symbcol(BLANK2), ''))
	ttypes.append(createType([f'FinalizeN*<'], [(f'{B[0]}^', 1)], symbcol(B[0]), ''))
	for s_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE, BLANK] + B[1:7]:
		ttypes.append(createType([f'FinalizeN>', f'{s_in}_pN^'], [(f'{s_in}^', 1), (f'FinalizeN>',1)], symbcol(s_in), ''))
	ttypes.append(createType([f'FinalizeN>', f'{B[7]}_pN^'], [(f'{BLANK}^', 1), (f'FinalizeN>',1)], symbcol(BLANK), ''))
	ttypes.append(createType([f'FinalizeN>', f'FK^'], [(f'{BLANK}^', 1), (f'FinalizeN>',1)], symbcol(BLANK), ''))
	ttypes.append(createType([f'FinalizeN>', f'outNa^'], [(f'{BLANK}^', 1), (f'FinalizeNa>',1)], symbcol(BLANK), ''))
	ttypes.append(createType([f'FinalizeNa>', f'outN^'], [(f'{BLANK}^', 1), (f'FinalizeNa>',1)], symbcol(BLANK), ''))
	ttypes.append(createType([f'FinalizeNa>', f'outNa^'], [(f'{BLANK2}^', 1), (f'FinalizeN>',1)], symbcol(BLANK2), ''))


	ttypes.append(createType([f'Finalize*>'], [(f'FinalizeEv', 1), (f'FinalizeEa^', 1), (f'{BLANK}>', 1)], symbcol(BLANK), ''))
	ttypes.append(createType([f'FinalizeEa^', f'outEa>'], [(f'FinalizeE^', 1), (f'{BLANK}>', 1)], symbcol(BLANK), ''))
	ttypes.append(createType([f'FinalizeE^', f'outE>'], [(f'FinalizeE^', 1), (f'{BLANK}>', 1)], symbcol(BLANK), ''))
	ttypes.append(createType([f'FinalizeE^', f'outEa>'], [(f'FinalizeE^', 1), (f'{BLANK2}>', 1)], symbcol(BLANK2), ''))
	for e_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE, BLANK] + B[1:7]:
		ttypes.append(createType([f'FinalizeEv', f'{e_in}_pE>'], [(f'{e_in}>', 1), (f'FinalizeEv',1)], symbcol(e_in), ''))
	ttypes.append(createType([f'FinalizeEv', f'{B[7]}_pE>'], [(f'{BLANK}>', 1), (f'FinalizeEv',1)], symbcol(BLANK), ''))
	ttypes.append(createType([f'FinalizeEv', f'FJ>'], [(f'{BLANK}>', 1), (f'FinalizeEv',1)], symbcol(BLANK), ''))
	ttypes.append(createType([f'FinalizeEv', f'outEa>'], [(f'{BLANK}>', 1), (f'FinalizeEav',1)], symbcol(BLANK), ''))
	ttypes.append(createType([f'FinalizeEav', f'outE>'], [(f'{BLANK}>', 1), (f'FinalizeEav',1)], symbcol(BLANK), ''))
	ttypes.append(createType([f'FinalizeEav', f'outEa>'], [(f'{BLANK2}>', 1), (f'FinalizeE*v',2)], symbcol(BLANK2), ''))
	ttypes.append(createType([f'FinalizeE*v'], [(f'{B[7]}>', 1)], symbcol(B[7]), ''))

	ttypes.append(createType([f'FinalizeN>', f'FinalizeE^'], [(f'{B[7]}^', 1), (f'{B[0]}>', 1)], symbcol(B[0]), ''))


	ttypes += sparse_frame_tiles('^', X_binary, Y_binary) + sparse_frame_tiles('<', X_binary, Y_binary) + sparse_frame_tiles('v', X_binary, Y_binary) + sparse_frame_tiles('>', X_binary, Y_binary)
	ttypes += generic_counter_tiles('^') + generic_counter_tiles('<') + generic_counter_tiles('v') + generic_counter_tiles('>')

	#print(f'generated {len(ttypes)} unique tile types')

	ttypes += propagate('^') + propagate('>') + propagate('v') + propagate('<')

	return ttypes


def create_dummy_seed(H=50):
	#H = 50

	# gX = list('000')
	# gS = list('100')
	# gV = list('101')
	# gH = list('110')
	# g0 = list('010')
	# g1 = list('011')

	# tS_GTE = ((gS, gX, gX, gX), (gV, gH, gX, gX))
	# tV_GTE = ((gV, gX, gX, gX), (gV, g1, gX, gX))
	# tH_GTE = ((gX, gH, gX, gX), (g1, gH, gX, gX))
	
	# t00_GTE = ((g0, g0, gX, gX), (g0, g0, gX, gX))
	# t01_GTE = ((g0, g1, gX, gX), (g1, g1, gX, gX))
	# t10_GTE = ((g1, g0, gX, gX), (g1, g1, gX, gX))
	# t11_GTE = ((g1, g1, gX, gX), (g0, g0, gX, gX))

	# glue_sig = (gS, gX, gX, gX)

	# glue_table = []
	# glue_table += [GT_PRE] + [GS_PRE] + tS_GTE[0][0] + [GS_PRE] + tS_GTE[0][1] + [GS_PRE] + tS_GTE[0][2] + [GS_PRE] + tS_GTE[0][3]
	# glue_table += [GTE_SEP1, GTE_SEP2] + [GS_PRE] + tS_GTE[1][0] + [GS_PRE] + tS_GTE[1][1] + [GS_PRE] + tS_GTE[1][2] + [GS_PRE] + tS_GTE[1][3]
	
	# glue_table += [GT_PRE] + [GS_PRE] + tV_GTE[0][0] + [GS_PRE] + tV_GTE[0][1] + [GS_PRE] + tV_GTE[0][2] + [GS_PRE] + tV_GTE[0][3]
	# glue_table += [GTE_SEP1, GTE_SEP2] + [GS_PRE] + tV_GTE[1][0] + [GS_PRE] + tV_GTE[1][1] + [GS_PRE] + tV_GTE[1][2] + [GS_PRE] + tV_GTE[1][3]

	# glue_table += [GT_PRE] + [GS_PRE] + tH_GTE[0][0] + [GS_PRE] + tH_GTE[0][1] + [GS_PRE] + tH_GTE[0][2] + [GS_PRE] + tH_GTE[0][3]
	# glue_table += [GTE_SEP1, GTE_SEP2] + [GS_PRE] + tH_GTE[1][0] + [GS_PRE] + tH_GTE[1][1] + [GS_PRE] + tH_GTE[1][2] + [GS_PRE] + tH_GTE[1][3]

	# glue_table += [GT_PRE] + [GS_PRE] + t00_GTE[0][0] + [GS_PRE] + t00_GTE[0][1] + [GS_PRE] + t00_GTE[0][2] + [GS_PRE] + t00_GTE[0][3]
	# glue_table += [GTE_SEP1, GTE_SEP2] + [GS_PRE] + t00_GTE[1][0] + [GS_PRE] + t00_GTE[1][1] + [GS_PRE] + t00_GTE[1][2] + [GS_PRE] + t00_GTE[1][3]

	# glue_table += [GT_PRE] + [GS_PRE] + t01_GTE[0][0] + [GS_PRE] + t01_GTE[0][1] + [GS_PRE] + t01_GTE[0][2] + [GS_PRE] + t01_GTE[0][3]
	# glue_table += [GTE_SEP1, GTE_SEP2] + [GS_PRE] + t01_GTE[1][0] + [GS_PRE] + t01_GTE[1][1] + [GS_PRE] + t01_GTE[1][2] + [GS_PRE] + t01_GTE[1][3]

	# glue_table += [GT_PRE] + [GS_PRE] + t10_GTE[0][0] + [GS_PRE] + t10_GTE[0][1] + [GS_PRE] + t10_GTE[0][2] + [GS_PRE] + t10_GTE[0][3]
	# glue_table += [GTE_SEP1, GTE_SEP2] + [GS_PRE] + t10_GTE[1][0] + [GS_PRE] + t10_GTE[1][1] + [GS_PRE] + t10_GTE[1][2] + [GS_PRE] + t10_GTE[1][3]
	
	# glue_table += [GT_PRE] + [GS_PRE] + t11_GTE[0][0] + [GS_PRE] + t11_GTE[0][1] + [GS_PRE] + t11_GTE[0][2] + [GS_PRE] + t11_GTE[0][3]
	# glue_table += [GTE_SEP1, GTE_SEP2] + [GS_PRE] + t11_GTE[1][0] + [GS_PRE] + t11_GTE[1][1] + [GS_PRE] + t11_GTE[1][2] + [GS_PRE] + t11_GTE[1][3]	
	
	# BLANK_SIZE_OUTER = 2
	# BLANK_SIZE_INNER = 2

	# seed_row = []
	# seed_row += [B[0]] + ([BLANK] * BLANK_SIZE_OUTER) 
	# seed_row += [B[1]] + [GS_PRE] + glue_sig[0] + [GS_PRE] + glue_sig[1] + [GS_PRE] + glue_sig[2] + [GS_PRE] + glue_sig[3]
	# seed_row += [B[2]] + ([BLANK] * BLANK_SIZE_INNER)
	# seed_row += [B[3]] + glue_table
	# seed_row += [B[4]] + ([BLANK] * BLANK_SIZE_INNER)
	# seed_row += [B[5]] + [GS_PRE] + glue_sig[0] + [GS_PRE] + glue_sig[1] + [GS_PRE] + glue_sig[2] + [GS_PRE] + glue_sig[3]
	# seed_row += [B[6]] + ([BLANK] * BLANK_SIZE_OUTER)
	# seed_row += [B[7]] 
	
	seed_row = [B[0], BLANK, BLANK, B[1],GS_PRE,'0','1',B[2],BLANK,BLANK,B[3],GT_PRE,GTE_SEP1,GTE_SEP2,B[4],BLANK,BLANK,B[5],GS_PRE,'0','1',B[6], BLANK, BLANK, B[7]]

	sys = PyTASTileSystem()

	for i, symb in enumerate(seed_row):
		name = f'SEED_{symb}'

		if name not in sys.tileTypes:
			ttype = TileType(name)
			ttype.sides[N] = Glue(1, f'{symb}^')
			if i == 0:
				ttype.sides[W] = Glue(1, f'west*<')
			ttype.color = symbcol(symb)
			sys.tileTypes[name] = ttype
		ttype = sys.tileTypes[name]
		sys.seedPatch.append((ttype, (i, H-1)))

	w_ttype = TileType('SEED_W')
	w_ttype.sides[N] = Glue(1, f'X^')
	w_ttype.sides[S] = Glue(1, f'X^')
	w_ttype.sides[E] = Glue(1, f'X>')
	w_ttype.sides[W] = Glue(1, f'west<')
	w_ttype.color = WHITE
	sys.tileTypes[w_ttype.name] = w_ttype

	e_ttype = TileType('SEED_E')
	e_ttype.sides[N] = Glue(1, f'X^')
	e_ttype.sides[S] = Glue(1, f'X^')
	e_ttype.sides[W] = Glue(1, f'X>')
	e_ttype.color = WHITE
	sys.tileTypes[e_ttype.name] = e_ttype

	c_ttype = TileType('SEED_C')
	c_ttype.sides[N] = Glue(1, f'X^')
	c_ttype.sides[S] = Glue(1, f'X^')
	c_ttype.sides[E] = Glue(1, f'X>')
	c_ttype.sides[W] = Glue(1, f'X>')
	c_ttype.color = WHITE
	sys.tileTypes[c_ttype.name] = c_ttype

	se_ttype = TileType('SEED_SE')
	se_ttype.sides[N] = Glue(1, f'X^')
	se_ttype.sides[W] = Glue(1, f'X>')
	se_ttype.color = WHITE
	sys.tileTypes[se_ttype.name] = se_ttype

	s_ttype = TileType('SEED_S')
	s_ttype.sides[N] = Glue(1, f'X^')
	s_ttype.sides[E] = Glue(1, f'X>')
	s_ttype.sides[W] = Glue(1, f'X>')
	s_ttype.color = WHITE
	sys.tileTypes[s_ttype.name] = s_ttype

	sw_ttype = TileType('SEED_SW')
	sw_ttype.sides[N] = Glue(1, f'X^')
	sw_ttype.sides[E] = Glue(1, f'X>')
	sw_ttype.sides[W] = Glue(2, f'{SEED_GLUE}<')
	sw_ttype.color = WHITE
	sys.tileTypes[sw_ttype.name] = sw_ttype

	for x in range(len(seed_row)):
		for y in range(H-1):
			if x == 0:
				if y == 0:
					ttype = sw_ttype
				else:
					ttype = w_ttype
			elif x == len(seed_row) - 1:
				if y == 0:
					ttype = se_ttype
				else:
					ttype = e_ttype 
			elif y == 0:
				ttype = s_ttype
			else:
				ttype = c_ttype

			sys.seedPatch.append((ttype, (x, y)))

	return sys


if __name__ == "__main__":
	sys = create_dummy_seed()

	ttypes = gen_square_tile_set()

	for ttype in ttypes:
		name = ttype.name

		if name in sys.tileTypes:
			print(f'duplicate tile type name: {name}')
			exit()

		sys.tileTypes[name] = ttype

	sys.SaveToFile('square-test')