from typing import *
from itertools import product
from PyTASTileSystem import Glue, TileType, Tile, PyTASTileSystem

# assumptions:
# all {GS_PRE} are followed by at least 1 bit

N = 0
E = 1
S = 2
W = 3

IN = True
OUT = False

GS_PRE = '#'
GTE_SEP1 = '|1'
GTE_SEP2 = '|2'
GT_PRE = '$'

S2 = '!'

BLANK = '_'
BLANK2 = '-'
B = ['B0', 'B1', 'B2', 'B3', 'B4', 'B5', 'B6', 'B7']

arrow_map  = { '^': N, '>': E, 'v': S, '<': W }
arrow_name = { '^': 'N', '>': 'E', 'v': 'S', '<': 'W' }
arrow_cw   = { '^':'>', '>':'v', 'v':'<', '<':'^' }
arrow_ccw  = { '^':'<', '>':'^', 'v':'>', '<':'v' }
arrow_opp  = { '^':'v', '>':'<', 'v':'^', '<':'>' }

WHITE = 'rgb(255,255,255)'
GRAY = 'rgb(150,150,150)'
RED = 'rgb(255,50,50)'
GREEN = 'rgb(50,255,50)'
BLUE = 'rgb(50,120,255)'
MAGENTA = 'rgb(255,50,255)'
CYAN = 'rgb(50,255,255)'
YELLOW = 'rgb(255,255,50)'
PURPLE = 'rgb(150,100,255)'

def symbcol(symb):
	if symb == '0' or symb == '0$':
		return YELLOW
	elif symb == '1' or symb  == '1$':
		return BLUE
	elif symb in B:
		return PURPLE
	elif symb in [GS_PRE, GTE_SEP1, GTE_SEP2, GT_PRE]:
		return GRAY
	else:
		return WHITE 

def symbxor(sa, sb):
	if sa == '0' and sb == '0':
		return '0'
	elif sa == '1' and sb == '1':
		return '0'
	elif sa == '0' and sb == '1':
		return '1'
	elif sa == '1' and sb == '0':
		return '1'
	elif sa == GS_PRE or sb == GS_PRE:
		return GS_PRE
	elif sa == BLANK:
		return sb
	else:
		return sa

def createType(inputs, outputs, color, prefix):
	sorted_inputs = sorted(inputs)
	name = f'{prefix}({",".join(inputs)})'
	
	seen = [False, False, False, False]

	ttype = TileType(name)
	for g in inputs:
		arrow_dir = arrow_map[g[-1]]
		side = (arrow_dir + 2) % 4

		if seen[side]:
			print('two glues for same direction on a tile')
			exit()
		seen[side] = True

		if len(inputs) == 2:
			strength = 1
		elif len(inputs) == 1:
			strength = 2
		else:
			print('there can only be one or two inputs')
			exit()

		ttype.sides[side] = Glue(strength, g)

	for (g, strength) in outputs:
		arrow_dir = arrow_map[g[-1]]
		side = arrow_dir

		if seen[side]:
			print('two glues for same direction on a tile')
			exit()
		seen[side] = True

		ttype.sides[side] = Glue(strength, g)

	ttype.color = color
	return ttype

def propagate(d='^'):
	na = d
	ea = arrow_cw[na]
	wa = arrow_ccw[na]
	sa = arrow_opp[na]

	nn = arrow_name[na]
	en = arrow_name[ea]
	wn = arrow_name[wa]
	sn = arrow_name[sa]

	ttypes = []
	prefix = ''

	for s_in, w_in in product(['0', '1', GS_PRE, GTE_SEP1, GTE_SEP2, GT_PRE, BLANK], repeat=2):
		col = symbcol(symbxor(s_in, w_in))#symbcol(s_in) if s_in != BLANK else symbcol(w_in)
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'{w_in}_p{en}{ea}'],[(f'{s_in}_p{nn}{na}',1), (f'{w_in}_p{en}{ea}',1)], col, prefix))

	return ttypes

def macrotile_output_generic(d='^'):
	na = d
	ea = arrow_cw[na]
	wa = arrow_ccw[na]
	sa = arrow_opp[na]

	nn = arrow_name[na]
	en = arrow_name[ea]
	wn = arrow_name[wa]
	sn = arrow_name[sa]

	ttypes = []
	prefix = ''

	ttypes.append(createType([f'GSW_z_MO{wn}{na}', f'{B[2]}_MO{wn}{wa}'],[(f'pz_MO{wn}{na}',1),(f'{B[2]}_MO{wn}*{wa}',1)], symbcol(B[2]), prefix))
	ttypes.append(createType([f'GSW_p_MO{wn}{na}', f'{B[2]}_MO{wn}{wa}'],[(f'pp_MO{wn}{na}',1),(f'{B[2]}_MO{wn}*{wa}',1)], symbcol(B[2]), prefix))

	for e_in in [BLANK, GS_PRE, GT_PRE, GTE_SEP1, GTE_SEP2, '0', '1', B[3], B[4]]:
		ttypes.append(createType([f'pz_MO{wn}{na}', f'{e_in}_MO{wn}{wa}'],[(f'pz_MO{wn}{na}',1),(f'{e_in}_MO{wn}*{wa}',1)], symbcol(e_in), prefix))
		ttypes.append(createType([f'pp_MO{wn}{na}', f'{e_in}_MO{wn}{wa}'],[(f'pp_MO{wn}{na}',1),(f'{e_in}_MO{wn}*{wa}',1)], symbcol(e_in), prefix))

	for e_in in [BLANK, GS_PRE, GT_PRE, GTE_SEP1, GTE_SEP2, '0', '1', '0$', '1$', B[2], B[3], B[4]]:
		ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'{e_in}_MO{wn}*{wa}'],[(f'{BLANK}_p{nn}{na}',1),(f'{e_in}_MO{wn}*{wa}',1)], symbcol(e_in), prefix))	

	ttypes.append(createType([f'pz_MO{wn}{na}', f'{B[5]}_MO{wn}{wa}'],[(f'GS2_MO{wn}{na}',1),(f'{B[5]}_z_MO{wn}*{wa}',1)], symbcol(B[5]), prefix))
	ttypes.append(createType([f'pp_MO{wn}{na}', f'{B[5]}_MO{wn}{wa}'],[(f'GS2_MO{wn}{na}',1),(f'{B[5]}_p_MO{wn}*{wa}',1)], symbcol(B[5]), prefix))

	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'{B[5]}_z_MO{wn}*{wa}'],[(f'{BLANK}_p{nn}{na}',1),(f'{B[5]}_z_MO{wn}*{wa}',1)], symbcol(B[5]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'{B[5]}_p_MO{wn}*{wa}'],[(f'{BLANK}_p{nn}{na}',1),(f'{B[5]}_p_MO{wn}*{wa}',1)], symbcol(B[5]), prefix))


	ttypes.append(createType([f'GS2W_MO{wn}{na}', f'{B[6]}_MO{wn}{wa}'],[(f'sq{nn}{wn}a{na}',2),(f'{B[6]}_MO{wn}*{wa}',1)], symbcol(B[6]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'{B[6]}_MO{wn}*{wa}'],[(f'sq{nn}{wn}{na}',1),(f'{B[6]}_MO{wn}*{wa}',1)], symbcol(B[6]), prefix))

	for e_in in [BLANK, GS_PRE, GT_PRE, GTE_SEP1, GTE_SEP2, '0', '1', '0$', '1$', B[2], B[3], B[4]]:
		ttypes.append(createType([f'BB_p{nn}{na}', f'{e_in}_MO{wn}*{wa}'],[(f'BB_p{nn}{na}',1),(f'{e_in}_MO{wn}*{wa}',1)], symbcol(B[6]), prefix))
	
	ttypes.append(createType([f'BB_p{nn}{na}', f'{B[5]}_z_MO{wn}*{wa}'],[(f'BB_p{nn}{na}',1),(f'{B[5]}_z_MO{wn}**{wa}',2)], symbcol(B[6]), prefix))
	ttypes.append(createType([f'BB_p{nn}{na}', f'{B[5]}_p_MO{wn}*{wa}'],[(f'BB_p{nn}{na}',1),(f'{B[5]}_p_MO{wn}**{wa}',2)], symbcol(B[6]), prefix))
	ttypes.append(createType([f'BB_p{nn}{na}', f'{B[6]}_MO{wn}*{wa}'],[(f'BB_p{nn}{na}',1),(f'{B[6]}_MO{wn}*{wa}',1)], symbcol(B[6]), prefix))
	ttypes.append(createType([f'BB_p{nn}{na}', f'sq{nn}{wn}{wa}'],[(f'BB_p{nn}{na}',1),(f'{BLANK}_MO{wn}*{wa}',1)], symbcol(B[6]), prefix))
	ttypes.append(createType([f'BB_p{nn}{na}', f'sq{nn}{wn}b{wa}'],[(f'BB_p{nn}*{na}',2),(f'{BLANK}_MO{wn}*{wa}',1)], symbcol(B[6]), prefix))
	ttypes.append(createType([f'BB_p{nn}*{na}'],[(f'{BLANK2}_MO{nn}*{na}',1),(f'BB_p{en}{ea}',1),(f'{BLANK2}_MO{wn}*{wa}',1)], symbcol(B[6]), prefix))
	
	ttypes.append(createType([f'sq{nn}{wn}{na}', f'BB_p{en}{ea}'],[(f'{BLANK}_MO{nn}*{na}',1),(f'BB_p{en}{ea}',1)], symbcol(B[6]), prefix))
	ttypes.append(createType([f'{B[1]}_MO{nn}{na}', f'BB_p{en}{ea}'],[(f'{B[1]}_MO{nn}*{na}',1),(f'BB_p{en}{ea}',1)], symbcol(B[6]), prefix))


	ttypes.append(createType([f'{B[5]}_z_MO{wn}**{wa}'],[(f'BAa_z_p{nn}{na}',1),(f'BAb_z_p{sn}{sa}',1)                   ], symbcol(B[6]), prefix))
	ttypes.append(createType([f'{B[5]}_p_MO{wn}**{wa}'],[(f'BAa_p_p{nn}{na}',1),(f'BAb_p_p{sn}{sa}',1), (f'{B[5]}{wa}',1)], symbcol(B[6]), prefix))

	for e_in in [BLANK,GS_PRE,GT_PRE,GTE_SEP1,GTE_SEP2,'0','1',B[4],B[3],B[2],B[1]]:
		ttypes.append(createType([f'BAb_z_p{sn}{sa}', f'{e_in}_MO{wn}*{wa}'],[(f'BAb_z_p{sn}{sa}',1)                  ], symbcol(B[6]), prefix))
		ttypes.append(createType([f'BAb_p_p{sn}{sa}', f'{e_in}_MO{wn}*{wa}'],[(f'BAb_p_p{sn}{sa}',1),(f'{e_in}{wa}',1)], symbcol(B[6]), prefix))

	ttypes.append(createType([f'BAb_z_p{sn}{sa}', f'0$_MO{wn}*{wa}'],[(f'BAb_z_p{sn}{sa}',1)             ], symbcol(B[6]), prefix))
	ttypes.append(createType([f'BAb_p_p{sn}{sa}', f'0$_MO{wn}*{wa}'],[(f'BAb_p_p{sn}{sa}',1),(f'0{wa}',1)], symbcol(B[6]), prefix))

	ttypes.append(createType([f'BAb_p_p{sn}{sa}', f'1$_MO{wn}*{wa}'],[(f'BAb_p_p{sn}{sa}',1),(f'1!{wa}',2)], symbcol(B[6]), prefix)) # <---------------------------THIS ONE

	for e_in in [GS_PRE,'0','1',B[6],BLANK]:
		ttypes.append(createType([f'BAa_z_p{nn}{na}', f'{e_in}_MO{wn}*{wa}'],[(f'BAa_z_p{nn}{na}',1)                  ], symbcol(B[6]), prefix))
		ttypes.append(createType([f'BAa_p_p{nn}{na}', f'{e_in}_MO{wn}*{wa}'],[(f'BAa_p_p{nn}{na}',1),(f'{e_in}{wa}',1)], symbcol(B[6]), prefix))

	ttypes.append(createType([f'BAb_z_p{sn}{sa}', f'{BLANK2}_MO{wn}**{wa}'],[(f'BAb_z_p{sn}*{sa}',2)                    ], symbcol(B[6]), prefix))
	ttypes.append(createType([f'BAb_p_p{sn}{sa}', f'{BLANK2}_MO{wn}**{wa}'],[(f'BAb_p_p{sn}*{sa}',2),(f'{BLANK2}{wa}',1)], symbcol(B[6]), prefix))

	ttypes.append(createType([f'BAb_z_p{sn}*{sa}'],[                 ], symbcol(B[6]), prefix))
	ttypes.append(createType([f'BAb_p_p{sn}*{sa}'],[(f'{B[0]}{wa}',1)], symbcol(B[6]), prefix))

	ttypes.append(createType([f'BB_p{sn}{sa}', f'{BLANK}_pz{en}{ea}'],[(f'{BLANK2}_MO{en}**{ea}',1)], symbcol(B[6]), prefix))

	ttypes.append(createType([f'BAa_z_p{sn}{sa}', f'{BLANK2}_MO{en}**{ea}'],[(f'BAa_z_p{sn}**{sa}',2)                    ], symbcol(B[6]), prefix))
	ttypes.append(createType([f'BAa_p_p{sn}{sa}', f'{BLANK2}_MO{en}**{ea}'],[(f'BAa_p_p{sn}**{sa}',2),(f'{BLANK2}{ea}',1)], symbcol(B[6]), prefix))

	ttypes.append(createType([f'BAa_z_p{sn}**{sa}'],[                 ], symbcol(B[6]), prefix))
	ttypes.append(createType([f'BAa_p_p{sn}**{sa}'],[(f'{B[7]}{ea}',1)], symbcol(B[6]), prefix))

	ttypes.append(createType([f'BAb_z_p{nn}{na}', f'{BLANK2}_MO{en}*{ea}'],[(f'BAb_z_p{nn}{na}',1)                    ], symbcol(B[6]), prefix))
	ttypes.append(createType([f'BAb_p_p{nn}{na}', f'{BLANK2}_MO{en}*{ea}'],[(f'BAb_p_p{nn}{na}',1),(f'{BLANK2}{ea}',1)], symbcol(B[6]), prefix))

	ttypes.append(createType([f'BAa_z_p{en}{ea}', f'{BLANK2}_MO{nn}*{na}'],[(f'BAa_z_p{en}{ea}',1)                    ], symbcol(B[6]), prefix))
	ttypes.append(createType([f'BAa_p_p{en}{ea}', f'{BLANK2}_MO{nn}*{na}'],[(f'BAa_p_p{en}{ea}',1),(f'{BLANK2}{na}',1)], symbcol(B[6]), prefix))

	ttypes.append(createType([f'BAb_z_p{nn}{na}', f'BAa_z_p{en}{ea}'],[                                   ], symbcol(B[6]), prefix))
	ttypes.append(createType([f'BAb_p_p{nn}{na}', f'BAa_z_p{en}{ea}'],[(f'{B[0]}{ea}',1)                  ], symbcol(B[6]), prefix))
	ttypes.append(createType([f'BAb_z_p{nn}{na}', f'BAa_p_p{en}{ea}'],[(f'{B[7]}{na}',1)                  ], symbcol(B[6]), prefix))
	ttypes.append(createType([f'BAb_p_p{nn}{na}', f'BAa_p_p{en}{ea}'],[(f'{B[7]}{na}',1),(f'{B[0]}{ea}',1)], symbcol(B[6]), prefix))

	return ttypes

def macrotile_output_W():
	ttypes = []
	prefix = ''

	ttypes.append(createType([f'{BLANK}_pN^', f'{GS_PRE}_MOW<'],[(f'GSN_MOW^',1),(f'{GS_PRE}_MOW*<',1)], symbcol(GS_PRE), prefix))
	
	for b in ['0','1']:
		ttypes.append(createType([f'GSN_MOW^', f'{b}_MOW<'],[(f'GSN_MOW^',1),(f'0_MOW*<',1)], symbcol('0'), prefix))
		ttypes.append(createType([f'GSE_MOW^', f'{b}_MOW<'],[(f'GSE_MOW^',1),(f'0_MOW*<',1)], symbcol('0'), prefix))
		ttypes.append(createType([f'GSS_MOW^', f'{b}_MOW<'],[(f'GSS_MOW^',1),(f'0_MOW*<',1)], symbcol('0'), prefix))

	ttypes.append(createType([f'GSN_MOW^', f'{GS_PRE}_MOW<'],[(f'GSE_MOW^',1),(f'{GS_PRE}_MOW*<',1)], symbcol(GS_PRE), prefix))
	ttypes.append(createType([f'GSE_MOW^', f'{GS_PRE}_MOW<'],[(f'GSS_MOW^',1),(f'{GS_PRE}_MOW*<',1)], symbcol(GS_PRE), prefix))
	ttypes.append(createType([f'GSS_MOW^', f'{GS_PRE}_MOW<'],[(f'GSW$_z_MOW^',1),(f'{GS_PRE}_MOW*<',1)], symbcol(GS_PRE), prefix))

	ttypes.append(createType([f'GSW$_z_MOW^', f'0_MOW<'],[(f'GSW_z_MOW^',1),(f'0$_MOW*<',1)], symbcol('0'), prefix))
	ttypes.append(createType([f'GSW$_z_MOW^', f'1_MOW<'],[(f'GSW_p_MOW^',1),(f'1$_MOW*<',1)], symbcol('1'), prefix))

	ttypes.append(createType([f'GSW_z_MOW^', f'0_MOW<'],[(f'GSW_z_MOW^',1),(f'0_MOW*<',1)], symbcol('0'), prefix))
	ttypes.append(createType([f'GSW_z_MOW^', f'1_MOW<'],[(f'GSW_p_MOW^',1),(f'1_MOW*<',1)], symbcol('1'), prefix))
	ttypes.append(createType([f'GSW_p_MOW^', f'0_MOW<'],[(f'GSW_p_MOW^',1),(f'0_MOW*<',1)], symbcol('0'), prefix))
	ttypes.append(createType([f'GSW_p_MOW^', f'1_MOW<'],[(f'GSW_p_MOW^',1),(f'1_MOW*<',1)], symbcol('1'), prefix))


	ttypes.append(createType([f'GS2_MOW^', f'{GS_PRE}_MOW<'],[(f'GS2N_MOW^',1),(f'{GS_PRE}_MOW*<',1)], symbcol(GS_PRE), prefix))

	for b in ['0','1']:
		ttypes.append(createType([f'GS2N_MOW^', f'{b}_MOW<'],[(f'GS2N_MOW^',1),(f'0_MOW*<',1)], symbcol('0'), prefix))
		ttypes.append(createType([f'GS2E_MOW^', f'{b}_MOW<'],[(f'GS2E_MOW^',1),(f'0_MOW*<',1)], symbcol('0'), prefix))
		ttypes.append(createType([f'GS2S_MOW^', f'{b}_MOW<'],[(f'GS2S_MOW^',1),(f'0_MOW*<',1)], symbcol('0'), prefix))
		ttypes.append(createType([f'GS2W_MOW^', f'{b}_MOW<'],[(f'GS2W_MOW^',1),(f'{b}_MOW*<',1)], symbcol(b), prefix))

	ttypes.append(createType([f'GS2N_MOW^', f'{GS_PRE}_MOW<'],[(f'GS2E_MOW^',1),(f'{GS_PRE}_MOW*<',1)], symbcol(GS_PRE), prefix))
	ttypes.append(createType([f'GS2E_MOW^', f'{GS_PRE}_MOW<'],[(f'GS2S_MOW^',1),(f'{GS_PRE}_MOW*<',1)], symbcol(GS_PRE), prefix))
	ttypes.append(createType([f'GS2S_MOW^', f'{GS_PRE}_MOW<'],[(f'GS2W_MOW^',1),(f'{GS_PRE}_MOW*<',1)], symbcol(GS_PRE), prefix))

	return ttypes


def macrotile_output_N():
	ttypes = []
	prefix = ''

	ttypes.append(createType([f'{BLANK}_pE>', f'{GS_PRE}_MON^'],[(f'GSN$_z_MON>',1),(f'{GS_PRE}_MON*^',1)], symbcol(GS_PRE), prefix))
	
	for b in ['0','1']:
		ttypes.append(createType([f'GSE_z_MON>', f'{b}_MON^'],[(f'GSE_z_MON>',1),(f'0_MON*^',1)], symbcol('0'), prefix))
		ttypes.append(createType([f'GSE_p_MON>', f'{b}_MON^'],[(f'GSE_p_MON>',1),(f'0_MON*^',1)], symbcol('0'), prefix))
		
		ttypes.append(createType([f'GSS_z_MON>', f'{b}_MON^'],[(f'GSS_z_MON>',1),(f'0_MON*^',1)], symbcol('0'), prefix))
		ttypes.append(createType([f'GSS_p_MON>', f'{b}_MON^'],[(f'GSS_p_MON>',1),(f'0_MON*^',1)], symbcol('0'), prefix))

		ttypes.append(createType([f'GSW_z_MON>', f'{b}_MON^'],[(f'GSW_z_MON>',1),(f'0_MON*^',1)], symbcol('0'), prefix))
		ttypes.append(createType([f'GSW_p_MON>', f'{b}_MON^'],[(f'GSW_p_MON>',1),(f'0_MON*^',1)], symbcol('0'), prefix))

	ttypes.append(createType([f'GSN_z_MON>', f'{GS_PRE}_MON^'],[(f'GSE_z_MON>',1),(f'{GS_PRE}_MON*^',1)], symbcol(GS_PRE), prefix))
	ttypes.append(createType([f'GSN_p_MON>', f'{GS_PRE}_MON^'],[(f'GSE_p_MON>',1),(f'{GS_PRE}_MON*^',1)], symbcol(GS_PRE), prefix))

	ttypes.append(createType([f'GSE_z_MON>', f'{GS_PRE}_MON^'],[(f'GSS_z_MON>',1),(f'{GS_PRE}_MON*^',1)], symbcol(GS_PRE), prefix))
	ttypes.append(createType([f'GSE_p_MON>', f'{GS_PRE}_MON^'],[(f'GSS_p_MON>',1),(f'{GS_PRE}_MON*^',1)], symbcol(GS_PRE), prefix))

	ttypes.append(createType([f'GSS_z_MON>', f'{GS_PRE}_MON^'],[(f'GSW_z_MON>',1),(f'{GS_PRE}_MON*^',1)], symbcol(GS_PRE), prefix))
	ttypes.append(createType([f'GSS_p_MON>', f'{GS_PRE}_MON^'],[(f'GSW_p_MON>',1),(f'{GS_PRE}_MON*^',1)], symbcol(GS_PRE), prefix))

	ttypes.append(createType([f'GSN$_z_MON>', f'0_MON^'],[(f'GSN_z_MON>',1),(f'0$_MON*^',1)], symbcol('0'), prefix))
	ttypes.append(createType([f'GSN$_z_MON>', f'1_MON^'],[(f'GSN_p_MON>',1),(f'1$_MON*^',1)], symbcol('1'), prefix))

	ttypes.append(createType([f'GSN_z_MON>', f'0_MON^'],[(f'GSN_z_MON>',1),(f'0_MON*^',1)], symbcol('0'), prefix))
	ttypes.append(createType([f'GSN_z_MON>', f'1_MON^'],[(f'GSN_p_MON>',1),(f'1_MON*^',1)], symbcol('1'), prefix))
	ttypes.append(createType([f'GSN_p_MON>', f'0_MON^'],[(f'GSN_p_MON>',1),(f'0_MON*^',1)], symbcol('0'), prefix))
	ttypes.append(createType([f'GSN_p_MON>', f'1_MON^'],[(f'GSN_p_MON>',1),(f'1_MON*^',1)], symbcol('1'), prefix))


	ttypes.append(createType([f'GS2_MON>', f'{GS_PRE}_MON^'],[(f'GS2N_MON>',1),(f'{GS_PRE}_MON*^',1)], symbcol(GS_PRE), prefix))

	for b in ['0','1']:
		ttypes.append(createType([f'GS2N_MON>', f'{b}_MON^'],[(f'GS2N_MON>',1),(f'{b}_MON*^',1)], symbcol(b), prefix))
		ttypes.append(createType([f'GS2E_MON>', f'{b}_MON^'],[(f'GS2E_MON>',1),(f'0_MON*^',1)], symbcol('0'), prefix))
		ttypes.append(createType([f'GS2S_MON>', f'{b}_MON^'],[(f'GS2S_MON>',1),(f'0_MON*^',1)], symbcol('0'), prefix))
		ttypes.append(createType([f'GS2W_MON>', f'{b}_MON^'],[(f'GS2W_MON>',1),(f'0_MON*^',1)], symbcol('0'), prefix))

	ttypes.append(createType([f'GS2N_MON>', f'{GS_PRE}_MON^'],[(f'GS2E_MON>',1),(f'{GS_PRE}_MON*^',1)], symbcol(GS_PRE), prefix))
	ttypes.append(createType([f'GS2E_MON>', f'{GS_PRE}_MON^'],[(f'GS2S_MON>',1),(f'{GS_PRE}_MON*^',1)], symbcol(GS_PRE), prefix))
	ttypes.append(createType([f'GS2S_MON>', f'{GS_PRE}_MON^'],[(f'GS2W_MON>',1),(f'{GS_PRE}_MON*^',1)], symbcol(GS_PRE), prefix))

	return ttypes

def macrotile_output_E():
	ttypes = []
	prefix = ''

	ttypes.append(createType([f'{BLANK}_pSv', f'{GS_PRE}_MOE>'],[(f'GSN_MOEv',1),(f'{GS_PRE}_MOE*>',1)], symbcol(GS_PRE), prefix))
	
	for b in ['0','1']:
		ttypes.append(createType([f'GSN_MOEv', f'{b}_MOE>'],[(f'GSN_MOEv',1),(f'0_MOE*>',1)], symbcol('0'), prefix))

		ttypes.append(createType([f'GSS_z_MOEv', f'{b}_MOE>'],[(f'GSS_z_MOEv',1),(f'0_MOE*>',1)], symbcol('0'), prefix))
		ttypes.append(createType([f'GSS_p_MOEv', f'{b}_MOE>'],[(f'GSS_p_MOEv',1),(f'0_MOE*>',1)], symbcol('0'), prefix))

		ttypes.append(createType([f'GSW_z_MOEv', f'{b}_MOE>'],[(f'GSW_z_MOEv',1),(f'0_MOE*>',1)], symbcol('0'), prefix))
		ttypes.append(createType([f'GSW_p_MOEv', f'{b}_MOE>'],[(f'GSW_p_MOEv',1),(f'0_MOE*>',1)], symbcol('0'), prefix))

	ttypes.append(createType([f'GSN_MOEv', f'{GS_PRE}_MOE>'],[(f'GSE$_z_MOEv',1),(f'{GS_PRE}_MOE*>',1)], symbcol(GS_PRE), prefix))

	ttypes.append(createType([f'GSE_z_MOEv', f'{GS_PRE}_MOE>'],[(f'GSS_z_MOEv',1),(f'{GS_PRE}_MOE*>',1)], symbcol(GS_PRE), prefix))
	ttypes.append(createType([f'GSE_p_MOEv', f'{GS_PRE}_MOE>'],[(f'GSS_p_MOEv',1),(f'{GS_PRE}_MOE*>',1)], symbcol(GS_PRE), prefix))

	ttypes.append(createType([f'GSS_z_MOEv', f'{GS_PRE}_MOE>'],[(f'GSW_z_MOEv',1),(f'{GS_PRE}_MOE*>',1)], symbcol(GS_PRE), prefix))
	ttypes.append(createType([f'GSS_p_MOEv', f'{GS_PRE}_MOE>'],[(f'GSW_p_MOEv',1),(f'{GS_PRE}_MOE*>',1)], symbcol(GS_PRE), prefix))

	ttypes.append(createType([f'GSE$_z_MOEv', f'0_MOE>'],[(f'GSE_z_MOEv',1),(f'0$_MOE*>',1)], symbcol('0'), prefix))
	ttypes.append(createType([f'GSE$_z_MOEv', f'1_MOE>'],[(f'GSE_p_MOEv',1),(f'1$_MOE*>',1)], symbcol('1'), prefix))

	ttypes.append(createType([f'GSE_z_MOEv', f'0_MOE>'],[(f'GSE_z_MOEv',1),(f'0_MOE*>',1)], symbcol('0'), prefix))
	ttypes.append(createType([f'GSE_z_MOEv', f'1_MOE>'],[(f'GSE_p_MOEv',1),(f'1_MOE*>',1)], symbcol('1'), prefix))
	ttypes.append(createType([f'GSE_p_MOEv', f'0_MOE>'],[(f'GSE_p_MOEv',1),(f'0_MOE*>',1)], symbcol('0'), prefix))
	ttypes.append(createType([f'GSE_p_MOEv', f'1_MOE>'],[(f'GSE_p_MOEv',1),(f'1_MOE*>',1)], symbcol('1'), prefix))


	ttypes.append(createType([f'GS2_MOEv', f'{GS_PRE}_MOE>'],[(f'GS2N_MOEv',1),(f'{GS_PRE}_MOE*>',1)], symbcol(GS_PRE), prefix))

	for b in ['0','1']:
		ttypes.append(createType([f'GS2N_MOEv', f'{b}_MOE>'],[(f'GS2N_MOEv',1),(f'0_MOE*>',1)], symbcol('0'), prefix))
		ttypes.append(createType([f'GS2E_MOEv', f'{b}_MOE>'],[(f'GS2E_MOEv',1),(f'{b}_MOE*>',1)], symbcol(b), prefix))
		ttypes.append(createType([f'GS2S_MOEv', f'{b}_MOE>'],[(f'GS2S_MOEv',1),(f'0_MOE*>',1)], symbcol('0'), prefix))
		ttypes.append(createType([f'GS2W_MOEv', f'{b}_MOE>'],[(f'GS2W_MOEv',1),(f'0_MOE*>',1)], symbcol('0'), prefix))

	ttypes.append(createType([f'GS2N_MOEv', f'{GS_PRE}_MOE>'],[(f'GS2E_MOEv',1),(f'{GS_PRE}_MOE*>',1)], symbcol(GS_PRE), prefix))
	ttypes.append(createType([f'GS2E_MOEv', f'{GS_PRE}_MOE>'],[(f'GS2S_MOEv',1),(f'{GS_PRE}_MOE*>',1)], symbcol(GS_PRE), prefix))
	ttypes.append(createType([f'GS2S_MOEv', f'{GS_PRE}_MOE>'],[(f'GS2W_MOEv',1),(f'{GS_PRE}_MOE*>',1)], symbcol(GS_PRE), prefix))

	return ttypes

def macrotile_output_S():
	
	ttypes = []
	prefix = ''

	ttypes.append(createType([f'{BLANK}_pW<', f'{GS_PRE}_MOSv'],[(f'GSN_MOS<',1),(f'{GS_PRE}_MOS*v',1)], symbcol(GS_PRE), prefix))
	
	for b in ['0','1']:
		ttypes.append(createType([f'GSN_MOS<', f'{b}_MOSv'],[(f'GSN_MOS<',1),(f'0_MOS*v',1)], symbcol('0'), prefix))

		ttypes.append(createType([f'GSE_MOS<', f'{b}_MOSv'],[(f'GSE_MOS<',1),(f'0_MOS*v',1)], symbcol('0'), prefix))

		ttypes.append(createType([f'GSW_z_MOS<', f'{b}_MOSv'],[(f'GSW_z_MOS<',1),(f'0_MOS*v',1)], symbcol('0'), prefix))
		ttypes.append(createType([f'GSW_p_MOS<', f'{b}_MOSv'],[(f'GSW_p_MOS<',1),(f'0_MOS*v',1)], symbcol('0'), prefix))

	ttypes.append(createType([f'GSN_MOS<', f'{GS_PRE}_MOSv'],[(f'GSE_MOS<',1),(f'{GS_PRE}_MOS*v',1)], symbcol(GS_PRE), prefix))
	
	ttypes.append(createType([f'GSE_MOS<', f'{GS_PRE}_MOSv'],[(f'GSS$_z_MOS<',1),(f'{GS_PRE}_MOS*v',1)], symbcol(GS_PRE), prefix))

	ttypes.append(createType([f'GSS_z_MOS<', f'{GS_PRE}_MOSv'],[(f'GSW_z_MOS<',1),(f'{GS_PRE}_MOS*v',1)], symbcol(GS_PRE), prefix))
	ttypes.append(createType([f'GSS_p_MOS<', f'{GS_PRE}_MOSv'],[(f'GSW_p_MOS<',1),(f'{GS_PRE}_MOS*v',1)], symbcol(GS_PRE), prefix))

	ttypes.append(createType([f'GSS$_z_MOS<', f'0_MOSv'],[(f'GSS_z_MOS<',1),(f'0$_MOS*v',1)], symbcol('0'), prefix))
	ttypes.append(createType([f'GSS$_z_MOS<', f'1_MOSv'],[(f'GSS_p_MOS<',1),(f'1$_MOS*v',1)], symbcol('1'), prefix))

	ttypes.append(createType([f'GSS_z_MOS<', f'0_MOSv'],[(f'GSS_z_MOS<',1),(f'0_MOS*v',1)], symbcol('0'), prefix))
	ttypes.append(createType([f'GSS_z_MOS<', f'1_MOSv'],[(f'GSS_p_MOS<',1),(f'1_MOS*v',1)], symbcol('1'), prefix))
	ttypes.append(createType([f'GSS_p_MOS<', f'0_MOSv'],[(f'GSS_p_MOS<',1),(f'0_MOS*v',1)], symbcol('0'), prefix))
	ttypes.append(createType([f'GSS_p_MOS<', f'1_MOSv'],[(f'GSS_p_MOS<',1),(f'1_MOS*v',1)], symbcol('1'), prefix))


	ttypes.append(createType([f'GS2_MOS<', f'{GS_PRE}_MOSv'],[(f'GS2N_MOS<',1),(f'{GS_PRE}_MOS*v',1)], symbcol(GS_PRE), prefix))

	for b in ['0','1']:
		ttypes.append(createType([f'GS2N_MOS<', f'{b}_MOSv'],[(f'GS2N_MOS<',1),(f'0_MOS*v',1)], symbcol('0'), prefix))
		ttypes.append(createType([f'GS2E_MOS<', f'{b}_MOSv'],[(f'GS2E_MOS<',1),(f'0_MOS*v',1)], symbcol('0'), prefix))
		ttypes.append(createType([f'GS2S_MOS<', f'{b}_MOSv'],[(f'GS2S_MOS<',1),(f'{b}_MOS*v',1)], symbcol(b), prefix))
		ttypes.append(createType([f'GS2W_MOS<', f'{b}_MOSv'],[(f'GS2W_MOS<',1),(f'0_MOS*v',1)], symbcol('0'), prefix))

	ttypes.append(createType([f'GS2N_MOS<', f'{GS_PRE}_MOSv'],[(f'GS2E_MOS<',1),(f'{GS_PRE}_MOS*v',1)], symbcol(GS_PRE), prefix))
	ttypes.append(createType([f'GS2E_MOS<', f'{GS_PRE}_MOSv'],[(f'GS2S_MOS<',1),(f'{GS_PRE}_MOS*v',1)], symbcol(GS_PRE), prefix))
	ttypes.append(createType([f'GS2S_MOS<', f'{GS_PRE}_MOSv'],[(f'GS2W_MOS<',1),(f'{GS_PRE}_MOS*v',1)], symbcol(GS_PRE), prefix))

	return ttypes


def str2_start(d='^'):
	na = d
	ea = arrow_cw[na]
	wa = arrow_ccw[na]
	sa = arrow_opp[na]

	nn = arrow_name[na]
	en = arrow_name[ea]
	wn = arrow_name[wa]
	sn = arrow_name[sa]

	ttypes = []
	prefix = ''

	ttypes.append(createType([f'1{S2}{na}'],[(f'1_p{nn}{na}',1), (f'S{nn}_R7C1:2{ea}',1), (f'S{nn}_R7C1:2{wa}',1)], symbcol(B[0]), prefix))

	for s_in in ['0', '1', '#']:
		ttypes.append(createType([f'{s_in}{na}', f'S{nn}_R7C1:2{ea}'],[(f'{s_in}_p{nn}{na}',1), (f'S{nn}_R7C1:2{ea}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'{s_in}{na}', f'S{nn}_R7C1:2{wa}'],[(f'{s_in}_p{nn}{na}',1), (f'S{nn}_R7C1:2{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R7C1:2{wa}', f'{B[1]}{na}'],[(f'S{nn}_R6:7C1{na}',1), (f'S{nn}_R7C0:1a{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R7C0:1a{wa}', f'{BLANK}{na}'],[(f'sq{nn}{wn}a{na}',2), (f'S{nn}_R7C0:1{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R7C0:1{wa}', f'{BLANK}{na}'],[(f'sq{nn}{wn}{na}',1), (f'S{nn}_R7C0:1{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'sq{nn}{wn}a{na}'],[(f'sq{nn}{wn}{na}',1), (f'sq{nn}{wn}{ea}',1), (f'sq{nn}{wn}b{wa}',1)], MAGENTA, prefix))
	ttypes.append(createType([f'sq{nn}{wn}{na}', f'sq{nn}{wn}b{wa}'],[(f'sq{nn}{wn}a{na}',2), (f'sq{nn}{wn}{wa}',1)], MAGENTA, prefix))
	ttypes.append(createType([f'sq{nn}{wn}{na}', f'sq{nn}{wn}{wa}'],[(f'sq{nn}{wn}{na}',1), (f'sq{nn}{wn}{wa}',1)], MAGENTA, prefix))
	ttypes.append(createType([f'sq{nn}{wn}{na}', f'sq{nn}{wn}{ea}'],[(f'sq{nn}{wn}{na}',1), (f'sq{nn}{wn}{ea}',1)], 'rgb(220,0,220)', prefix))

	prop_proxy = BLANK

	ttypes.append(createType([f'S{nn}_R6:7C1{na}', f'sq{nn}{wn}{ea}'],[(f'S{nn}_R6:7C1{na}',1), (f'{prop_proxy}_p{en}{ea}',1)], symbcol(B[0]), prefix))


	# ttypes.append(createType([f'S{nn}_R7C0:1{wa}', f'{B[0]}{na}'],[(f'S{nn}_R6:7C0{na}',1), (f'{B[0]}{wa}',1)], symbcol(B[0]), prefix))
	# ttypes.append(createType([f'S{nn}_R6:7C0{na}', f'sq{nn}{wn}{wa}'],[(f'S{nn}_R6:7C0{na}',1), (f'{BLANK}{wa}',1)], symbcol(B[0]), prefix))
	# ttypes.append(createType([f'S{nn}_R6:7C0{na}', f'sq{nn}{wn}b{wa}'],[(f'S{nn}_R6:7C0a{na}',2), (f'{BLANK}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R7C0:1{wa}', f'{BLANK2}{na}'],[(f'S{nn}_BBa{na}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_BBa{na}', f'sq{nn}{wn}{wa}'],[(f'S{nn}_BBb{na}',1), (f'{BLANK2}_MO{wn}**{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_BBb{na}', f'sq{nn}{wn}{wa}'],[(f'S{nn}_BBb{na}',1), (f'{BLANK}_MO{wn}*{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_BBb{na}', f'sq{nn}{wn}b{wa}'],[(f'S{nn}_BBc{na}',2), (f'{BLANK}_MO{wn}*{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_BBc{na}'],[(f'S{nn}_BBd{na}',2),(f'sq{nn}{wn}{ea}',1),(f'{BLANK}_MO{wn}*{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_BBd{na}'],[(f'BB_p{nn}{na}',1),(f'S{nn}_R6C0:1{ea}',1),(f'{B[1]}_MO{wn}*{wa}',1)], symbcol(B[0]), prefix))


	

	ttypes.append(createType([f'{B[2]}{na}', f'S{nn}_R7C1:2{ea}'],[(f'S{nn}_R6:7C2{na}',1), (f'S{nn}_R7C2:3{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R6:7C2{na}', f'{prop_proxy}_p{en}{ea}'],[(f'S{nn}_R6:7C2{na}',1), (f'{prop_proxy}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}{na}', f'S{nn}_R7C2:3{ea}'],[(f'{prop_proxy}_p{nn}{na}',1), (f'S{nn}_R7C2:3{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{B[3]}{na}', f'S{nn}_R7C2:3{ea}'],[(f'S{nn}_R6:7C3{na}',1), (f'S{nn}_R7C3:4{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R6:7C3{na}', f'{prop_proxy}_p{en}{ea}'],[(f'S{nn}_R6:7C3{na}',1), (f'{prop_proxy}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	for s_in in ['0', '1', GS_PRE, GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'{s_in}{na}', f'S{nn}_R7C3:4{ea}'],[(f'{s_in}_p{nn}{na}',1), (f'S{nn}_R7C3:4{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R6:7C0a{na}'],[(f'{B[1]}{wa}',1), (f'S{nn}_R6C0:1a{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R6C0:1a{ea}', f'sq{nn}{wn}{na}'],[(f'0_p{nn}{na}',1), (f'S{nn}_R6C0:1{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R6C0:1{ea}', f'sq{nn}{wn}{na}'],[(f'{BLANK}_p{nn}{na}',1), (f'S{nn}_R6C0:1{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R6:7C1{na}', f'S{nn}_R6C0:1{ea}'],[(f'S{nn}_R5:6C1{na}',1), (f'S{nn}_R6C1:2a{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GS_PRE}_p{nn}{na}', f'S{nn}_R6C1:2a{ea}'],[(f'S{nn}_R6C1:2b{ea}',1)], symbcol(B[0]), prefix))

	for s_in in ['0', '1']:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'S{nn}_R6C1:2b{ea}'],[(f'S{nn}_{s_in}_GCa{na}',2),(f'S{nn}_R6C1:2{ea}',1)], symbcol(B[0]), prefix))

	for s_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'S{nn}_R6C1:2{ea}'],[(f'S{nn}_{s_in}_GC{na}',1),(f'S{nn}_R6C1:2{ea}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'S{nn}_{s_in}_GCa{na}'],[(f'S{nn}_{s_in}_GCb{ea}',1)], symbcol(s_in), prefix))
		for w_in in ['0', '1', GS_PRE]:
			ttypes.append(createType([f'S{nn}_{s_in}_GC{na}', f'S{nn}_{w_in}_GCb{ea}'],[(f'S{nn}_{w_in}_GCa{na}',2),(f'S{nn}_{s_in}_GC{ea}',1)], symbcol(s_in), prefix))
			ttypes.append(createType([f'S{nn}_{s_in}_GC{na}', f'S{nn}_{w_in}_GC{ea}'],[(f'S{nn}_{w_in}_GC{na}',1),(f'S{nn}_{s_in}_GC{ea}',1)], symbcol(s_in), prefix))

	ttypes.append(createType([f'S{nn}_R6:7C2{na}', f'S{nn}_R6C1:2{ea}'],[(f'S{nn}_R5:6C2{na}',1),(f'S{nn}_R6C2:3{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{prop_proxy}_p{nn}{na}', f'S{nn}_R6C2:3{ea}'],[(f'{prop_proxy}_p{nn}{na}',1),(f'S{nn}_R6C2:3{ea}',1)], symbcol(B[0]), prefix))

	for w_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'S{nn}_R5:6C2{na}', f'S{nn}_{w_in}_GC{ea}'],[(f'S{nn}_R5:6C2{na}',1),(f'{w_in}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	for w_in in ['0', '1']:
		ttypes.append(createType([f'S{nn}_R5:6C2{na}', f'S{nn}_{w_in}_GCb{ea}'],[(f'S{nn}_R5:6C2a{na}',2),(f'{w_in}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R5:6C2a{na}'],[(f'S{nn}_R5:6C2b{na}',2),(f'{GS_PRE}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R6:7C3{na}', f'S{nn}_R6C2:3{ea}'],[(f'S{nn}_R5:6C3{na}',1), (f'S{nn}_R6C3:4{ea}',1)], symbcol(B[0]), prefix))
	
	for w_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'S{nn}_R5:6C3{na}', f'{w_in}_p{en}{ea}'],[(f'S{nn}_R5:6C3{na}',1), (f'{w_in}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	for s_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'S{nn}_R6C3:4{ea}'],[(f'{s_in}_GT{nn}{na}',1), (f'S{nn}_R6C3:4{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GT_PRE}_p{nn}{na}', f'S{nn}_R6C3:4{ea}'],[(f'{GT_PRE}_GT{nn}a{na}',1), (f'S{nn}_R6C3:4{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GTE_SEP1}_p{nn}{na}', f'S{nn}_R6C3:4{ea}'],[(f'{GTE_SEP1}_GT{nn}a{na}',1), (f'S{nn}_R6C3:4{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GTE_SEP2}_p{nn}{na}', f'S{nn}_R6C3:4{ea}'],[(f'S{nn}_R6C3:4{ea}',1)], symbcol(B[0]), prefix))
	

	ttypes.append(createType([f'S{nn}_R5:6C2b{na}'],[(f'S{nn}_R4:5C2{na}',1),(f'S{nn}_R5C2:3{ea}',1),(f'S{nn}_R5C1:2a{wa}', 1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'S{nn}_R5C2:3{ea}'],[(f'sq{nn}{wn}{na}',1),(f'S{nn}_R5C2:3{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R5:6C3{na}', f'S{nn}_R5C2:3{ea}'],[(f'S{nn}_R4:5C3a{na}',2),(f'S{nn}_R5C3:4{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R4:5C3a{na}'],[(f'S{nn}_R4:5C3{na}',1),(f'{BLANK}_p{en}{ea}',1),(f'sq{nn}{wn}x{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'sq{nn}{wn}{na}', f'sq{nn}{wn}x{wa}'],[(f'sq{nn}{wn}{na}',1),(f'sq{nn}{wn}b{wa}',1)], MAGENTA, prefix))

	ttypes.append(createType([f'S{nn}_R4:5C3{na}', f'sq{nn}{wn}{ea}'],[(f'S{nn}_R4:5C3{na}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R4:5C2{na}', f'sq{nn}{wn}{wa}'],[(f'S{nn}_R4:5C2{na}',1),(f'{BLANK}_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R4:5C2{na}', f'sq{nn}{wn}b{wa}'],[(f'S{nn}_R4:5C2a{na}',2),(f'{BLANK}_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R4:5C2a{na}'],[(f'S{nn}_R3:4C2{na}',1),(f'S{nn}_R4C2:3{ea}',1),(f'S{nn}_R4C1:2{wa}',1)], symbcol(B[0]), prefix))

	for s_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'{s_in}_GT{nn}{na}', f'S{nn}_R5C3:4{ea}'],[(f'{s_in}_p{nn}{na}',1), (f'S{nn}_R5C3:4{ea}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'S{nn}_R5C3:4{ea}'],[(f'{s_in}_p{nn}{na}',1), (f'S{nn}_R5C3:4{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GT_PRE}_GT{nn}b{na}', f'S{nn}_R5C3:4{ea}'],[(f'{s_in}_p{nn}{na}',1), (f'S{nn}_R5C3:4{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'sq{nn}{wn}{na}', f'S{nn}_R4C2:3{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'S{nn}_R4C2:3{ea}',1)], symbcol(B[0]), prefix))

	####

	ttypes.append(createType([f'S{nn}_R4:5C3{na}', f'S{nn}_R4C2:3{ea}'],[(f'S{nn}_R3:4C3{na}',1),(f'S{nn}_R4C3:4a{ea}',1)], symbcol(B[0]), prefix))
	
	ttypes.append(createType([f'{GT_PRE}_p{nn}{na}', f'S{nn}_R4C3:4a{ea}'],[(f'{GT_PRE}_r3{nn}a{na}',2),(f'S{nn}_R4C3:4{ea}',1)], symbcol(B[0]), prefix))
	for s_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'S{nn}_R4C3:4{ea}'],[(f'{s_in}_r3{nn}{na}',1),(f'S{nn}_R4C3:4{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{GT_PRE}_r3{nn}a{na}'],[(f'{GT_PRE}_p{nn}{na}',1),(f'{GT_PRE}_r3{nn}a{ea}',1),(f'{GT_PRE}_p{wn}{wa}',1)], symbcol(GT_PRE), prefix))
	ttypes.append(createType([f'{GS_PRE}_r3{nn}a{na}'],[(f'{GS_PRE}_p{nn}{na}',1),(f'{GS_PRE}_r3{nn}a{ea}',1),(f'{GS_PRE}_p{wn}{wa}',1)], symbcol(GS_PRE), prefix))

	for s_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'{s_in}_r3{nn}{na}', f'{GT_PRE}_r3{nn}a{ea}'],[(f'{s_in}+{GT_PRE}_r3{nn}a{na}',2),(f'{s_in}_r3{nn}{ea}',1)], symbcol(symbxor(s_in, GT_PRE)), prefix))
		ttypes.append(createType([f'{s_in}_r3{nn}{na}', f'{GS_PRE}_r3{nn}a{ea}'],[(f'{s_in}+{GS_PRE}_r3{nn}a{na}',2),(f'{s_in}_r3{nn}{ea}',1)], symbcol(symbxor(s_in, GS_PRE)), prefix))
	
	for s_in, w_in in product(['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE], repeat=2):
		ttypes.append(createType([f'{s_in}_r3{nn}{na}', f'{w_in}_r3{nn}{ea}'],[(f'{s_in}+{w_in}_r3{nn}{na}',1),(f'{s_in}_r3{nn}{ea}',1)], symbcol(symbxor(s_in, w_in)), prefix))
	
	for s_in1 in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'{s_in1}+{GT_PRE}_r3{nn}a{na}'],[(f'{s_in1}_p{nn}{na}',1),(f'{GT_PRE}_r3{nn}a{ea}',1), (f'{s_in1}_p{wn}{wa}',1)], symbcol(symbxor(s_in1, GT_PRE)), prefix))
		ttypes.append(createType([f'{s_in1}+{GS_PRE}_r3{nn}a{na}'],[(f'{s_in1}_p{nn}{na}',1),(f'{GS_PRE}_r3{nn}a{ea}',1), (f'{s_in1}_p{wn}{wa}',1)], symbcol(symbxor(s_in1, GS_PRE)), prefix))

	for s_in1, s_in2 in product(['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE], repeat=2):
		ttypes.append(createType([f'{s_in1}+{s_in2}_r3{nn}{na}', f'{GT_PRE}_r3{nn}a{ea}'],[(f'{s_in1}+{GT_PRE}_r3{nn}a{na}',2),(f'{s_in2}_r3{nn}{ea}',1)], symbcol(symbxor(s_in2, GT_PRE)), prefix))
		ttypes.append(createType([f'{s_in1}+{s_in2}_r3{nn}{na}', f'{GS_PRE}_r3{nn}a{ea}'],[(f'{s_in1}+{GS_PRE}_r3{nn}a{na}',2),(f'{s_in2}_r3{nn}{ea}',1)], symbcol(symbxor(s_in2, GS_PRE)), prefix))

	for s_in1, s_in2, w_in in product(['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE], repeat=3):
		ttypes.append(createType([f'{s_in1}+{s_in2}_r3{nn}{na}', f'{w_in}_r3{nn}{ea}'],[(f'{s_in1}+{w_in}_r3{nn}{na}',1),(f'{s_in2}_r3{nn}{ea}',1)], symbcol(symbxor(s_in2, w_in)), prefix))
	
	# end 3 way rotation

	ttypes.append(createType([f'{B[4]}{na}', f'S{nn}_R7C3:4{ea}'],[(f'S{nn}_R6:7C4{na}',1),(f'S{nn}_R7C4:5{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}{na}', f'S{nn}_R7C4:5{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'S{nn}_R7C4:5{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R6:7C4{na}', f'{BLANK}_p{en}{ea}'],[(f'S{nn}_R6:7C4{na}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{B[5]}{na}', f'S{nn}_R7C4:5{ea}'],[(f'S{nn}_R6:7C5{na}',1),(f'S{nn}_R7C5:6{ea}',1)], symbcol(B[0]), prefix))
	for s_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'{s_in}{na}', f'S{nn}_R7C5:6{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'S{nn}_R7C5:6{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R6:7C5{na}', f'{BLANK}_p{en}{ea}'],[(f'S{nn}_R6:7C5{na}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))



	ttypes.append(createType([f'{B[6]}{na}', f'S{nn}_R7C5:6{ea}'],[(f'S{nn}_R6:7C6a{na}',1),(f'S{nn}_R7C6:7{ea}',1)], symbcol(B[0]), prefix))
	# ttypes.append(createType([f'{BLANK}{na}', f'S{nn}_R7C6:7{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'S{nn}_R7C6:7{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R6:7C6a{na}', f'{BLANK}_p{en}{ea}'],[(f'S{nn}_R6:7C6{na}',1),(f'{BLANK}_pz{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R6:7C6{na}', f'{BLANK}_p{en}{ea}'],[(f'S{nn}_R6:7C6{na}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	# ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'{BLANK}_pz{en}{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'{BLANK}_pz{en}{ea}',1)], symbcol(BLANK), prefix))

	ttypes.append(createType([f'S{nn}_R6:7C4{na}', f'S{nn}_R6C3:4{ea}'],[(f'S{nn}_R5:6C4{na}',1),(f'S{nn}_R6C4:5{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'S{nn}_R6C4:5{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'S{nn}_R6C4:5{ea}',1)], symbcol(B[0]), prefix))
	for w_in in ['0', '1']:
		ttypes.append(createType([f'S{nn}_R5:6C4{na}', f'P+{w_in}_GTF{nn}{ea}'],[(f'S{nn}_R5:6C4{na}',1),(f'{w_in}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	for w_in1,w_in2 in product(['0', '1'],['0', '1']):
		ttypes.append(createType([f'S{nn}_R5:6C4{na}', f'M+{w_in1}+{w_in2}_GTF{nn}{ea}'],[(f'S{nn}_R5:6C4{na}',1),(f'{w_in2}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	for w_in1,w_in2 in product(['0', '1'],['0', '1']):
		ttypes.append(createType([f'S{nn}_R5:6C4{na}', f'{w_in1}+{w_in2}_GTF{nn}{ea}'],[(f'S{nn}_R5:6C4{na}',1),(f'{w_in2}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R5:6C4{na}', f'{GS_PRE}+{GS_PRE}_GTF{nn}{ea}'],[(f'S{nn}_R5:6C4{na}',1),(f'{GS_PRE}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	for w_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'S{nn}_R5:6C4{na}', f'{w_in}_p{en}{ea}'],[(f'S{nn}_R5:6C4{na}',1),(f'{w_in}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R6:7C5{na}', f'S{nn}_R6C4:5{ea}'],[(f'S{nn}_R5:6C5{na}',1),(f'S{nn}_R6C5:6{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'S{nn}_R6C5:6{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'S{nn}_R6C5:6{ea}',1)], symbcol(B[0]), prefix))
	for w_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'S{nn}_R5:6C5{na}', f'{w_in}_p{en}{ea}'],[(f'S{nn}_R5:6C5{na}',1),(f'{w_in}_r{en}{nn}{ea}',1)], symbcol(B[0]), prefix))


	# rotation tile types begin
	
	for w_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'{w_in}_r{en}{nn}{ea}'],[(f'{w_in}_r{en}{nn}{na}',1),(f'{w_in}_p{en}{ea}',1)], symbcol(w_in), prefix))
		for s_in in ['0','1',GS_PRE]:
			ttypes.append(createType([f'{s_in}_r{en}{nn}{na}', f'{w_in}_r{en}{nn}{ea}'],[(f'{w_in}_r{en}{nn}{na}',1),(f'{w_in}+{s_in}_r{en}{nn}{ea}',1)], symbcol(symbxor(w_in, s_in)), prefix))
	for w_in1, w_in2 in product(['0','1',GS_PRE], repeat=2):
		ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'{w_in1}+{w_in2}_r{en}{nn}{ea}'],[(f'{w_in2}_r{en}{nn}{na}',1),(f'{w_in1}_p{en}{ea}',1)], symbcol(symbxor(w_in1, w_in2)), prefix))
		for s_in in ['0','1',GS_PRE]:
			ttypes.append(createType([f'{s_in}_r{en}{nn}{na}', f'{w_in1}+{w_in2}_r{en}{nn}{ea}'],[(f'{w_in2}_r{en}{nn}{na}',1),(f'{w_in1}+{s_in}_r{en}{nn}{ea}',1)], symbcol(symbxor(s_in, w_in2)), prefix))
	
	# end rotation tile types

	ttypes.append(createType([f'S{nn}_R5:6C4{na}', f'S{nn}_R5C3:4{ea}'],[(f'S{nn}_R4:5C4{na}',1),(f'S{nn}_R5C4:5{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'S{nn}_R5C4:5{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'S{nn}_R5C4:5{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R4:5C4{na}', f'{BLANK}_p{en}{ea}'],[(f'S{nn}_R4:5C4{na}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R5:6C5{na}', f'S{nn}_R5C4:5{ea}'],[(f'S{nn}_R4:5C5{na}',1),(f'S{nn}_R5C5:6{ea}',1)], symbcol(B[0]), prefix))
	for s_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'{s_in}_r{en}{nn}{na}', f'S{nn}_R5C5:6{ea}'],[(f'{s_in}_p{nn}{na}',1),(f'S{nn}_R5C5:6{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R4:5C5{na}', f'{BLANK}_p{en}{ea}'],[(f'S{nn}_R4:5C5{na}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R4:5C4{na}', f'S{nn}_R4C3:4{ea}'],[(f'S{nn}_R3:4C4{na}',1),(f'S{nn}_R4C4:5{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'S{nn}_R4C4:5{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'S{nn}_R4C4:5{ea}',1)], symbcol(B[0]), prefix))
	for w_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'S{nn}_R3:4C4{na}', f'{w_in}_r3{nn}{ea}'],[(f'S{nn}_R3:4C4{na}',1),(f'{w_in}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R3:4C4{na}', f'{GT_PRE}_r3{nn}a{ea}'],[(f'S{nn}_R3:4C4a{na}',2),(f'{GT_PRE}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	
	ttypes.append(createType([f'S{nn}_R4:5C5{na}', f'S{nn}_R4C4:5{ea}'],[(f'S{nn}_R3:4C5{na}',1),(f'S{nn}_R4C5:6{ea}',1)], symbcol(B[0]), prefix))
	for s_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'S{nn}_R4C5:6{ea}'],[(f'{s_in}_p{nn}{na}',1),(f'S{nn}_R4C5:6{ea}',1)], symbcol(B[0]), prefix))
	for w_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'S{nn}_R3:4C5{na}', f'{w_in}_p{en}{ea}'],[(f'S{nn}_R3:4C5{na}',1),(f'{w_in}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R3:4C4a{na}'],[(f'S{nn}_R2:3C4{na}',1),(f'S{nn}_R3C4:5{ea}',1), (f'S{nn}_R3C3:4{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'S{nn}_R3C4:5{ea}'],[(f'sq{nn}{wn}{na}',1),(f'S{nn}_R3C4:5{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R3:4C5{na}', f'S{nn}_R3C4:5{ea}'],[(f'S{nn}_R2:3C5a{na}',2),(f'S{nn}_R3C5:6{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R2:3C5a{na}'],[(f'S{nn}_R2:3C5{na}',1),(f'{BLANK}_p{en}{ea}',1), (f'sq{nn}{wn}x{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R2:3C5{na}', f'sq{nn}{wn}{ea}'],[(f'S{nn}_R2:3C5{na}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	for s_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'S{nn}_R3C5:6{ea}'],[(f'{s_in}_p{nn}{na}',1),(f'S{nn}_R3C5:6{ea}',1)], symbcol(B[0]), prefix))
	
	for s_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'S{nn}_R3C3:4{wa}'],[(f'{s_in}_p{nn}{na}',1),(f'S{nn}_R3C3:4{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R2:3C4{na}', f'sq{nn}{wn}{wa}'],[(f'S{nn}_R2:3C4{na}',1),(f'{BLANK}_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R2:3C4{na}', f'sq{nn}{wn}b{wa}'],[(f'S{nn}_R2:3C4a{na}',2),(f'{BLANK}_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R2:3C4a{na}'],[(f'S{nn}_R1:2C4{na}',1),(f'S{nn}_R2C4:5{ea}',1),(f'S{nn}_R2C3:4{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'sq{nn}{wn}{na}', f'S{nn}_R2C4:5{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'S{nn}_R2C4:5{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R2:3C5{na}', f'S{nn}_R2C4:5{ea}'],[(f'S{nn}_R1:2C5{na}',1),(f'S{nn}_R2C5:6a{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{GS_PRE}_p{nn}{na}', f'S{nn}_R2C5:6a{ea}'],[(f'{GS_PRE}_r3{nn}a{na}',2),(f'S{nn}_R2C5:6{ea}',1)], symbcol(B[0]), prefix))
	for s_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'S{nn}_R2C5:6{ea}'],[(f'{s_in}_r3{nn}{na}',1),(f'S{nn}_R2C5:6{ea}',1)], symbcol(B[0]), prefix))

	for e_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'S{nn}_R1:2C5{na}', f'{e_in}_p{wn}{wa}'],[(f'S{nn}_R1:2C5{na}',1),(f'{e_in}_p{wn}{wa}',1)], symbcol(B[0]), prefix))

	for e_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'S{nn}_R3:4C3{na}', f'{e_in}_p{wn}{wa}'],[(f'S{nn}_R3:4C3{na}',1),(f'{e_in}_p{wn}{wa}',1)], symbcol(B[0]), prefix))

	for e_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'S{nn}_R1:2C4{na}', f'{e_in}_p{wn}{wa}'],[(f'S{nn}_R1:2C4{na}',1),(f'{e_in}_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	for s_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'S{nn}_R2C3:4{wa}'],[(f'{s_in}_p{nn}{na}',1),(f'S{nn}_R2C3:4{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R3:4C3{na}', f'S{nn}_R3C3:4{wa}'],[(f'S{nn}_R2:3C3{na}',1),(f'S{nn}_R3C2:3{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R2:3C3{na}', f'{BLANK}_p{wn}{wa}'],[(f'S{nn}_R2:3C3{na}',1),(f'{BLANK}_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'S{nn}_R3C2:3{wa}'],[(f'{BLANK}_p{nn}{na}',1),(f'S{nn}_R3C2:3{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R2:3C3{na}', f'S{nn}_R2C3:4{wa}'],[(f'S{nn}_R1:2C3{na}',1),(f'S{nn}_R2C2:3{wa}',1)], symbcol(B[0]), prefix))
	for e_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'S{nn}_R1:2C3{na}', f'{e_in}_p{wn}{wa}'],[(f'S{nn}_R1:2C3{na}',1),(f'{e_in}_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'S{nn}_R2C2:3{wa}'],[(f'{BLANK}_p{nn}{na}',1),(f'S{nn}_R2C2:3{wa}',1)], symbcol(B[0]), prefix))

	for e_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'S{nn}_R3:4C2{na}', f'{e_in}_p{wn}{wa}'],[(f'S{nn}_R3:4C2{na}',1),(f'{e_in}_p{wn}{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R3:4C2{na}', f'S{nn}_R3C2:3{wa}'],[(f'S{nn}_R2:3C2{na}',1),(f'S{nn}_R3C1:2{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R2:3C2{na}', f'{BLANK}_p{wn}{wa}'],[(f'S{nn}_R2:3C2{na}',1),(f'{BLANK}_p{wn}{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R2:3C2{na}', f'S{nn}_R2C2:3{wa}'],[(f'S{nn}_R1:2C2a{na}',1),(f'S{nn}_R2C1:2{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R1:2C2a{na}', f'{GS_PRE}_p{wn}{wa}'],[(f'S{nn}_R1:2C2{na}',1),(f'{GS_PRE}_r3{wn}a{wa}',2)], symbcol(B[0]), prefix))
	for e_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'S{nn}_R1:2C2{na}', f'{e_in}_p{wn}{wa}'],[(f'S{nn}_R1:2C2{na}',1),(f'{e_in}_r3{wn}{wa}',1)], symbcol(B[0]), prefix))

	for n_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'{n_in}_p{sn}{sa}', f'S{nn}_R2C1:2{wa}'],[(f'{n_in}_p{sn}{sa}',1),(f'S{nn}_R2C1:2{wa}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'{n_in}_p{sn}{sa}', f'S{nn}_R3C1:2{wa}'],[(f'{n_in}_p{sn}{sa}',1),(f'S{nn}_R3C1:2{wa}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'{n_in}_p{sn}{sa}', f'S{nn}_R4C1:2{wa}'],[(f'{n_in}_p{sn}{sa}',1),(f'S{nn}_R4C1:2{wa}',1)], symbcol(B[0]), prefix))
		
	ttypes.append(createType([f'{GS_PRE}_p{sn}{sa}', f'S{nn}_R5C1:2a{wa}'],[(f'{GS_PRE}_dr{nn}a{sa}',2),(f'S{nn}_R5C1:2{wa}',1)], symbcol(B[0]), prefix))
	for n_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'{n_in}_p{sn}{sa}', f'S{nn}_R5C1:2{wa}'],[(f'{n_in}_dr{nn}{sa}',1),(f'S{nn}_R5C1:2{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{GS_PRE}_dr{nn}a{sa}'],[(f'{GS_PRE}_dr{nn}a{wa}',1)], symbcol(GS_PRE), prefix))
	for n_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'{n_in}_dr{nn}{sa}', f'{GS_PRE}_dr{nn}a{wa}'],[(f'{GS_PRE}_dr{nn}a{sa}',2),(f'{n_in}_dr{nn}{wa}',1)], symbcol(n_in), prefix))

	for n_in, e_in in product(['0','1',GS_PRE], repeat=2):
		ttypes.append(createType([f'{n_in}_dr{nn}{sa}', f'{e_in}_dr{nn}{wa}'],[(f'{e_in}_dr{nn}{sa}',1),(f'{n_in}_dr{nn}{wa}',1)], symbcol(n_in), prefix))

	###### Initiate macrotile outputs

	ttypes.append(createType([f'S{nn}_R5:6C1{na}', f'{GS_PRE}_dr{nn}a{wa}'],[(f'S{nn}_MOI{wn}{na}',1),(f'{GS_PRE}_MO{wn}{wa}',1)], symbcol(B[0]), prefix))
	for e_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'S{nn}_MOI{wn}{na}', f'{e_in}_dr{nn}{wa}'],[(f'S{nn}_MOI{wn}{na}',1),(f'{e_in}_MO{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_MOI{wn}{na}', f'S{nn}_R5C1:2{wa}'],[(f'S{nn}_MOI{wn}{na}',1),(f'{B[2]}_MO{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_MOI{wn}{na}', f'{BLANK}_p{wn}{wa}'],[(f'S{nn}_MOI{wn}{na}',1),(f'{BLANK}_MO{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_MOI{wn}{na}', f'S{nn}_R4C1:2{wa}'],[(f'S{nn}_MOI{wn}{na}',1),(f'{B[3]}_MO{wn}{wa}',1)], symbcol(B[0]), prefix))
	for e_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'S{nn}_MOI{wn}{na}', f'{e_in}_p{wn}{wa}'],[(f'S{nn}_MOI{wn}{na}',1),(f'{e_in}_MO{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_MOI{wn}{na}', f'S{nn}_R3C1:2{wa}'],[(f'S{nn}_MOI{wn}{na}',1),(f'{B[4]}_MO{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_MOI{wn}{na}', f'S{nn}_R2C1:2{wa}'],[(f'S{nn}_MOI{wn}{na}',1),(f'{B[5]}_MO{wn}{wa}',1)], symbcol(B[0]), prefix))



	ttypes.append(createType([f'S{nn}_R6:7C6{na}', f'S{nn}_R6C5:6{ea}'],[(f'S{nn}_MOI{en}{na}',1),(f'S{nn}_R6C6:7{ea}',1)], symbcol(B[0]), prefix))
	for w_in in ['0','1',GS_PRE,GTE_SEP1,GTE_SEP2,GT_PRE,BLANK]:
		ttypes.append(createType([f'S{nn}_MOI{en}{na}', f'{w_in}_p{en}{ea}'],[(f'S{nn}_MOI{en}{na}',1),(f'{w_in}_MO{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_MOI{en}{na}', f'S{nn}_R5C5:6{ea}'],[(f'S{nn}_MOI{en}{na}',1),(f'{B[5]}_MO{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_MOI{en}{na}', f'S{nn}_R4C5:6{ea}'],[(f'S{nn}_MOI{en}{na}',1),(f'{B[4]}_MO{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_MOI{en}{na}', f'S{nn}_R3C5:6{ea}'],[(f'S{nn}_MOI{en}{na}',1),(f'{B[3]}_MO{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_MOI{en}{na}', f'S{nn}_R2C5:6{ea}'],[(f'S{nn}_MOI{en}{na}',1),(f'{B[2]}_MO{en}{ea}',1)], symbcol(B[0]), prefix))
	for w_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'S{nn}_MOI{en}{na}', f'{w_in}_r3{nn}{ea}'],[(f'S{nn}_MOI{en}{na}',1),(f'{w_in}_MO{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_MOI{en}{na}', f'{GS_PRE}_r3{nn}a{ea}'],[(f'S{nn}_MOI{en}a{na}',2),(f'{GS_PRE}_MO{en}{ea}',1)], symbcol(B[0]), prefix))


	ttypes.append(createType([f'S{nn}_MOI{en}a{na}'],[(f'{B[6]}_MO{nn}{na}',1),(f'{B[1]}_MO{en}{ea}',1),(f'S{nn}_MOI{nn}{wa}',1)], symbcol(B[0]), prefix))


	for s_in in ['0','1',GS_PRE,GTE_SEP1,GTE_SEP2,GT_PRE,BLANK]:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'S{nn}_MOI{nn}{wa}'],[(f'{s_in}_MO{nn}{na}',1),(f'S{nn}_MOI{nn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R1:2C5{na}', f'S{nn}_MOI{nn}{wa}'],[(f'{B[5]}_MO{nn}{na}',1),(f'S{nn}_MOI{nn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R1:2C4{na}', f'S{nn}_MOI{nn}{wa}'],[(f'{B[4]}_MO{nn}{na}',1),(f'S{nn}_MOI{nn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R1:2C3{na}', f'S{nn}_MOI{nn}{wa}'],[(f'{B[3]}_MO{nn}{na}',1),(f'S{nn}_MOI{nn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R1:2C2{na}', f'S{nn}_MOI{nn}{wa}'],[(f'{B[2]}_MO{nn}{na}',1),(f'S{nn}_MOI{nn}{wa}',1)], symbcol(B[0]), prefix))
	for s_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'{s_in}_r3{wn}{na}', f'S{nn}_MOI{nn}{wa}'],[(f'{s_in}_MO{nn}{na}',1),(f'S{nn}_MOI{nn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GS_PRE}_r3{wn}a{na}', f'S{nn}_MOI{nn}{wa}'],[(f'{GS_PRE}_MO{nn}{na}',1),(f'S{nn}_MOI{nn}{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_MOI{wn}{na}', f'S{nn}_MOI{nn}{wa}'],[(f'{B[1]}_MO{nn}{na}',1),(f'{B[6]}_MO{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{B[1]}_MO{nn}{na}', f'sq{nn}{wn}{ea}'],[(f'{B[1]}_MO{nn}{na}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	######

	ttypes.append(createType([f'GS2W_MO{en}{sa}', f'S{nn}_R6C6:7{ea}'],[(f'{BLANK}_p{sn}{sa}',1),(f'S{nn}_R6C6:7{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{sn}{sa}', f'S{nn}_R6C6:7{ea}'],[(f'{BLANK}_p{sn}{sa}',1),(f'S{nn}_R6C6:7{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'BB_p{sn}{sa}', f'S{nn}_R6C6:7{ea}'],[(f'BB_p{sn}{sa}',1),(f'{B[6]}_MO{en}*{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}{na}', f'S{nn}_R7C6:7{ea}'],[(f'S{nn}_R7C6:7{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{BLANK2}{na}', f'S{nn}_R7C6:7{ea}'],[], symbcol(B[0]), prefix))

	######

	ttypes.append(createType([f'BB_p{sn}{sa}', f'{BLANK}_p{en}{ea}'],[(f'BB_p{sn}{sa}',1),(f'{BLANK}_MO{en}*{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{sn}{sa}', f'{BLANK}_pz{en}{ea}'],[(f'{BLANK}_pz{en}{ea}',1)], symbcol(BLANK), prefix))


	return ttypes

def coop_start(d='^'):
	na = d
	ea = arrow_cw[na]
	wa = arrow_ccw[na]
	sa = arrow_opp[na]

	nn = arrow_name[na]
	en = arrow_name[ea]
	wn = arrow_name[wa]
	sn = arrow_name[sa]

	ttypes = []

	prefix = ''

	ttypes.append(createType([f'{B[0]}{na}', f'{B[7]}{ea}'],[(f'C{nn}_R6:7C0{na}',1), (f'C{nn}_R7C0:1{ea}',1)], symbcol(B[0]), prefix))

	prop_proxy = BLANK

	ttypes.append(createType([f'C{nn}_R6:7C0{na}', f'{BLANK}{ea}'],[(f'C{nn}_R6:7C0{na}',1), (f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R6:7C0{na}', f'{BLANK2}{ea}'],[(f'C{nn}_R6:7C0{na}',1), (f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}{na}', f'C{nn}_R7C0:1{ea}'],[(f'{prop_proxy}_p{nn}{na}',1), (f'C{nn}_R7C0:1{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK2}{na}', f'C{nn}_R7C0:1{ea}'],[(f'{BLANK}_p{nn}{na}',1), (f'C{nn}_R7C0:1{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{B[1]}{na}', f'C{nn}_R7C0:1{ea}'],[(f'C{nn}_R6:7C1{na}',1), (f'C{nn}_R7C1:2{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R6:7C1{na}', f'{prop_proxy}_p{en}{ea}'],[(f'C{nn}_R6:7C1{na}',1), (f'{prop_proxy}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	for s_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'{s_in}{na}', f'C{nn}_R7C1:2{ea}'],[(f'{s_in}_p{nn}{na}',1), (f'C{nn}_R7C1:2{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{B[2]}{na}', f'C{nn}_R7C1:2{ea}'],[(f'C{nn}_R6:7C2{na}',1), (f'C{nn}_R7C2:3{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R6:7C2{na}', f'{prop_proxy}_p{en}{ea}'],[(f'C{nn}_R6:7C2{na}',1), (f'{prop_proxy}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}{na}', f'C{nn}_R7C2:3{ea}'],[(f'{prop_proxy}_p{nn}{na}',1), (f'C{nn}_R7C2:3{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{B[3]}{na}', f'C{nn}_R7C2:3{ea}'],[(f'C{nn}_R6:7C3{na}',1), (f'C{nn}_R7C3:4{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R6:7C3{na}', f'{prop_proxy}_p{en}{ea}'],[(f'C{nn}_R6:7C3{na}',1), (f'{prop_proxy}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	for s_in in ['0', '1', GS_PRE, GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'{s_in}{na}', f'C{nn}_R7C3:4{ea}'],[(f'{s_in}_p{nn}{na}',1), (f'C{nn}_R7C3:4{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_R6:7C0{na}', f'{B[6]}{ea}'],[(f'C{nn}_R5:6C0{na}',1), (f'C{nn}_R6C0:1{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{prop_proxy}_p{nn}{na}', f'C{nn}_R6C0:1{ea}'],[(f'{prop_proxy}_p{nn}{na}',1), (f'C{nn}_R6C0:1{ea}',1)], symbcol(B[0]), prefix))

	for w_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'C{nn}_R5:6C0{na}', f'{w_in}{ea}'],[(f'C{nn}_R5:6C0{na}',1), (f'{w_in}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_R6:7C1{na}', f'C{nn}_R6C0:1{ea}'],[(f'C{nn}_R5:6C1{na}',1), (f'C{nn}_R6C1:2{ea}',1)], symbcol(B[0]), prefix))

	# tiles for combining glue signatures

	for w_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'C{nn}_R5:6C1{na}', f'{w_in}_p{en}{ea}'],[(f'C{nn}_R5:6C1{na}',1), (f'C{nn}_{w_in}_GC{ea}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'{w_in}_p{nn}{na}', f'C{nn}_R6C1:2{ea}'],[(f'C{nn}_{w_in}_GC{na}',1), (f'C{nn}_R6C1:2{ea}',1)], symbcol(B[0]), prefix))

	for w_in, s_in in product(['0', '1', GS_PRE], repeat=2):
		ttypes.append(createType([f'C{nn}_{s_in}_GC{na}', f'C{nn}_{w_in}_GC{ea}'],[(f'C{nn}_GC{na}',1), (f'C{nn}_{w_in}+{s_in}_GC{ea}',1)], symbcol(symbxor(w_in, s_in)), prefix))

	for w_in1, w_in2, s_in in product(['0', '1', GS_PRE], repeat=3):
		ttypes.append(createType([f'C{nn}_{s_in}_GC{na}', f'C{nn}_{w_in1}+{w_in2}_GC{ea}'],[(f'C{nn}_{w_in2}_GC{na}',1), (f'C{nn}_{w_in1}+{s_in}_GC{ea}',1)], symbcol(symbxor(w_in2, s_in)), prefix))

	for w_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'C{nn}_GC{na}', f'C{nn}_{w_in}_GC{ea}'],[(f'C{nn}_GC{na}',1), (f'C{nn}_{w_in}_GC{ea}',1)], symbcol(w_in), prefix))

	ttypes.append(createType([f'C{nn}_R6:7C2{na}', f'C{nn}_R6C1:2{ea}'],[(f'C{nn}_R5:6C2{na}',1), (f'C{nn}_R6C2:3{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{prop_proxy}_p{nn}{na}', f'C{nn}_R6C2:3{ea}'],[(f'{prop_proxy}_p{nn}{na}',1), (f'C{nn}_R6C2:3{ea}',1)], symbcol(B[0]), prefix))

	for w_in1, w_in2 in product(['0', '1', GS_PRE], repeat=2):
		ttypes.append(createType([f'C{nn}_R5:6C2{na}', f'C{nn}_{w_in1}+{w_in2}_GC{ea}'],[(f'C{nn}_R5:6C2{na}',1), (f'{symbxor(w_in1, w_in2)}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_R6:7C3{na}', f'C{nn}_R6C2:3{ea}'],[(f'C{nn}_R5:6C3{na}',1), (f'C{nn}_R6C3:4{ea}',1)], symbcol(B[0]), prefix))
	for w_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'C{nn}_R5:6C3{na}', f'{w_in}_p{en}{ea}'],[(f'C{nn}_R5:6C3{na}',1), (f'{w_in}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	for s_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'C{nn}_R6C3:4{ea}'],[(f'{s_in}_GT{nn}{na}',1), (f'C{nn}_R6C3:4{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GT_PRE}_p{nn}{na}', f'C{nn}_R6C3:4{ea}'],[(f'{GT_PRE}_GT{nn}a{na}',1), (f'C{nn}_R6C3:4{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GTE_SEP1}_p{nn}{na}', f'C{nn}_R6C3:4{ea}'],[(f'{GTE_SEP1}_GT{nn}a{na}',1), (f'C{nn}_R6C3:4{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GTE_SEP2}_p{nn}{na}', f'C{nn}_R6C3:4{ea}'],[(f'C{nn}_R6C3:4{ea}',1)], symbcol(B[0]), prefix))

	# glue table query functionality

	for w_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'{GT_PRE}_GT{nn}a{na}', f'{w_in}_p{en}{ea}'],[(f'{GT_PRE}_GT{nn}{na}',1), (f'{w_in}_GT{nn}a{ea}',1)], symbcol(w_in), prefix))
		ttypes.append(createType([f'{GT_PRE}_GT{nn}{na}', f'{w_in}_p{en}{ea}'],[(f'{GT_PRE}_GT{nn}{na}',1), (f'{w_in}_GT{nn}{ea}',1)], symbcol(w_in), prefix))

		for s_in in ['0', '1', GS_PRE]:
			ttypes.append(createType([f'{s_in}_GT{nn}{na}', f'{w_in}_GT{nn}a{ea}'],[(f'{s_in}_GT{nn}{na}',1), (f'{w_in}+{s_in}_GT{nn}b{ea}',1)], symbcol(symbxor(w_in, s_in)), prefix))
			ttypes.append(createType([f'{s_in}_GT{nn}{na}', f'{w_in}_GT{nn}{ea}'],[(f'{s_in}_GT{nn}{na}',1), (f'{w_in}_GT{nn}{ea}',1)], symbcol(symbxor(w_in, s_in)), prefix))

	for w_in1, w_in2, s_in in product(['0', '1', GS_PRE], repeat=3):
		ttypes.append(createType([f'{s_in}_GT{nn}{na}', f'{w_in1}+{w_in2}_GT{nn}b{ea}'],[(f'{s_in}+{w_in2}_GT{nn}b{na}',1), (f'{w_in1}+{s_in}_GT{nn}{ea}',1)], symbcol(symbxor(w_in2, s_in)), prefix))
		ttypes.append(createType([f'{s_in}_GT{nn}{na}', f'{w_in1}+{w_in2}_GT{nn}{ea}'],[(f'{s_in}+{w_in2}_GT{nn}{na}',1), (f'{w_in1}+{s_in}_GT{nn}{ea}',1)], symbcol(symbxor(w_in2, s_in)), prefix))

	for w_in, s_in1, s_in2 in product(['0', '1', GS_PRE], repeat=3):
		ttypes.append(createType([f'{s_in1}+{s_in2}_GT{nn}b{na}', f'{w_in}_GT{nn}{ea}'],[(f'{s_in1}_GT{nn}{na}',1), (f'{w_in}+{s_in2}_GT{nn}b{ea}',1)], symbcol(symbxor(w_in, s_in2)), prefix))


	for w_in1, w_in2, s_in1, s_in2 in product(['0', '1', GS_PRE], repeat=4):
		ttypes.append(createType([f'{s_in1}+{s_in2}_GT{nn}{na}', f'{w_in1}+{w_in2}_GT{nn}b{ea}'],[(f'{s_in1}+{w_in2}_GT{nn}b{na}',1), (f'{w_in1}+{s_in2}_GT{nn}{ea}',1)], symbcol(symbxor(w_in2, s_in2)), prefix))
		ttypes.append(createType([f'{s_in1}+{s_in2}_GT{nn}{na}', f'{w_in1}+{w_in2}_GT{nn}{ea}'],[(f'{s_in1}+{w_in2}_GT{nn}{na}',1), (f'{w_in1}+{s_in2}_GT{nn}{ea}',1)], symbcol(symbxor(w_in2, s_in2)), prefix))

	for w_in1, w_in2 in product(['0', '1'], repeat=2):
		if w_in1 == w_in2:
			ttypes.append(createType([f'{GTE_SEP1}_GT{nn}{na}', f'{w_in1}+{w_in2}_GT{nn}{ea}'],[(f'{GTE_SEP1}_GT{nn}{na}',1), (f'T+{w_in1}_GT{nn}{ea}',1)], GREEN, prefix))
			ttypes.append(createType([f'{GTE_SEP1}_GT{nn}a{na}', f'{w_in1}+{w_in2}_GT{nn}{ea}'],[(f'{GTE_SEP1}_GT{nn}{na}',1), (f'T+{w_in1}_GT{nn}a{ea}',1)], GREEN, prefix))
		else:
			ttypes.append(createType([f'{GTE_SEP1}_GT{nn}{na}', f'{w_in1}+{w_in2}_GT{nn}{ea}'],[(f'{GTE_SEP1}_GT{nn}{na}',1), (f'F+{w_in1}_GT{nn}{ea}',1)], RED, prefix))
			ttypes.append(createType([f'{GTE_SEP1}_GT{nn}a{na}', f'{w_in1}+{w_in2}_GT{nn}{ea}'],[(f'{GTE_SEP1}_GT{nn}{na}',1), (f'F+{w_in1}_GT{nn}a{ea}',1)], RED, prefix))

	ttypes.append(createType([f'{GTE_SEP1}_GT{nn}{na}', f'{GS_PRE}+{GS_PRE}_GT{nn}{ea}'],[(f'{GTE_SEP1}_GT{nn}{na}',1), (f'T+{GS_PRE}_GT{nn}{ea}',1)], GREEN, prefix))
	ttypes.append(createType([f'{GTE_SEP1}_GT{nn}{na}', f'{GS_PRE}+{GS_PRE}_GT{nn}b{ea}'],[(f'{GTE_SEP1}_GT{nn}{na}',1), (f'match_GT{nn}a{ea}',2)], GREEN, prefix))
	ttypes.append(createType([f'match_GT{nn}a{ea}'],[(f'{GTE_SEP2}_GT{nn}{na}',1), (f'{GS_PRE}_p{en}{ea}',1), (f'T_GT{nn}{sa}',1)], GREEN, prefix))

	for w_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'T_GT{nn}{sa}', f'T+{w_in}_GT{nn}{ea}'],[(f'T_GT{nn}{sa}',1), (f'{w_in}_p{en}{ea}',1)], GREEN, prefix))
		ttypes.append(createType([f'T_GT{nn}{sa}', f'F+{w_in}_GT{nn}{ea}'],[(f'F_GT{nn}{sa}',1), (f'{w_in}_p{en}{ea}',1)], RED, prefix))
		ttypes.append(createType([f'F_GT{nn}{sa}', f'F+{w_in}_GT{nn}{ea}'],[(f'F_GT{nn}{sa}',1), (f'{w_in}_p{en}{ea}',1)], RED, prefix))
		ttypes.append(createType([f'F_GT{nn}{sa}', f'T+{w_in}_GT{nn}{ea}'],[(f'F_GT{nn}{sa}',1), (f'{w_in}_p{en}{ea}',1)], RED, prefix))

		ttypes.append(createType([f'T_GT{nn}{sa}', f'T+{w_in}_GT{nn}a{ea}'],[(f'M+{w_in}_GT{nn}{ea}',1)], GREEN, prefix))
		ttypes.append(createType([f'T_GT{nn}{sa}', f'F+{w_in}_GT{nn}a{ea}'],[(f'NM+{w_in}_GT{nn}{ea}',1)], RED, prefix))
		ttypes.append(createType([f'F_GT{nn}{sa}', f'F+{w_in}_GT{nn}a{ea}'],[(f'NM+{w_in}_GT{nn}{ea}',1)], RED, prefix))
		ttypes.append(createType([f'F_GT{nn}{sa}', f'T+{w_in}_GT{nn}a{ea}'],[(f'NM+{w_in}_GT{nn}{ea}',1)], RED, prefix))

	for w_in, s_in in product(['0', '1'], ['0','1',GS_PRE]):
		ttypes.append(createType([f'{s_in}_GT{nn}{na}', f'NM+{w_in}_GT{nn}{ea}'],[(f'{s_in}_p{nn}{na}',1), (f'NM+{w_in}_GT{nn}{ea}',1)], symbcol(symbxor(s_in,w_in)), prefix))

	for w_in in ['0','1']:
		ttypes.append(createType([f'{GT_PRE}_GT{nn}a{na}', f'NM+{w_in}_GT{nn}{ea}'],[(f'{GT_PRE}_GT{nn}{na}',1), (f'{w_in}_GT{nn}a{ea}',1)], symbcol(w_in), prefix))

	for w_in, s_in in product(['0','1'], ['0','1',GS_PRE]):
		ttypes.append(createType([f'{s_in}_GT{nn}{na}', f'M+{w_in}_GT{nn}{ea}'],[(f'{s_in}_p{nn}{na}',1), (f'M+{w_in}+{s_in}_GTF{nn}{ea}',1)], symbcol(symbxor(w_in,s_in)), prefix))
	
	for w_in1, w_in2, s_in in product(['0','1'], ['0','1',GS_PRE], ['0','1',GS_PRE]):
		ttypes.append(createType([f'{s_in}_GT{nn}{na}', f'M+{w_in1}+{w_in2}_GTF{nn}{ea}'],[(f'{s_in}+{w_in2}_GTF{nn}{na}',1), (f'M+{w_in1}+{s_in}_GTF{nn}{ea}',1)], symbcol(symbxor(w_in1,s_in)), prefix))

	for w_in, s_in1, s_in2 in product(['0','1',GS_PRE], repeat=3):
		ttypes.append(createType([f'{s_in1}+{s_in2}_GTF{nn}{na}', f'{w_in}_p{en}{ea}'],[(f'{s_in1}_p{nn}{na}',1), (f'{w_in}+{s_in2}_GTF{nn}{ea}',1)], symbcol(symbxor(w_in,s_in2)), prefix))

	for w_in1, w_in2, s_in1, s_in2 in product(['0','1',GS_PRE], repeat=4):
		ttypes.append(createType([f'{s_in1}+{s_in2}_GTF{nn}{na}', f'{w_in1}+{w_in2}_GTF{nn}{ea}'],[(f'{s_in1}+{w_in2}_GTF{nn}{na}',1), (f'{w_in1}+{s_in2}_GTF{nn}{ea}',1)], symbcol(symbxor(w_in1,s_in2)), prefix))

	for w_in1, w_in2 in product(['0', '1'], ['0','1']):
	 	ttypes.append(createType([f'{GT_PRE}_GT{nn}a{na}', f'M+{w_in1}+{w_in2}_GTF{nn}{ea}'],[(f'{GT_PRE}_GT{nn}b{na}',1), (f'P+{w_in2}_GTF{nn}{ea}',1)], symbcol(GT_PRE), prefix))
	 	ttypes.append(createType([f'{GT_PRE}_GT{nn}b{na}', f'{w_in1}+{w_in2}_GTF{nn}{ea}'],[(f'{GT_PRE}_GT{nn}b{na}',1), (f'{w_in2}_p{en}{ea}',1)], symbcol(GT_PRE), prefix))

	ttypes.append(createType([f'{GT_PRE}_GT{nn}b{na}', f'{GS_PRE}+{GS_PRE}_GTF{nn}{ea}'],[(f'{GT_PRE}_GT{nn}b{na}',1), (f'{GS_PRE}_p{en}{ea}',1)], symbcol(GT_PRE), prefix))

	for w_in in ['0','1']:
		for s_in in ['0','1',GS_PRE]:
			ttypes.append(createType([f'{s_in}_GT{nn}{na}', f'P+{w_in}_GTF{nn}{ea}'],[(f'{s_in}_p{nn}{na}',1), (f'P+{w_in}_GTF{nn}{ea}',1)], symbcol(GT_PRE), prefix))
		
		ttypes.append(createType([f'{GT_PRE}_GT{nn}a{na}', f'P+{w_in}_GTF{nn}{ea}'],[(f'{GT_PRE}_p{nn}{na}',1), (f'P+{w_in}_GTF{nn}{ea}',1)], symbcol(GT_PRE), prefix))
		ttypes.append(createType([f'{GTE_SEP1}_GT{nn}a{na}', f'P+{w_in}_GTF{nn}{ea}'],[(f'{GTE_SEP1}_p{nn}{na}',1), (f'P+{w_in}_GTF{nn}!{ea}',2)], symbcol(GTE_SEP1), prefix))
		ttypes.append(createType([f'P+{w_in}_GTF{nn}!{ea}'],[(f'{GTE_SEP2}_p{nn}{na}',1), (f'P+{w_in}_GTF{nn}{ea}',1)], symbcol(GTE_SEP2), prefix))
	
	# end glue table query functionality

	ttypes.append(createType([f'C{nn}_R5:6C0{na}', f'{B[5]}{ea}'],[(f'C{nn}_R4:5C0{na}',1), (f'C{nn}_R5C0:1{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'C{nn}_R5C0:1{ea}'],[(f'{BLANK}_p{nn}{na}',1), (f'C{nn}_R5C0:1{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R4:5C0{na}', f'{BLANK}{ea}'],[(f'C{nn}_R4:5C0{na}',1), (f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_R5:6C1{na}', f'C{nn}_R5C0:1{ea}'],[(f'C{nn}_R4:5C1{na}',1), (f'C{nn}_R5C1:2{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_GC{na}', f'C{nn}_R5C1:2{ea}'],[(f'{BLANK}_p{nn}{na}',1), (f'C{nn}_R5C1:2{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R4:5C1{na}', f'{BLANK}_p{en}{ea}'],[(f'C{nn}_R4:5C1{na}',1), (f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_R5:6C2{na}', f'C{nn}_R5C1:2{ea}'],[(f'C{nn}_R4:5C2{na}',1), (f'C{nn}_R5C2:3{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'C{nn}_R5C2:3{ea}'],[(f'{BLANK}_p{nn}{na}',1), (f'C{nn}_R5C2:3{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R4:5C2{na}', f'{BLANK}_p{en}{ea}'],[(f'C{nn}_R4:5C2{na}',1), (f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_R5:6C3{na}', f'C{nn}_R5C2:3{ea}'],[(f'C{nn}_R4:5C3{na}',1), (f'C{nn}_R5C3:4{ea}',1)], symbcol(B[0]), prefix))
	for s_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'{s_in}_GT{nn}{na}', f'C{nn}_R5C3:4{ea}'],[(f'{s_in}_p{nn}{na}',1), (f'C{nn}_R5C3:4{ea}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'C{nn}_R5C3:4{ea}'],[(f'{s_in}_p{nn}{na}',1), (f'C{nn}_R5C3:4{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GT_PRE}_GT{nn}b{na}', f'C{nn}_R5C3:4{ea}'],[(f'{s_in}_p{nn}{na}',1), (f'C{nn}_R5C3:4{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_R4:5C3{na}', f'{BLANK}_p{en}{ea}'],[(f'C{nn}_R4:5C3{na}',1), (f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))


	ttypes.append(createType([f'C{nn}_R4:5C0{na}', f'{B[4]}{ea}'],[(f'C{nn}_R3:4C0{na}',1), (f'C{nn}_R4C0:1{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'C{nn}_R4C0:1{ea}'],[(f'{BLANK}_p{nn}{na}',1), (f'C{nn}_R4C0:1{ea}',1)], symbcol(B[0]), prefix))
	for w_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'C{nn}_R3:4C0{na}', f'{w_in}{ea}'],[(f'C{nn}_R3:4C0{na}',1), (f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_R4:5C1{na}', f'C{nn}_R4C0:1{ea}'],[(f'C{nn}_R3:4C1{na}',1), (f'C{nn}_R4C1:2{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'C{nn}_R4C1:2{ea}'],[(f'{BLANK}_p{nn}{na}',1), (f'C{nn}_R4C1:2{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R3:4C1{na}', f'{BLANK}_p{en}{ea}'],[(f'C{nn}_R3:4C1{na}',1), (f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_R4:5C2{na}', f'C{nn}_R4C1:2{ea}'],[(f'C{nn}_R3:4C2{na}',1), (f'C{nn}_R4C2:3a{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'C{nn}_R4C2:3a{ea}'],[(f'C{nn}_z{nn}a{na}',1), (f'C{nn}_R4C2:3b{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'C{nn}_R4C2:3b{ea}'],[(f'C{nn}_z{nn}b{na}',1), (f'C{nn}_R4C2:3{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'C{nn}_R4C2:3{ea}'],[(f'{BLANK}_p{nn}{na}',1), (f'C{nn}_R4C2:3{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R3:4C2{na}', f'{BLANK}_p{en}{ea}'],[(f'C{nn}_R3:4C2{na}',1), (f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_z{nn}a{na}', f'{BLANK}_p{en}{ea}'],[(f'C{nn}_z{nn}a{na}',1)], symbcol(BLANK), prefix))

	###

	ttypes.append(createType([f'C{nn}_R4:5C3{na}', f'C{nn}_R4C2:3{ea}'],[(f'C{nn}_R3:4C3{na}',1),(f'C{nn}_R4C3:4a{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GT_PRE}_p{nn}{na}', f'C{nn}_R4C3:4a{ea}'],[(f'{GT_PRE}_r3{nn}a{na}',2),(f'C{nn}_R4C3:4{ea}',1)], symbcol(B[0]), prefix))
	for s_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'C{nn}_R4C3:4{ea}'],[(f'{s_in}_r3{nn}{na}',1),(f'C{nn}_R4C3:4{ea}',1)], symbcol(B[0]), prefix))

	# end glue table tile types

	for e_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'C{nn}_R3:4C3{na}', f'{e_in}_p{wn}{wa}'],[(f'C{nn}_R3:4C3{na}',1),(f'{BLANK}_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_z{nn}b{na}', f'{BLANK}_p{wn}{wa}'],[(f'C{nn}_z{nn}b{na}',1)], symbcol(BLANK), prefix))

	ttypes.append(createType([f'{B[4]}{na}', f'C{nn}_R7C3:4{ea}'],[(f'C{nn}_R6:7C4{na}',1),(f'C{nn}_R7C4:5{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}{na}', f'C{nn}_R7C4:5{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'C{nn}_R7C4:5{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R6:7C4{na}', f'{BLANK}_p{en}{ea}'],[(f'C{nn}_R6:7C4{na}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{B[5]}{na}', f'C{nn}_R7C4:5{ea}'],[(f'C{nn}_R6:7C5{na}',1),(f'C{nn}_R7C5:6{ea}',1)], symbcol(B[0]), prefix))
	for s_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'{s_in}{na}', f'C{nn}_R7C5:6{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'C{nn}_R7C5:6{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R6:7C5{na}', f'{BLANK}_p{en}{ea}'],[(f'C{nn}_R6:7C5{na}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{B[6]}{na}', f'C{nn}_R7C5:6{ea}'],[(f'C{nn}_R6:7C6a{na}',1),(f'C{nn}_R7C6:7{ea}',1)], symbcol(B[0]), prefix))
	# ttypes.append(createType([f'{BLANK}{na}', f'C{nn}_R7C6:7{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'C{nn}_R7C6:7{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R6:7C6a{na}', f'{BLANK}_p{en}{ea}'],[(f'C{nn}_R6:7C6{na}',1),(f'{BLANK}_pz{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R6:7C6{na}', f'{BLANK}_p{en}{ea}'],[(f'C{nn}_R6:7C6{na}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	# ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'{BLANK}_pz{en}{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'{BLANK}_pz{en}{ea}',1)], symbcol(BLANK), prefix))


	ttypes.append(createType([f'C{nn}_R6:7C4{na}', f'C{nn}_R6C3:4{ea}'],[(f'C{nn}_R5:6C4{na}',1),(f'C{nn}_R6C4:5{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'C{nn}_R6C4:5{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'C{nn}_R6C4:5{ea}',1)], symbcol(B[0]), prefix))
	for w_in in ['0', '1']:
		ttypes.append(createType([f'C{nn}_R5:6C4{na}', f'P+{w_in}_GTF{nn}{ea}'],[(f'C{nn}_R5:6C4{na}',1),(f'{w_in}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	for w_in1,w_in2 in product(['0', '1'],['0', '1']):
		ttypes.append(createType([f'C{nn}_R5:6C4{na}', f'M+{w_in1}+{w_in2}_GTF{nn}{ea}'],[(f'C{nn}_R5:6C4{na}',1),(f'{w_in2}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	for w_in1,w_in2 in product(['0', '1'],['0', '1']):
		ttypes.append(createType([f'C{nn}_R5:6C4{na}', f'{w_in1}+{w_in2}_GTF{nn}{ea}'],[(f'C{nn}_R5:6C4{na}',1),(f'{w_in2}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R5:6C4{na}', f'{GS_PRE}+{GS_PRE}_GTF{nn}{ea}'],[(f'C{nn}_R5:6C4{na}',1),(f'{GS_PRE}_p{en}{ea}',1)], symbcol(B[0]), prefix))


	for w_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'C{nn}_R5:6C4{na}', f'{w_in}_p{en}{ea}'],[(f'C{nn}_R5:6C4{na}',1),(f'{w_in}_p{en}{ea}',1)], symbcol(B[0]), prefix))


	ttypes.append(createType([f'C{nn}_R6:7C5{na}', f'C{nn}_R6C4:5{ea}'],[(f'C{nn}_R5:6C5{na}',1),(f'C{nn}_R6C5:6{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'C{nn}_R6C5:6{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'C{nn}_R6C5:6{ea}',1)], symbcol(B[0]), prefix))
	for w_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'C{nn}_R5:6C5{na}', f'{w_in}_p{en}{ea}'],[(f'C{nn}_R5:6C5{na}',1),(f'{w_in}_r{en}{nn}{ea}',1)], symbcol(B[0]), prefix))


	ttypes.append(createType([f'C{nn}_R5:6C4{na}', f'C{nn}_R5C3:4{ea}'],[(f'C{nn}_R4:5C4{na}',1),(f'C{nn}_R5C4:5{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'C{nn}_R5C4:5{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'C{nn}_R5C4:5{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R4:5C4{na}', f'{BLANK}_p{en}{ea}'],[(f'C{nn}_R4:5C4{na}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_R5:6C5{na}', f'C{nn}_R5C4:5{ea}'],[(f'C{nn}_R4:5C5{na}',1),(f'C{nn}_R5C5:6{ea}',1)], symbcol(B[0]), prefix))
	for s_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'{s_in}_r{en}{nn}{na}', f'C{nn}_R5C5:6{ea}'],[(f'{s_in}_p{nn}{na}',1),(f'C{nn}_R5C5:6{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R4:5C5{na}', f'{BLANK}_p{en}{ea}'],[(f'C{nn}_R4:5C5{na}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_R4:5C4{na}', f'C{nn}_R4C3:4{ea}'],[(f'C{nn}_R3:4C4{na}',1),(f'C{nn}_R4C4:5{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'C{nn}_R4C4:5{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'C{nn}_R4C4:5{ea}',1)], symbcol(B[0]), prefix))
	for w_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'C{nn}_R3:4C4{na}', f'{w_in}_r3{nn}{ea}'],[(f'C{nn}_R3:4C4{na}',1),(f'{w_in}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R3:4C4{na}', f'{GT_PRE}_r3{nn}a{ea}'],[(f'C{nn}_R3:4C4a{na}',2),(f'{GT_PRE}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	
	ttypes.append(createType([f'C{nn}_R4:5C5{na}', f'C{nn}_R4C4:5{ea}'],[(f'C{nn}_R3:4C5{na}',1),(f'C{nn}_R4C5:6{ea}',1)], symbcol(B[0]), prefix))
	for s_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'C{nn}_R4C5:6{ea}'],[(f'{s_in}_p{nn}{na}',1),(f'C{nn}_R4C5:6{ea}',1)], symbcol(B[0]), prefix))
	for w_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'C{nn}_R3:4C5{na}', f'{w_in}_p{en}{ea}'],[(f'C{nn}_R3:4C5{na}',1),(f'{w_in}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_R3:4C4a{na}'],[(f'C{nn}_R2:3C4{na}',1),(f'C{nn}_R3C4:5{ea}',1), (f'C{nn}_R3C3:4{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'C{nn}_R3C4:5{ea}'],[(f'sq{nn}{wn}{na}',1),(f'C{nn}_R3C4:5{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R3:4C5{na}', f'C{nn}_R3C4:5{ea}'],[(f'C{nn}_R2:3C5a{na}',2),(f'C{nn}_R3C5:6{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R2:3C5a{na}'],[(f'C{nn}_R2:3C5{na}',1),(f'{BLANK}_p{en}{ea}',1), (f'sq{nn}{wn}x{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R2:3C5{na}', f'sq{nn}{wn}{ea}'],[(f'C{nn}_R2:3C5{na}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	for s_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'C{nn}_R3C5:6{ea}'],[(f'{s_in}_p{nn}{na}',1),(f'C{nn}_R3C5:6{ea}',1)], symbcol(B[0]), prefix))
	
	for s_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'C{nn}_R3C3:4{wa}'],[(f'{s_in}_p{nn}{na}',1),(f'C{nn}_R3C3:4{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_R2:3C4{na}', f'sq{nn}{wn}{wa}'],[(f'C{nn}_R2:3C4{na}',1),(f'{BLANK}_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R2:3C4{na}', f'sq{nn}{wn}b{wa}'],[(f'C{nn}_R2:3C4a{na}',2),(f'{BLANK}_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R2:3C4a{na}'],[(f'C{nn}_R1:2C4{na}',1),(f'C{nn}_R2C4:5{ea}',1),(f'C{nn}_R2C3:4{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'sq{nn}{wn}{na}', f'C{nn}_R2C4:5{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'C{nn}_R2C4:5{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R2:3C5{na}', f'C{nn}_R2C4:5{ea}'],[(f'C{nn}_R1:2C5{na}',1),(f'C{nn}_R2C5:6a{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{GS_PRE}_p{nn}{na}', f'C{nn}_R2C5:6a{ea}'],[(f'{GS_PRE}_r3{nn}a{na}',2),(f'C{nn}_R2C5:6{ea}',1)], symbcol(B[0]), prefix))
	for s_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'C{nn}_R2C5:6{ea}'],[(f'{s_in}_r3{nn}{na}',1),(f'C{nn}_R2C5:6{ea}',1)], symbcol(B[0]), prefix))

	for e_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'C{nn}_R1:2C5{na}', f'{e_in}_p{wn}{wa}'],[(f'C{nn}_R1:2C5{na}',1),(f'{e_in}_p{wn}{wa}',1)], symbcol(B[0]), prefix))

	for e_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'C{nn}_R1:2C4{na}', f'{e_in}_p{wn}{wa}'],[(f'C{nn}_R1:2C4{na}',1),(f'{e_in}_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	for s_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'C{nn}_R2C3:4{wa}'],[(f'{s_in}_p{nn}{na}',1),(f'C{nn}_R2C3:4{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_R3:4C3{na}', f'C{nn}_R3C3:4{wa}'],[(f'C{nn}_R2:3C3{na}',1),(f'C{nn}_R3C2:3{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R2:3C3{na}', f'{BLANK}_p{wn}{wa}'],[(f'C{nn}_R2:3C3{na}',1),(f'{BLANK}_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'C{nn}_R3C2:3{wa}'],[(f'{BLANK}_p{nn}{na}',1),(f'C{nn}_R3C2:3{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_R3:4C0{na}', f'{B[3]}{ea}'],[(f'C{nn}_R2:3C0{na}',1),(f'C{nn}_R3C0:1a{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R2:3C0{na}', f'{BLANK}{ea}'],[(f'C{nn}_R2:3C0{na}',1),(f'{BLANK}_zp{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'C{nn}_R3C0:1a{ea}'],[(f'z_{nn}{na}',1),(f'C{nn}_R3C0:1{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'C{nn}_R3C0:1{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'C{nn}_R3C0:1{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'z_{nn}{na}', f'{BLANK}_zp{en}{ea}'],[(f'z_{nn}{na}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(BLANK), prefix))

	ttypes.append(createType([f'C{nn}_R3:4C1{na}', f'C{nn}_R3C0:1{ea}'],[(f'C{nn}_R2:3C1{na}',1),(f'C{nn}_R3C1:2{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R2:3C1{na}', f'{BLANK}_p{en}{ea}'],[(f'C{nn}_R2:3C1{na}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'C{nn}_R3C1:2{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'C{nn}_R3C1:2{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_R3:4C2{na}', f'C{nn}_R3C1:2{ea}'],[(f'C{nn}_R2:3C2{na}',1),(f'C{nn}_R3C2:3{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R2:3C2{na}', f'{BLANK}_p{en}{ea}'],[(f'C{nn}_R2:3C2{na}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_z{nn}a{na}', f'C{nn}_R3C2:3{ea}'],[(f'C{nn}_z{nn}a{na}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_z{nn}b{na}', f'C{nn}_R3C2:3{wa}'],[(f'C{nn}_z{nn}b{na}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_R2:3C3{na}', f'C{nn}_R2C3:4{wa}'],[(f'C{nn}_R1:2C3{na}',1),(f'C{nn}_R2C2:3{wa}',1)], symbcol(B[0]), prefix))
	for e_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'C{nn}_R1:2C3{na}', f'{e_in}_p{wn}{wa}'],[(f'C{nn}_R1:2C3{na}',1),(f'{e_in}_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'C{nn}_R2C2:3{wa}'],[(f'{BLANK}_p{nn}{na}',1),(f'C{nn}_R2C2:3{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_z{nn}a{na}', f'C{nn}_R2C2:3{wa}'],[(f'{BLANK}_p{nn}{na}',1),(f'C{nn}_R2C2:3{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_z{nn}b{na}', f'C{nn}_R2C2:3{wa}'],[(f'{BLANK}_p{nn}{na}',1),(f'C{nn}_R2C2:3{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_R2:3C2{na}', f'C{nn}_R2C2:3{wa}'],[(f'C{nn}_R1:2C2{na}',1),(f'C{nn}_R2C1:2{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'C{nn}_R2C1:2{wa}'],[(f'{BLANK}_p{nn}{na}',1),(f'C{nn}_R2C1:2{wa}',1)], symbcol(B[0]), prefix))
	for e_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'C{nn}_R1:2C2{na}', f'{e_in}_p{wn}{wa}'],[(f'C{nn}_R1:2C2{na}',1),(f'{e_in}_fr{nn}{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'{GS_PRE}_fr{nn}{wa}'],[(f'{GS_PRE}_fr{nn}{na}',1),(f'{BLANK}_p{wn}{wa}',1)], symbcol(GS_PRE), prefix))

	for e_in, s_in in product(['0','1',GS_PRE], repeat=2):
		ttypes.append(createType([f'{s_in}_fr{nn}{na}', f'{e_in}_fr{nn}{wa}'],[(f'{e_in}_fr{nn}{na}',1),(f'{s_in}_fr{nn}{wa}',1)], symbcol(e_in), prefix))

	ttypes.append(createType([f'C{nn}_R2:3C1{na}', f'C{nn}_R2C1:2{wa}'],[(f'C{nn}_R1:2C1{na}',1),(f'C{nn}_R2C0:1{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R1:2C1{na}', f'{BLANK}_p{wn}{wa}'],[(f'C{nn}_R1:2C1{na}',1),(f'{BLANK}_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'C{nn}_R2C0:1{wa}'],[(f'{BLANK}_p{nn}{na}',1),(f'C{nn}_R2C0:1{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'z_{nn}{na}', f'C{nn}_R2C0:1{wa}'],[(f'z_{nn}{na}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'z_{nn}{na}', f'{BLANK}_p{wn}{wa}'],[(f'z_{nn}{na}',1)], symbcol(BLANK), prefix))

	ttypes.append(createType([f'C{nn}_R2:3C0{na}', f'{B[2]}{ea}'],[(f'C{nn}_R1:2C0{na}',1)], symbcol(B[0]), prefix))
	for w_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'C{nn}_R1:2C0{na}', f'{w_in}{ea}'],[(f'C{nn}_R1:2C0{na}',1)], symbcol(B[0]), prefix))

	###### Initiate macrotile outputs

	# ttypes.append(createType([f'C{nn}_R5:6C1{na}', f'{GS_PRE}_dr{nn}a{wa}'],[(f'S{nn}_MOI{wn}{na}',1),(f'{GS_PRE}_MO{wn}{wa}',1)], symbcol(B[0]), prefix))
	# for e_in in ['0','1',GS_PRE]:
	# 	ttypes.append(createType([f'S{nn}_MOI{wn}{na}', f'{e_in}_dr{nn}{wa}'],[(f'S{nn}_MOI{wn}{na}',1),(f'{e_in}_MO{wn}{wa}',1)], symbcol(B[0]), prefix))
	# ttypes.append(createType([f'S{nn}_MOI{wn}{na}', f'S{nn}_R5C1:2{wa}'],[(f'S{nn}_MOI{wn}{na}',1),(f'{B[2]}_MO{wn}{wa}',1)], symbcol(B[0]), prefix))
	# ttypes.append(createType([f'S{nn}_MOI{wn}{na}', f'{BLANK}_p{wn}{wa}'],[(f'S{nn}_MOI{wn}{na}',1),(f'{BLANK}_MO{wn}{wa}',1)], symbcol(B[0]), prefix))
	# ttypes.append(createType([f'S{nn}_MOI{wn}{na}', f'S{nn}_R4C1:2{wa}'],[(f'S{nn}_MOI{wn}{na}',1),(f'{B[3]}_MO{wn}{wa}',1)], symbcol(B[0]), prefix))
	# for e_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
	# 	ttypes.append(createType([f'S{nn}_MOI{wn}{na}', f'{e_in}_p{wn}{wa}'],[(f'S{nn}_MOI{wn}{na}',1),(f'{e_in}_MO{wn}{wa}',1)], symbcol(B[0]), prefix))
	# ttypes.append(createType([f'S{nn}_MOI{wn}{na}', f'S{nn}_R3C1:2{wa}'],[(f'S{nn}_MOI{wn}{na}',1),(f'{B[4]}_MO{wn}{wa}',1)], symbcol(B[0]), prefix))
	# ttypes.append(createType([f'S{nn}_MOI{wn}{na}', f'S{nn}_R2C1:2{wa}'],[(f'S{nn}_MOI{wn}{na}',1),(f'{B[5]}_MO{wn}{wa}',1)], symbcol(B[0]), prefix))



	ttypes.append(createType([f'C{nn}_R6:7C6{na}', f'C{nn}_R6C5:6{ea}'],[(f'C{nn}_MOI{en}{na}',1),(f'C{nn}_R6C6:7{ea}',1)], symbcol(B[0]), prefix))
	for w_in in ['0','1',GS_PRE,GTE_SEP1,GTE_SEP2,GT_PRE,BLANK]:
		ttypes.append(createType([f'C{nn}_MOI{en}{na}', f'{w_in}_p{en}{ea}'],[(f'C{nn}_MOI{en}{na}',1),(f'{w_in}_MO{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_MOI{en}{na}', f'C{nn}_R5C5:6{ea}'],[(f'C{nn}_MOI{en}{na}',1),(f'{B[5]}_MO{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_MOI{en}{na}', f'C{nn}_R4C5:6{ea}'],[(f'C{nn}_MOI{en}{na}',1),(f'{B[4]}_MO{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_MOI{en}{na}', f'C{nn}_R3C5:6{ea}'],[(f'C{nn}_MOI{en}{na}',1),(f'{B[3]}_MO{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_MOI{en}{na}', f'C{nn}_R2C5:6{ea}'],[(f'C{nn}_MOI{en}{na}',1),(f'{B[2]}_MO{en}{ea}',1)], symbcol(B[0]), prefix))
	for w_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'C{nn}_MOI{en}{na}', f'{w_in}_r3{nn}{ea}'],[(f'C{nn}_MOI{en}{na}',1),(f'{w_in}_MO{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_MOI{en}{na}', f'{GS_PRE}_r3{nn}a{ea}'],[(f'C{nn}_MOI{en}a{na}',2),(f'{GS_PRE}_MO{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_MOI{en}a{na}'],[(f'{B[6]}_MO{nn}{na}',1),(f'{B[1]}_MO{en}{ea}',1),(f'C{nn}_MOI{nn}{wa}',1)], symbcol(B[0]), prefix))

	for s_in in ['0','1',GS_PRE,GTE_SEP1,GTE_SEP2,GT_PRE,BLANK]:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'C{nn}_MOI{nn}{wa}'],[(f'{s_in}_MO{nn}{na}',1),(f'C{nn}_MOI{nn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R1:2C5{na}', f'C{nn}_MOI{nn}{wa}'],[(f'{B[5]}_MO{nn}{na}',1),(f'C{nn}_MOI{nn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R1:2C4{na}', f'C{nn}_MOI{nn}{wa}'],[(f'{B[4]}_MO{nn}{na}',1),(f'C{nn}_MOI{nn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R1:2C3{na}', f'C{nn}_MOI{nn}{wa}'],[(f'{B[3]}_MO{nn}{na}',1),(f'C{nn}_MOI{nn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R1:2C2{na}', f'C{nn}_MOI{nn}{wa}'],[(f'{B[2]}_MO{nn}{na}',1),(f'C{nn}_MOI{nn}{wa}',1)], symbcol(B[0]), prefix))
	for s_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'{s_in}_fr{nn}{na}', f'C{nn}_MOI{nn}{wa}'],[(f'{s_in}_MO{nn}{na}',1),(f'C{nn}_MOI{nn}{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_R1:2C1{na}', f'C{nn}_MOI{nn}{wa}'],[(f'{B[1]}_MO{nn}{na}',1),(f'C{nn}_R1C0:1{wa}',1)], symbcol(B[0]), prefix))
	
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'C{nn}_R1C0:1{wa}'],[(f'{BLANK}_p{nn}{na}',1),(f'C{nn}_R1C0:1{wa}',1)], symbcol(B[0]), prefix))
	
	ttypes.append(createType([f'z_{nn}{na}', f'C{nn}_R1C0:1{wa}'],[(f'z_{nn}{na}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'z_{nn}{na}', f'{BLANK}_p{en}{ea}'],[(f'z_{nn}{na}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(BLANK), prefix))

	ttypes.append(createType([f'C{nn}_R1:2C0{na}', f'{B[1]}{ea}'],[(f'C{nn}_R1:2C0{na}',1)], symbcol(B[0]), prefix))
	
	ttypes.append(createType([f'C{nn}_R1:2C0{na}', f'{BLANK}{ea}'],[(f'C{nn}_R1:2C0{na}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	
	ttypes.append(createType([f'{B[1]}_MO{nn}{na}', f'{BLANK}_p{en}{ea}'],[(f'{B[1]}_MO{nn}{na}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))


	ttypes.append(createType([f'GS2W_MO{en}{sa}', f'C{nn}_R6C6:7{ea}'],[(f'{BLANK}_p{sn}{sa}',1),(f'C{nn}_R6C6:7{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{sn}{sa}', f'C{nn}_R6C6:7{ea}'],[(f'{BLANK}_p{sn}{sa}',1),(f'C{nn}_R6C6:7{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'BB_p{sn}{sa}', f'C{nn}_R6C6:7{ea}'],[(f'BB_p{sn}{sa}',1),(f'{B[6]}_MO{en}*{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}{na}', f'C{nn}_R7C6:7{ea}'],[(f'C{nn}_R7C6:7{ea}',1)], symbcol(B[0]), prefix))	


	ttypes.append(createType([f'C{nn}_R1:2C0{na}', f'{BLANK2}{ea}'],[(f'BB_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'z_{nn}{na}', f'BB_p{en}{ea}'],[(f'{BLANK2}_MO{nn}**{na}',1),(f'BB_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{BLANK2}{na}', f'C{nn}_R7C6:7{ea}'],[], symbcol(B[0]), prefix))

	# ttypes.append(createType([f'C{nn}_R1:2C0{na}', f'{B[1]}{ea}'],[(f'C{nn}_R0:1C0{na}',1),(f'C{nn}_R1C0:1{ea}',1)], symbcol(B[0]), prefix))
	# ttypes.append(createType([f'z_{nn}{na}', f'C{nn}_R1C0:1{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'C{nn}_R1C0:1{ea}',1)], symbcol(B[0]), prefix))
	# ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'C{nn}_R1C0:1{ea}'],[(f'{BLANK}_p{nn}{na}',1),(f'C{nn}_R1C0:1{ea}',1)], symbcol(B[0]), prefix))
	# ttypes.append(createType([f'C{nn}_R0:1C0{na}', f'{BLANK}{ea}'],[(f'C{nn}_R0:1C0{na}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))


	return ttypes

def gen_iu_ttypes():
	ttypes = []

	ttypes.extend(str2_start('^') + str2_start('>') + str2_start('v') + str2_start('<'))
	ttypes.extend(coop_start('^') + coop_start('>') + coop_start('v') + coop_start('<'))
	ttypes.extend(propagate('^') + propagate('>') + propagate('v') + propagate('<'))

	#ttypes.extend(macrotile_output('^') + macrotile_output('>') + macrotile_output('v') + macrotile_output('<'))

	ttypes += macrotile_output_generic('^')
	ttypes += macrotile_output_generic('>')
	ttypes += macrotile_output_generic('v')
	ttypes += macrotile_output_generic('<')

	ttypes += macrotile_output_W()
	ttypes += macrotile_output_N()
	ttypes += macrotile_output_E()
	ttypes += macrotile_output_S()

	print(f'generated {len(ttypes)} unique tile types')

	return ttypes

def create_sierpinski_dummy_seed():
	gX = list('000')
	gS = list('100')
	gV = list('101')
	gH = list('110')
	g0 = list('010')
	g1 = list('011')

	tS_GTE = ((gS, gX, gX, gX), (gV, gH, gX, gX))
	tV_GTE = ((gV, gX, gX, gX), (gV, g1, gX, gX))
	tH_GTE = ((gX, gH, gX, gX), (g1, gH, gX, gX))
	
	t00_GTE = ((g0, g0, gX, gX), (g0, g0, gX, gX))
	t01_GTE = ((g0, g1, gX, gX), (g1, g1, gX, gX))
	t10_GTE = ((g1, g0, gX, gX), (g1, g1, gX, gX))
	t11_GTE = ((g1, g1, gX, gX), (g0, g0, gX, gX))

	glue_sig = (gS, gX, gX, gX)

	glue_table = []
	glue_table += [GT_PRE] + [GS_PRE] + tS_GTE[0][0] + [GS_PRE] + tS_GTE[0][1] + [GS_PRE] + tS_GTE[0][2] + [GS_PRE] + tS_GTE[0][3]
	glue_table += [GTE_SEP1, GTE_SEP2] + [GS_PRE] + tS_GTE[1][0] + [GS_PRE] + tS_GTE[1][1] + [GS_PRE] + tS_GTE[1][2] + [GS_PRE] + tS_GTE[1][3]
	
	glue_table += [GT_PRE] + [GS_PRE] + tV_GTE[0][0] + [GS_PRE] + tV_GTE[0][1] + [GS_PRE] + tV_GTE[0][2] + [GS_PRE] + tV_GTE[0][3]
	glue_table += [GTE_SEP1, GTE_SEP2] + [GS_PRE] + tV_GTE[1][0] + [GS_PRE] + tV_GTE[1][1] + [GS_PRE] + tV_GTE[1][2] + [GS_PRE] + tV_GTE[1][3]

	glue_table += [GT_PRE] + [GS_PRE] + tH_GTE[0][0] + [GS_PRE] + tH_GTE[0][1] + [GS_PRE] + tH_GTE[0][2] + [GS_PRE] + tH_GTE[0][3]
	glue_table += [GTE_SEP1, GTE_SEP2] + [GS_PRE] + tH_GTE[1][0] + [GS_PRE] + tH_GTE[1][1] + [GS_PRE] + tH_GTE[1][2] + [GS_PRE] + tH_GTE[1][3]

	glue_table += [GT_PRE] + [GS_PRE] + t00_GTE[0][0] + [GS_PRE] + t00_GTE[0][1] + [GS_PRE] + t00_GTE[0][2] + [GS_PRE] + t00_GTE[0][3]
	glue_table += [GTE_SEP1, GTE_SEP2] + [GS_PRE] + t00_GTE[1][0] + [GS_PRE] + t00_GTE[1][1] + [GS_PRE] + t00_GTE[1][2] + [GS_PRE] + t00_GTE[1][3]

	glue_table += [GT_PRE] + [GS_PRE] + t01_GTE[0][0] + [GS_PRE] + t01_GTE[0][1] + [GS_PRE] + t01_GTE[0][2] + [GS_PRE] + t01_GTE[0][3]
	glue_table += [GTE_SEP1, GTE_SEP2] + [GS_PRE] + t01_GTE[1][0] + [GS_PRE] + t01_GTE[1][1] + [GS_PRE] + t01_GTE[1][2] + [GS_PRE] + t01_GTE[1][3]

	glue_table += [GT_PRE] + [GS_PRE] + t10_GTE[0][0] + [GS_PRE] + t10_GTE[0][1] + [GS_PRE] + t10_GTE[0][2] + [GS_PRE] + t10_GTE[0][3]
	glue_table += [GTE_SEP1, GTE_SEP2] + [GS_PRE] + t10_GTE[1][0] + [GS_PRE] + t10_GTE[1][1] + [GS_PRE] + t10_GTE[1][2] + [GS_PRE] + t10_GTE[1][3]
	
	glue_table += [GT_PRE] + [GS_PRE] + t11_GTE[0][0] + [GS_PRE] + t11_GTE[0][1] + [GS_PRE] + t11_GTE[0][2] + [GS_PRE] + t11_GTE[0][3]
	glue_table += [GTE_SEP1, GTE_SEP2] + [GS_PRE] + t11_GTE[1][0] + [GS_PRE] + t11_GTE[1][1] + [GS_PRE] + t11_GTE[1][2] + [GS_PRE] + t11_GTE[1][3]	
	
	BLANK_SIZE_OUTER = 24
	BLANK_SIZE_INNER = 8

	seed_row = []
	seed_row += [B[0]] + ([BLANK2] + [BLANK] * BLANK_SIZE_OUTER) 

	s2_idx = len(seed_row) + 2

	seed_row += [B[1]] + [GS_PRE] + glue_sig[0] + [GS_PRE] + glue_sig[1] + [GS_PRE] + glue_sig[2] + [GS_PRE] + glue_sig[3]
	seed_row += [B[2]] + ([BLANK] * BLANK_SIZE_INNER)
	seed_row += [B[3]] + glue_table
	seed_row += [B[4]] + ([BLANK] * BLANK_SIZE_INNER)
	seed_row += [B[5]] + [GS_PRE] + glue_sig[0] + [GS_PRE] + glue_sig[1] + [GS_PRE] + glue_sig[2] + [GS_PRE] + glue_sig[3]
	seed_row += [B[6]] + ([BLANK] * BLANK_SIZE_OUTER + [BLANK2])
	seed_row += [B[7]] 
	
	sys = PyTASTileSystem()

	for i, symb in enumerate(seed_row):
		if i == s2_idx:
			name = f'SEED_{symb}_S2'
			ttype = TileType(name)
			ttype.sides[N] = Glue(2, f'{symb}{S2}^')
			ttype.color = symbcol(symb)
			sys.tileTypes[name] = ttype
			sys.seedPatch.append((ttype, (i, -1)))
		else:
			name = f'SEED_{symb}'

			if name not in sys.tileTypes:
				ttype = TileType(name)
				ttype.sides[N] = Glue(1, f'{symb}^')
				ttype.color = symbcol(symb)
				sys.tileTypes[name] = ttype
			ttype = sys.tileTypes[name]
			sys.seedPatch.append((ttype, (i, -1)))


	glue_sig2H = (g1, gX, gX, gX)
	glue_sig2V = (gX, g1, gX, gX)

	seed_row2H = []
	seed_row2H += [B[0]] + ([BLANK2] + [BLANK] * BLANK_SIZE_OUTER) 
	seed_row2H += [B[1]] + [GS_PRE] + glue_sig2H[0] + [GS_PRE] + glue_sig2H[1] + [GS_PRE] + glue_sig2H[2] + [GS_PRE] + glue_sig2H[3]
	seed_row2H += [B[2]] + ([BLANK] * BLANK_SIZE_INNER)
	seed_row2H += [B[3]] + glue_table
	seed_row2H += [B[4]] + ([BLANK] * BLANK_SIZE_INNER)
	seed_row2H += [B[5]] + [GS_PRE] + glue_sig2H[0] + [GS_PRE] + glue_sig2H[1] + [GS_PRE] + glue_sig2H[2] + [GS_PRE] + glue_sig2H[3]
	seed_row2H += [B[6]] + ([BLANK] * BLANK_SIZE_OUTER + [BLANK2])
	seed_row2H += [B[7]]

	seed_row2V = []
	seed_row2V += [B[0]] + ([BLANK2] + [BLANK] * BLANK_SIZE_OUTER) 
	seed_row2V += [B[1]] + [GS_PRE] + glue_sig2V[0] + [GS_PRE] + glue_sig2V[1] + [GS_PRE] + glue_sig2V[2] + [GS_PRE] + glue_sig2V[3]
	seed_row2V += [B[2]] + ([BLANK] * BLANK_SIZE_INNER)
	seed_row2V += [B[3]] + glue_table
	seed_row2V += [B[4]] + ([BLANK] * BLANK_SIZE_INNER)
	seed_row2V += [B[5]] + [GS_PRE] + glue_sig2V[0] + [GS_PRE] + glue_sig2V[1] + [GS_PRE] + glue_sig2V[2] + [GS_PRE] + glue_sig2V[3]
	seed_row2V += [B[6]] + ([BLANK] * BLANK_SIZE_OUTER + [BLANK2])
	seed_row2V += [B[7]]

	xoffs = -len(seed_row) - 20

	for i, symb in enumerate(seed_row2H):
		name = f'SEED_{symb}'
		ttype = sys.tileTypes[name]
		sys.seedPatch.append((ttype, (i + xoffs, -1)))

	for i, symb in enumerate(seed_row2V):
		name = f'SEEDV_{symb}'

		if name not in sys.tileTypes:
				ttype = TileType(name)
				ttype.sides[E] = Glue(1, f'{symb}>')
				ttype.color = symbcol(symb)
				sys.tileTypes[name] = ttype
		ttype = sys.tileTypes[name]
		sys.seedPatch.append((ttype, (xoffs-1, len(seed_row2V) - i - 1)))

	return sys


if __name__ == "__main__":
	sys = create_sierpinski_dummy_seed()

	ttypes = gen_iu_ttypes()

	for ttype in ttypes:
		name = ttype.name

		if name in sys.tileTypes:
			print(f'duplicate tile type name: {name}')
			exit()

		sys.tileTypes[name] = ttype

	sys.SaveToFile('IU-test')


# ttypes.append(createType([f'S{nn}_R5:6C1{na}', f'{GS_PRE}_dr{nn}a{wa}'],[(f'S{nn}_R5:6C1_t{nn}{na}',1),(f'{GS_PRE}_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	# for e_in in ['0','1']:
	# 	ttypes.append(createType([f'S{nn}_R5:6C1_t{nn}{na}', f'{e_in}_dr{nn}{wa}'],[(f'S{nn}_R5:6C1_t{nn}{na}',1),(f'0_p{wn}{wa}',1)], symbcol(B[0]), prefix))

	# ttypes.append(createType([f'S{nn}_R5:6C1_t{nn}{na}', f'{GS_PRE}_dr{nn}{wa}'],[(f'S{nn}_R5:6C1_t{en}{na}',1),(f'{GS_PRE}_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	# for e_in in ['0','1']:
	# 	ttypes.append(createType([f'S{nn}_R5:6C1_t{en}{na}', f'{e_in}_dr{nn}{wa}'],[(f'S{nn}_R5:6C1_t{en}{na}',1),(f'0_p{wn}{wa}',1)], symbcol(B[0]), prefix))

	# ttypes.append(createType([f'S{nn}_R5:6C1_t{en}{na}', f'{GS_PRE}_dr{nn}{wa}'],[(f'S{nn}_R5:6C1_t{sn}{na}',1),(f'{GS_PRE}_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	# for e_in in ['0','1']:
	# 	ttypes.append(createType([f'S{nn}_R5:6C1_t{sn}{na}', f'{e_in}_dr{nn}{wa}'],[(f'S{nn}_R5:6C1_t{sn}{na}',1),(f'0_p{wn}{wa}',1)], symbcol(B[0]), prefix))

	'''

	####
	# z means glue id is zero
	# n means glue id is non-zero

	ttypes.append(createType([f'S{nn}_R5:6C1_t{sn}{na}', f'{GS_PRE}_dr{nn}{wa}'],[(f'S{nn}_R5:6C1_t{wn}_msb{na}',1),(f'{GS_PRE}_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R5:6C1_t{wn}_msb{na}', f'0_dr{nn}{wa}'],[(f'S{nn}_R5:6C1_t{wn}_z{na}',1),(f'0_msb_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R5:6C1_t{wn}_msb{na}', f'1_dr{nn}{wa}'],[(f'S{nn}_R5:6C1_t{wn}_n{na}',1),(f'1_msb_p{wn}{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R5:6C1_t{wn}_z{na}', f'0_dr{nn}{wa}'],[(f'S{nn}_R5:6C1_t{wn}_z{na}',1),(f'0_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R5:6C1_t{wn}_z{na}', f'1_dr{nn}{wa}'],[(f'S{nn}_R5:6C1_t{wn}_n{na}',1),(f'1_p{wn}{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R5:6C1_t{wn}_n{na}', f'0_dr{nn}{wa}'],[(f'S{nn}_R5:6C1_t{wn}_n{na}',1),(f'0_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R5:6C1_t{wn}_n{na}', f'1_dr{nn}{wa}'],[(f'S{nn}_R5:6C1_t{wn}_n{na}',1),(f'1_p{wn}{wa}',1)], symbcol(B[0]), prefix))

	# ttypes.append(createType([f'S{nn}_R5:6C1_t{wn}_s{na}', f'0_dr{nn}{wa}'],[(f'S{nn}_R5:6C1_t{wn}_s{na}',1),(f'0_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	# ttypes.append(createType([f'S{nn}_R5:6C1_t{wn}_s{na}', f'1_dr{nn}{wa}'],[(f'S{nn}_R5:6C1_t{wn}_s{na}',1),(f'1_p{wn}{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'0_msb_p{wn}{wa}'],[(f'{BLANK}_p{nn}{na}',1),(f'0_msb_p{wn}{wa}',1)], symbcol('0'), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'1_msb_p{wn}{wa}'],[(f'{BLANK}_p{nn}{na}',1),(f'1_msb_p{wn}{wa}',1)], symbcol('1'), prefix))

	ttypes.append(createType([f'0_p{nn}{na}', f'0_msb_p{wn}{wa}'],[(f'0_p{nn}{na}',1),(f'0_msb_p{wn}{wa}',1)], symbcol('0'), prefix))
	ttypes.append(createType([f'0_p{nn}{na}', f'1_msb_p{wn}{wa}'],[(f'0_p{nn}{na}',1),(f'1_msb_p{wn}{wa}',1)], symbcol('1'), prefix))

	for g_type in ['z','n']:
		ttypes.append(createType([f'S{nn}_R5:6C1_t{wn}_{g_type}{na}', f'S{nn}_R5C1:2{wa}'],[(f'S{nn}_R4:5C1{na}',1),(f'S{nn}_R5C0:1_{g_type}{wa}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'S{nn}_R5C0:1_{g_type}{wa}'],[(f'{BLANK}_p{nn}{na}',1),(f'S{nn}_R5C0:1_{g_type}{wa}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'0_p{nn}{na}', f'S{nn}_R5C0:1_{g_type}{wa}'],[(f'{BLANK}_p{nn}{na}',1),(f'S{nn}_R5C0:1_{g_type}a{wa}',2)], symbcol(B[0]), prefix))
	
	ttypes.append(createType([f'S{nn}_R5C0:1_za{wa}'],[(f'S{nn}_R4:5C0_z{na}',1),(f'S{nn}_R5:6C0_z{sa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R5C0:1_na{wa}'],[(f'S{nn}_R4:5C0_n{na}',1),(f'S{nn}_R5:6C0_n{sa}',1), (f'{B[2]}{wa}', 1)], symbcol(B[0]), prefix))

	for e_in in ['0', '1', GS_PRE]:
		ttypes.append(createType([f'S{nn}_R5:6C0_z{sa}', f'{e_in}_p{wn}{wa}'],[(f'S{nn}_R5:6C0_z{sa}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'S{nn}_R5:6C0_n{sa}', f'{e_in}_p{wn}{wa}'],[(f'S{nn}_R5:6C0_n{sa}',1), (f'{e_in}{wa}', 1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R5:6C0_z{sa}', f'0_msb_p{wn}{wa}'],[(f'S{nn}_R5:6C0_z{sa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R5:6C0_n{sa}', f'0_msb_p{wn}{wa}'],[(f'S{nn}_R5:6C0_n{sa}',1), (f'0{wa}', 1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R5:6C0_n{sa}', f'1_msb_p{wn}{wa}'],[(f'S{nn}_R5:6C0_n{sa}',1), (f'1!{wa}', 2)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R4:5C1{na}', f'{BLANK}_p{wn}{wa}'],[(f'S{nn}_R4:5C1{na}',1), (f'{BLANK}_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R4:5C1{na}', f'S{nn}_R4C1:2{wa}'],[(f'S{nn}_R3:4C1{na}',1), (f'S{nn}_R4C0:1{wa}',1)], symbcol(B[0]), prefix))
	for e_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'S{nn}_R3:4C1{na}', f'{e_in}_p{wn}{wa}'],[(f'S{nn}_R3:4C1{na}',1), (f'{e_in}_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'S{nn}_R4C0:1{wa}'],[(f'{BLANK}_p{nn}{na}',1), (f'S{nn}_R4C0:1{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R4:5C0_z{na}', f'{BLANK}_p{wn}{wa}'],[(f'S{nn}_R4:5C0_z{na}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R4:5C0_n{na}', f'{BLANK}_p{wn}{wa}'],[(f'S{nn}_R4:5C0_n{na}',1), (f'{BLANK}{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R4:5C0_z{na}', f'S{nn}_R4C0:1{wa}'],[(f'S{nn}_R3:4C0_z{na}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R4:5C0_n{na}', f'S{nn}_R4C0:1{wa}'],[(f'S{nn}_R3:4C0_n{na}',1), (f'{B[3]}{wa}',1)], symbcol(B[0]), prefix))

	for e_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'S{nn}_R3:4C0_z{na}', f'{e_in}_p{wn}{wa}'],[(f'S{nn}_R3:4C0_z{na}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'S{nn}_R3:4C0_n{na}', f'{e_in}_p{wn}{wa}'],[(f'S{nn}_R3:4C0_n{na}',1), (f'{e_in}{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R3:4C1{na}', f'S{nn}_R3C1:2{wa}'],[(f'S{nn}_R2:3C1{na}',1), (f'S{nn}_R3C0:1{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R2:3C1{na}', f'{BLANK}_p{wn}{wa}'],[(f'S{nn}_R2:3C1{na}',1), (f'{BLANK}_p{wn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'S{nn}_R3C0:1{wa}'],[(f'{BLANK}_p{nn}{na}',1), (f'S{nn}_R3C0:1{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R2:3C1{na}', f'S{nn}_R2C1:2{wa}'],[(f'S{nn}_R1:2C1{na}',1), (f'S{nn}_R2C0:1{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'S{nn}_R2C0:1{wa}'],[(f'{BLANK}_p{nn}{na}',1), (f'S{nn}_R2C0:1{wa}',1)], symbcol(B[0]), prefix))
	for e_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'S{nn}_R1:2C1{na}', f'{e_in}_p{wn}{wa}'],[(f'S{nn}_R1:2C1{na}',1), (f'{e_in}_p{wn}{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R3:4C0_z{na}', f'S{nn}_R3C0:1{wa}'],[(f'S{nn}_R2:3C0_z{na}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R3:4C0_n{na}', f'S{nn}_R3C0:1{wa}'],[(f'S{nn}_R2:3C0_n{na}',1), (f'{B[4]}{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R2:3C0_z{na}', f'{BLANK}_p{wn}{wa}'],[(f'S{nn}_R2:3C0_z{na}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R2:3C0_n{na}', f'{BLANK}_p{wn}{wa}'],[(f'S{nn}_R2:3C0_n{na}',1), (f'{BLANK}{wa}',1)], symbcol(B[0]), prefix))

	#######################

	ttypes.append(createType([f'S{nn}_R2:3C0_n{na}', f'S{nn}_R2C0:1{wa}'],[(f'S{nn}_R1:2C0_n{na}',1), (f'{B[5]}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R2:3C0_z{na}', f'S{nn}_R2C0:1{wa}'],[(f'S{nn}_R1:2C0_z{na}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R1:2C0_n{na}', f'{GS_PRE}_p{wn}{wa}'],[(f'S{nn}_R1:2C0_t{nn}_n{na}',1), (f'{GS_PRE}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R1:2C0_z{na}', f'{GS_PRE}_p{wn}{wa}'],[(f'S{nn}_R1:2C0_t{nn}_z{na}',1)], symbcol(B[0]), prefix))

	for e_in in ['0','1']:
		ttypes.append(createType([f'S{nn}_R1:2C0_t{nn}_n{na}', f'{e_in}_p{wn}{wa}'],[(f'S{nn}_R1:2C0_t{nn}_n{na}',1), (f'0{wa}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'S{nn}_R1:2C0_t{nn}_z{na}', f'{e_in}_p{wn}{wa}'],[(f'S{nn}_R1:2C0_t{nn}_z{na}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R1:2C0_t{nn}_n{na}', f'{GS_PRE}_p{wn}{wa}'],[(f'S{nn}_R1:2C0_t{en}_n{na}',1), (f'{GS_PRE}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R1:2C0_t{nn}_z{na}', f'{GS_PRE}_p{wn}{wa}'],[(f'S{nn}_R1:2C0_t{en}_z{na}',1)], symbcol(B[0]), prefix))

	for e_in in ['0','1']:
		ttypes.append(createType([f'S{nn}_R1:2C0_t{en}_n{na}', f'{e_in}_p{wn}{wa}'],[(f'S{nn}_R1:2C0_t{en}_n{na}',1), (f'0{wa}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'S{nn}_R1:2C0_t{en}_z{na}', f'{e_in}_p{wn}{wa}'],[(f'S{nn}_R1:2C0_t{en}_z{na}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R1:2C0_t{en}_n{na}', f'{GS_PRE}_p{wn}{wa}'],[(f'S{nn}_R1:2C0_t{sn}_n{na}',1), (f'{GS_PRE}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R1:2C0_t{en}_z{na}', f'{GS_PRE}_p{wn}{wa}'],[(f'S{nn}_R1:2C0_t{sn}_z{na}',1)], symbcol(B[0]), prefix))

	for e_in in ['0','1']:
		ttypes.append(createType([f'S{nn}_R1:2C0_t{sn}_n{na}', f'{e_in}_p{wn}{wa}'],[(f'S{nn}_R1:2C0_t{sn}_n{na}',1), (f'0{wa}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'S{nn}_R1:2C0_t{sn}_z{na}', f'{e_in}_p{wn}{wa}'],[(f'S{nn}_R1:2C0_t{sn}_z{na}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R1:2C0_t{sn}_n{na}', f'{GS_PRE}_p{wn}{wa}'],[(f'S{nn}_R1:2C0_t{wn}msb_n{na}',1), (f'{GS_PRE}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R1:2C0_t{sn}_z{na}', f'{GS_PRE}_p{wn}{wa}'],[(f'S{nn}_R1:2C0_t{wn}_z{na}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R1:2C0_t{wn}msb_n{na}', f'0_p{wn}{wa}'],[(f'S{nn}_R1:2C0_t{wn}_n{na}',1), (f'0{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R1:2C0_t{wn}msb_n{na}', f'1_p{wn}{wa}'],[(f'S{nn}_R1:2C0_t{wn}_n{na}',1), (f'1{wa}',1)], symbcol(B[0]), prefix)) # STRENGTH 2?

	ttypes.append(createType([f'S{nn}_R1:2C0_t{wn}_n{na}', f'0_p{wn}{wa}'],[(f'S{nn}_R1:2C0_t{wn}_n{na}',1), (f'0{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R1:2C0_t{wn}_z{na}', f'0_p{wn}{wa}'],[(f'S{nn}_R1:2C0_t{wn}_z{na}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R1:2C0_t{wn}_n{na}', f'1_p{wn}{wa}'],[(f'S{nn}_R1:2C0_t{wn}_n{na}',1), (f'1{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R1:2C0_t{wn}_z{na}', f'1_p{wn}{wa}'],[(f'S{nn}_R1:2C0_t{wn}_z{na}',1)], symbcol(B[0]), prefix))

	#######################

	# for T in ['C','S']:
	# 	ttypes.append(createType([f'{T}{nn}_R6:7C6{na}', f'{T}{nn}_R6C5:6{ea}'],[(f'T{nn}_R5:6C6_t{wn}{na}',1), (f'T{nn}_R6C6:7{ea}',1)], symbcol(B[0]), prefix))
	# for w_in in ['0','1']:
	# 	ttypes.append(createType([f'T{nn}_R5:6C6_t{wn}{na}', f'{w_in}_p{en}{ea}'],[(f'T{nn}_R5:6C6_t{wn}{na}',1), (f'0_p{en}{ea}',1)], symbcol(B[0]), prefix))
	# ttypes.append(createType([f'T{nn}_R5:6C6_t{wn}{na}', f'{GS_PRE}_p{en}{ea}'],[(f'T{nn}_R5:6C6_t{sn}{na}',1), (f'{GS_PRE}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	# for w_in in ['0','1']:
	# 	ttypes.append(createType([f'T{nn}_R5:6C6_t{sn}{na}', f'{w_in}_p{en}{ea}'],[(f'T{nn}_R5:6C6_t{sn}{na}',1), (f'0_p{en}{ea}',1)], symbcol(B[0]), prefix))
	# ttypes.append(createType([f'T{nn}_R5:6C6_t{sn}{na}', f'{GS_PRE}_p{en}{ea}'],[(f'T{nn}_R5:6C6_t{en}_z{na}',1), (f'{GS_PRE}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	# ttypes.append(createType([f'T{nn}_R5:6C6_t{en}_z{na}', f'0_p{en}{ea}'],[(f'T{nn}_R5:6C6_t{en}_z{na}',1), (f'0_p{en}{ea}',1)], symbcol(B[0]), prefix))
	# ttypes.append(createType([f'T{nn}_R5:6C6_t{en}_z{na}', f'1_p{en}{ea}'],[(f'T{nn}_R5:6C6_t{en}_n{na}',1), (f'1_p{en}{ea}',1)], symbcol(B[0]), prefix))
	# ttypes.append(createType([f'T{nn}_R5:6C6_t{en}_n{na}', f'0_p{en}{ea}'],[(f'T{nn}_R5:6C6_t{en}_n{na}',1), (f'0_p{en}{ea}',1)], symbcol(B[0]), prefix))
	# ttypes.append(createType([f'T{nn}_R5:6C6_t{en}_n{na}', f'1_p{en}{ea}'],[(f'T{nn}_R5:6C6_t{en}_n{na}',1), (f'1_p{en}{ea}',1)], symbcol(B[0]), prefix))

	# ttypes.append(createType([f'T{nn}_R5:6C6_t{en}_z{na}', f'{GS_PRE}_p{en}{ea}'],[(f'T{nn}_R5:6C6_z{na}',1), (f'{GS_PRE}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	# ttypes.append(createType([f'T{nn}_R5:6C6_t{en}_n{na}', f'{GS_PRE}_p{en}{ea}'],[(f'T{nn}_R5:6C6_n{na}',1), (f'{GS_PRE}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	# for w_in in ['0','1',GS_PRE]:
	# 	ttypes.append(createType([f'T{nn}_R5:6C6_z{na}', f'{w_in}_p{en}{ea}'],[(f'T{nn}_R5:6C6_z{na}',1), (f'{w_in}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	# 	ttypes.append(createType([f'T{nn}_R5:6C6_n{na}', f'{w_in}_p{en}{ea}'],[(f'T{nn}_R5:6C6_n{na}',1), (f'{w_in}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	# ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'T{nn}_R6C6:7{ea}'],[(f'T{nn}_R5:6C6_t{wn}{na}',1), (f'T{nn}_R6C6:7{ea}',1)], symbcol(B[0]), prefix))

	for T in ['C','S']:
	 	ttypes.append(createType([f'{T}{nn}_R6:7C6{na}', f'{T}{nn}_R6C5:6{ea}'],[(f'T{nn}_R5:6C6{na}',1), (f'T{nn}_R6C6:7{ea}',1)], symbcol(B[0]), prefix))
	 	ttypes.append(createType([f'T{nn}_R3:4C6{na}', f'{T}{nn}_R3C5:6{ea}'],[(f'T{nn}_R2:3C6{na}',1), (f'T{nn}_R3C6:7{ea}',1)], symbcol(B[0]), prefix))
	 	ttypes.append(createType([f'T{nn}_R2:3C6{na}', f'{T}{nn}_R2C5:6{ea}'],[(f'T{nn}_R1:2C6_t{wn}{na}',1), (f'T{nn}_R2C6:7{ea}',1)], symbcol(B[0]), prefix))
	for w_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'T{nn}_R5:6C6{na}', f'{w_in}_p{en}{ea}'],[(f'T{nn}_R5:6C6{na}',1), (f'{w_in}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	for T in ['C','S']:
	 	ttypes.append(createType([f'T{nn}_R5:6C6{na}', f'{T}{nn}_R5C5:6{ea}'],[(f'T{nn}_R4:5C6{na}',1), (f'T{nn}_R5C6:7{ea}',1)], symbcol(B[0]), prefix))
	 	ttypes.append(createType([f'T{nn}_R4:5C6{na}', f'{T}{nn}_R4C5:6{ea}'],[(f'T{nn}_R3:4C6{na}',1), (f'T{nn}_R4C6:7{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R4:5C6{na}', f'{BLANK}_p{en}{ea}'],[(f'T{nn}_R4:5C6{na}',1), (f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R2:3C6{na}', f'{BLANK}_p{en}{ea}'],[(f'T{nn}_R2:3C6{na}',1), (f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'T{nn}_R6C6:7{ea}'],[(f'{BLANK}_p{nn}{na}',1), (f'T{nn}_R6C6:7{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'T{nn}_R5C6:7{ea}'],[(f'{BLANK}_p{nn}{na}',1), (f'T{nn}_R5C6:7{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'T{nn}_R4C6:7{ea}'],[(f'{BLANK}_p{nn}{na}',1), (f'T{nn}_R4C6:7{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'T{nn}_R3C6:7{ea}'],[(f'{BLANK}_p{nn}{na}',1), (f'T{nn}_R3C6:7{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'T{nn}_R2C6:7{ea}'],[(f'{BLANK}_p{nn}{na}',1), (f'T{nn}_R2C6:7{ea}',1)], symbcol(B[0]), prefix))

	for w_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'T{nn}_R3:4C6{na}', f'{w_in}_p{en}{ea}'],[(f'T{nn}_R3:4C6{na}',1), (f'{w_in}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	for w_in in ['0','1']:
		ttypes.append(createType([f'T{nn}_R1:2C6_t{wn}{na}', f'{w_in}_r3{nn}{ea}'],[(f'T{nn}_R1:2C6_t{wn}{na}',1), (f'0_p{en}{ea}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'T{nn}_R1:2C6_t{sn}{na}', f'{w_in}_r3{nn}{ea}'],[(f'T{nn}_R1:2C6_t{sn}{na}',1), (f'0_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R1:2C6_t{wn}{na}', f'{GS_PRE}_r3{nn}{ea}'],[(f'T{nn}_R1:2C6_t{sn}{na}',1), (f'{GS_PRE}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R1:2C6_t{sn}{na}', f'{GS_PRE}_r3{nn}{ea}'],[(f'T{nn}_R1:2C6_t{en}_z{na}',1), (f'{GS_PRE}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R1:2C6_t{en}_z{na}', f'0_r3{nn}{ea}'],[(f'T{nn}_R1:2C6_t{en}_z{na}',1), (f'0_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R1:2C6_t{en}_z{na}', f'1_r3{nn}{ea}'],[(f'T{nn}_R1:2C6_t{en}_n{na}',1), (f'1_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R1:2C6_t{en}_n{na}', f'0_r3{nn}{ea}'],[(f'T{nn}_R1:2C6_t{en}_n{na}',1), (f'0_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R1:2C6_t{en}_n{na}', f'1_r3{nn}{ea}'],[(f'T{nn}_R1:2C6_t{en}_n{na}',1), (f'1_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R1:2C6_t{en}_z{na}', f'{GS_PRE}_r3{nn}{ea}'],[(f'T{nn}_R1:2C6_z{na}',1), (f'{GS_PRE}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R1:2C6_t{en}_n{na}', f'{GS_PRE}_r3{nn}{ea}'],[(f'T{nn}_R1:2C6_n{na}',1), (f'{GS_PRE}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	for w_in in ['0','1']:
		ttypes.append(createType([f'T{nn}_R1:2C6_z{na}', f'{w_in}_r3{nn}{ea}'],[(f'T{nn}_R1:2C6_z{na}',1), (f'0_p{en}{ea}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'T{nn}_R1:2C6_n{na}', f'{w_in}_r3{nn}{ea}'],[(f'T{nn}_R1:2C6_n{na}',1), (f'0_p{en}{ea}',1)], symbcol(B[0]), prefix))
	
	ttypes.append(createType([f'T{nn}_R1:2C6_z{na}', f'{GS_PRE}_r3{nn}a{ea}'],[(f'T{nn}_R1:2C6a{na}',2), (f'{GS_PRE}_z{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R1:2C6_n{na}', f'{GS_PRE}_r3{nn}a{ea}'],[(f'T{nn}_R1:2C6a{na}',2), (f'{GS_PRE}_n{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'{GS_PRE}_z{en}{ea}'],[(f'{BLANK}_p{nn}{na}',1), (f'{GS_PRE}_z{en}{ea}',1)], symbcol(GS_PRE), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'{GS_PRE}_n{en}{ea}'],[(f'{BLANK}_p{nn}{na}',1), (f'{GS_PRE}_n{en}{ea}',1)], symbcol(GS_PRE), prefix))

	ttypes.append(createType([f'T{nn}_R1:2C6a{na}'],[(f'T{nn}_R0:1C6{na}',1), (f'T{nn}_R1C6:7{ea}',1), (f'T{nn}_R1C5:6{wa}',1)], symbcol(B[0]), prefix))
	for s_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'T{nn}_R1C5:6{wa}'],[(f'{s_in}_p{nn}{na}',1), (f'T{nn}_R1C5:6{wa}',1)], symbcol(B[0]), prefix))
	for T in ['C','S']:
		ttypes.append(createType([f'{T}{nn}_R1:2C5{na}', f'T{nn}_R1C5:6{wa}'],[(f'T{nn}_R0:1C5{na}',1), (f'T{nn}_R1C4:5{wa}',1)], symbcol(B[0]), prefix))
	
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'T{nn}_R1C4:5{wa}'],[(f'{BLANK}_p{nn}{na}',1), (f'T{nn}_R1C4:5{wa}',1)], symbcol(B[0]), prefix))

	for T in ['C','S']:
		ttypes.append(createType([f'{T}{nn}_R1:2C4{na}', f'T{nn}_R1C4:5{wa}'],[(f'T{nn}_R0:1C4{na}',1), (f'T{nn}_R1C3:4{wa}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'{T}{nn}_R1:2C3{na}', f'T{nn}_R1C3:4{wa}'],[(f'T{nn}_R0:1C3{na}',1), (f'T{nn}_R1C2:3{wa}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'{T}{nn}_R1:2C2{na}', f'T{nn}_R1C2:3{wa}'],[(f'T{nn}_R0:1C2{na}',1), (f'T{nn}_R1C1:2_t{wn}{wa}',1)], symbcol(B[0]), prefix))
	
	for s_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'T{nn}_R1C3:4{wa}'],[(f'{s_in}_p{nn}{na}',1), (f'T{nn}_R1C3:4{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'T{nn}_R1C2:3{wa}'],[(f'{BLANK}_p{nn}{na}',1), (f'T{nn}_R1C2:3{wa}',1)], symbcol(B[0]), prefix))

	for s_in in ['0','1']:
		ttypes.append(createType([f'{s_in}_r3{wn}{na}', f'T{nn}_R1C1:2_t{wn}{wa}'],[(f'0_p{nn}{na}',1), (f'T{nn}_R1C1:2_t{wn}{wa}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'{s_in}_r3{wn}{na}', f'T{nn}_R1C1:2_t{sn}{wa}'],[(f'0_p{nn}{na}',1), (f'T{nn}_R1C1:2_t{sn}{wa}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'{s_in}_r3{wn}{na}', f'T{nn}_R1C1:2_t{en}{wa}'],[(f'0_p{nn}{na}',1), (f'T{nn}_R1C1:2_t{en}{wa}',1)], symbcol(B[0]), prefix))

		ttypes.append(createType([f'{s_in}_fr{nn}{na}', f'T{nn}_R1C1:2_t{wn}{wa}'],[(f'0_p{nn}{na}',1), (f'T{nn}_R1C1:2_t{wn}{wa}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'{s_in}_fr{nn}{na}', f'T{nn}_R1C1:2_t{sn}{wa}'],[(f'0_p{nn}{na}',1), (f'T{nn}_R1C1:2_t{sn}{wa}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'{s_in}_fr{nn}{na}', f'T{nn}_R1C1:2_t{en}{wa}'],[(f'0_p{nn}{na}',1), (f'T{nn}_R1C1:2_t{en}{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{GS_PRE}_r3{wn}{na}', f'T{nn}_R1C1:2_t{wn}{wa}'],[(f'{GS_PRE}_p{nn}{na}',1), (f'T{nn}_R1C1:2_t{sn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GS_PRE}_r3{wn}{na}', f'T{nn}_R1C1:2_t{sn}{wa}'],[(f'{GS_PRE}_p{nn}{na}',1), (f'T{nn}_R1C1:2_t{en}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GS_PRE}_r3{wn}{na}', f'T{nn}_R1C1:2_t{en}{wa}'],[(f'{GS_PRE}_p{nn}{na}',1), (f'T{nn}_R1C1:2_t{nn}_z{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{GS_PRE}_fr{nn}{na}', f'T{nn}_R1C1:2_t{wn}{wa}'],[(f'{GS_PRE}_p{nn}{na}',1), (f'T{nn}_R1C1:2_t{sn}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GS_PRE}_fr{nn}{na}', f'T{nn}_R1C1:2_t{sn}{wa}'],[(f'{GS_PRE}_p{nn}{na}',1), (f'T{nn}_R1C1:2_t{en}{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GS_PRE}_fr{nn}{na}', f'T{nn}_R1C1:2_t{en}{wa}'],[(f'{GS_PRE}_p{nn}{na}',1), (f'T{nn}_R1C1:2_t{nn}_z{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'0_r3{wn}{na}', f'T{nn}_R1C1:2_t{nn}_z{wa}'],[(f'0_p{nn}{na}',1), (f'T{nn}_R1C1:2_t{nn}_z{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'1_r3{wn}{na}', f'T{nn}_R1C1:2_t{nn}_z{wa}'],[(f'1_p{nn}{na}',1), (f'T{nn}_R1C1:2_t{nn}_n{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'0_r3{wn}{na}', f'T{nn}_R1C1:2_t{nn}_n{wa}'],[(f'0_p{nn}{na}',1), (f'T{nn}_R1C1:2_t{nn}_n{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'1_r3{wn}{na}', f'T{nn}_R1C1:2_t{nn}_n{wa}'],[(f'1_p{nn}{na}',1), (f'T{nn}_R1C1:2_t{nn}_n{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'0_fr{nn}{na}', f'T{nn}_R1C1:2_t{nn}_z{wa}'],[(f'0_p{nn}{na}',1), (f'T{nn}_R1C1:2_t{nn}_z{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'1_fr{nn}{na}', f'T{nn}_R1C1:2_t{nn}_z{wa}'],[(f'1_p{nn}{na}',1), (f'T{nn}_R1C1:2_t{nn}_n{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'0_fr{nn}{na}', f'T{nn}_R1C1:2_t{nn}_n{wa}'],[(f'0_p{nn}{na}',1), (f'T{nn}_R1C1:2_t{nn}_n{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'1_fr{nn}{na}', f'T{nn}_R1C1:2_t{nn}_n{wa}'],[(f'1_p{nn}{na}',1), (f'T{nn}_R1C1:2_t{nn}_n{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{GS_PRE}_r3{wn}a{na}', f'T{nn}_R1C1:2_t{nn}_z{wa}'],[(f'{GS_PRE}_z{nn}{na}',1), (f'T{nn}_R1C1:2{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GS_PRE}_r3{wn}a{na}', f'T{nn}_R1C1:2_t{nn}_n{wa}'],[(f'{GS_PRE}_n{nn}{na}',1), (f'T{nn}_R1C1:2{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{GS_PRE}_fr{nn}{na}', f'T{nn}_R1C1:2_t{nn}_z{wa}'],[(f'{GS_PRE}_z{nn}{na}',1), (f'T{nn}_R1C1:2{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GS_PRE}_fr{nn}{na}', f'T{nn}_R1C1:2_t{nn}_n{wa}'],[(f'{GS_PRE}_n{nn}{na}',1), (f'T{nn}_R1C1:2{wa}',1)], symbcol(B[0]), prefix))

	for T in ['C','S']:
		ttypes.append(createType([f'{T}{nn}_R1:2C1{na}', f'T{nn}_R1C1:2{wa}'],[(f'T{nn}_R0:1C1a{na}',2), (f'T{nn}_R1C0:1{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'T{nn}_R1C0:1{wa}'],[(f'sq{nn}{wn}{na}',1), (f'T{nn}_R1C0:1{wa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R0:1C1a{na}'],[(f'T{nn}_R0:1C1{na}',1),(f'sq{nn}{wn}x{wa}',1),(f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R1:2C0_t{wn}_z{na}', f'T{nn}_R1C0:1{wa}'],[(f'S{nn}_R0:1C0_z{na}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R1:2C0_t{wn}_n{na}', f'T{nn}_R1C0:1{wa}'],[(f'S{nn}_R0:1C0_n{na}',1), (f'{B[6]}{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R0:1C0_z{na}', f'sq{nn}{wn}{wa}'],[(f'S{nn}_R0:1C0_z{na}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R0:1C0_n{na}', f'sq{nn}{wn}{wa}'],[(f'S{nn}_R0:1C0_n{na}',1), (f'{BLANK}{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R0:1C0_z{na}', f'sq{nn}{wn}b{wa}'],[(f'S{nn}_R0:1C0_za{na}',2)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R0:1C0_n{na}', f'sq{nn}{wn}b{wa}'],[(f'S{nn}_R0:1C0_na{na}',2), (f'{BLANK}{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'S{nn}_R0:1C0_za{na}'],[(f'{B[0]}{na}',1), (f'T{nn}_R0C0:1{ea}', 1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'S{nn}_R0:1C0_na{na}'],[(f'{B[0]}{na}',1), (f'T{nn}_R0C0:1{ea}', 1), (f'{B[7]}{wa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'C{nn}_R1:2C0{na}', f'{B[1]}{ea}'],[(f'C{nn}_R0:1C0{na}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R0:1C0{na}', f'{BLANK}{ea}'],[(f'C{nn}_R0:1C0{na}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'C{nn}_R0:1C0{na}', f'{B[0]}{ea}'],[(f'{B[0]}{na}',1),(f'T{nn}_R0C0:1{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'z_{nn}{na}', f'T{nn}_R1C0:1{wa}'],[(f'z_{nn}{na}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'z_{nn}{na}', f'sq{nn}{wn}{wa}'],[(f'z_{nn}{na}',1)], MAGENTA, prefix))
	ttypes.append(createType([f'z_{nn}{na}', f'sq{nn}{wn}b{wa}'],[(f'sq{nn}{wn}a{na}',2)], MAGENTA, prefix))
	
	ttypes.append(createType([f'T{nn}_R0:1C1{na}', f'sq{nn}{wn}{ea}'],[(f'T{nn}_R0:1C1{na}',1), (f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))

	for Z in ['n','z']:
		ttypes.append(createType([f'{GS_PRE}_{Z}{nn}{na}', f'{BLANK}_p{en}{ea}'],[(f'{GS_PRE}_{Z}{nn}{na}',1), (f'{BLANK}_p{en}{ea}',1)], symbcol(GS_PRE), prefix))

	ttypes.append(createType([f'T{nn}_R0C0:1{ea}', f'sq{nn}{wn}{na}'],[(f'{BLANK}{na}',1), (f'T{nn}_R0C0:1{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R0C0:1{ea}', f'T{nn}_R0:1C1{na}'],[(f'{B[1]}{na}',1), (f'T{nn}_R0C1:2{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{GS_PRE}_z{nn}{na}', f'T{nn}_R0C1:2{ea}'],[(f'T{nn}_R0C1:2_z{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GS_PRE}_n{nn}{na}', f'T{nn}_R0C1:2{ea}'],[(f'{GS_PRE}{na}',1), (f'T{nn}_R0C1:2_na{ea}',1)], symbcol(B[0]), prefix))

	# ALLOW PROPAGATION ############################################################################################################################################################
	ttypes.append(createType([f'0_p{nn}{na}', f'T{nn}_R0C1:2_na{ea}'],[(f'0{na}',1), (f'T{nn}_R0C1:2_n{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'1_p{nn}{na}', f'T{nn}_R0C1:2_na{ea}'],[(f'1!{na}',2), (f'T{nn}_R0C1:2_n{ea}',1)], symbcol(B[0]), prefix))
	# ttypes.append(createType([f'1_p{nn}{na}', f'T{nn}_R0C1:2_na{ea}'],[(f'1!{na}',1), (f'T{nn}_R0C1:2_n{ea}',1)], symbcol(B[0]), prefix)) # <- THIS ONE
	################################################################################################################################################################################

	ttypes.append(createType([f'0_p{nn}{na}', f'T{nn}_R0C1:2_n{ea}'],[(f'0{na}',1), (f'T{nn}_R0C1:2_n{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'1_p{nn}{na}', f'T{nn}_R0C1:2_n{ea}'],[(f'1{na}',1), (f'T{nn}_R0C1:2_n{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GS_PRE}_p{nn}{na}', f'T{nn}_R0C1:2_n{ea}'],[(f'{GS_PRE}{na}',1), (f'T{nn}_R0C1:2_n{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'0_p{nn}{na}', f'T{nn}_R0C1:2_z{ea}'],[(f'T{nn}_R0C1:2_z{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'1_p{nn}{na}', f'T{nn}_R0C1:2_z{ea}'],[(f'T{nn}_R0C1:2_z{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GS_PRE}_p{nn}{na}', f'T{nn}_R0C1:2_z{ea}'],[(f'T{nn}_R0C1:2_z{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R0:1C2{na}', f'{BLANK}_p{en}{ea}'],[(f'T{nn}_R0:1C2{na}',1), (f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R0:1C3{na}', f'{BLANK}_p{en}{ea}'],[(f'T{nn}_R0:1C3{na}',1), (f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R0:1C4{na}', f'{BLANK}_p{en}{ea}'],[(f'T{nn}_R0:1C4{na}',1), (f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R0:1C5{na}', f'{BLANK}_p{en}{ea}'],[(f'T{nn}_R0:1C5{na}',1), (f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))


	ttypes.append(createType([f'T{nn}_R0:1C2{na}', f'T{nn}_R0C1:2_z{ea}'],[(f'T{nn}_R0C2:3_z{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R0:1C2{na}', f'T{nn}_R0C1:2_n{ea}'],[(f'{B[2]}{na}',1), (f'T{nn}_R0C2:3_n{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'T{nn}_R0C2:3_z{ea}'],[(f'T{nn}_R0C2:3_z{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'T{nn}_R0C2:3_n{ea}'],[(f'{BLANK}{na}',1), (f'T{nn}_R0C2:3_n{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R0:1C3{na}', f'T{nn}_R0C2:3_z{ea}'],[(f'T{nn}_R0C3:4_z{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R0:1C3{na}', f'T{nn}_R0C2:3_n{ea}'],[(f'{B[3]}{na}',1), (f'T{nn}_R0C3:4_n{ea}',1)], symbcol(B[0]), prefix))

	for s_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'T{nn}_R0C3:4_z{ea}'],[(f'T{nn}_R0C3:4_z{ea}',1)], symbcol(B[0]), prefix))
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'T{nn}_R0C3:4_n{ea}'],[(f'{s_in}{na}',1), (f'T{nn}_R0C3:4_n{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R0:1C4{na}', f'T{nn}_R0C3:4_z{ea}'],[(f'T{nn}_R0C4:5_z{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R0:1C4{na}', f'T{nn}_R0C3:4_n{ea}'],[(f'{B[4]}{na}',1), (f'T{nn}_R0C4:5_n{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'T{nn}_R0C4:5_z{ea}'],[(f'T{nn}_R0C4:5_z{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'T{nn}_R0C4:5_n{ea}'],[(f'{BLANK}{na}',1), (f'T{nn}_R0C4:5_n{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R0:1C5{na}', f'T{nn}_R0C4:5_z{ea}'],[(f'T{nn}_R0C5:6_z{ea}',1)], symbcol(B[0]), prefix))
	for s_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'T{nn}_R0C5:6_z{ea}'],[(f'T{nn}_R0C5:6_z{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R0:1C5{na}', f'T{nn}_R0C4:5_n{ea}'],[(f'{B[5]}{na}',1), (f'T{nn}_R0C5:6_n{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GS_PRE}_p{nn}{na}', f'T{nn}_R0C5:6_n{ea}'],[(f'{GS_PRE}{na}',1), (f'T{nn}_R0C5:6_t{nn}_n{ea}',1)], symbcol(B[0]), prefix))
	for s_in in ['0','1']:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'T{nn}_R0C5:6_t{nn}_n{ea}'],[(f'{s_in}{na}',1), (f'T{nn}_R0C5:6_t{nn}_n{ea}',1)], symbcol(B[0]), prefix))
	
	ttypes.append(createType([f'{GS_PRE}_p{nn}{na}', f'T{nn}_R0C5:6_t{nn}_n{ea}'],[(f'{GS_PRE}{na}',1), (f'T{nn}_R0C5:6_t{en}_n{ea}',1)], symbcol(B[0]), prefix))
	for s_in in ['0','1']:
		ttypes.append(createType([f'{s_in}_p{nn}{na}', f'T{nn}_R0C5:6_t{en}_n{ea}'],[(f'0{na}',1), (f'T{nn}_R0C5:6_t{en}_n{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{GS_PRE}_p{nn}{na}', f'T{nn}_R0C5:6_t{en}_n{ea}'],[(f'{GS_PRE}{na}',1), (f'T{nn}_R0C5:6_t{en}_n{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R0:1C6{na}', f'{BLANK}_p{en}{ea}'],[(f'T{nn}_R0:1C6a{na}',1), (f'{BLANK}_p{en}**{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R0:1C6a{na}', f'{BLANK}_p{en}{ea}'],[(f'T{nn}_R0:1C6a{na}',1), (f'{BLANK}_p{en}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'T{nn}_R1C6:7{ea}'],[(f'{BLANK}_p{nn}{na}',1), (f'T{nn}_R1C6:7{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{BLANK}_p{nn}*{na}', f'{BLANK}_p{en}{ea}'],[(f'{BLANK}_p{nn}{na}',1), (f'{BLANK}_p{en}**{ea}',1)], symbcol(BLANK), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'{BLANK}_p{en}**{ea}'],[(f'{BLANK}_p{nn}*{na}',1), (f'{BLANK}_p{en}{ea}',1)], symbcol(BLANK), prefix))


	ttypes.append(createType([f'T{nn}_R0:1C6a{na}', f'T{nn}_R0C5:6_t{en}_n{ea}'],[(f'{B[6]}{na}',1), (f'T{nn}_R0C6:7_n{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R0:1C6a{na}', f'T{nn}_R0C5:6_z{ea}'],[(f'T{nn}_R0C6:7_z{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'T{nn}_R0C6:7_z{ea}'],[                    (f'T{nn}_R0C6:7_z{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}{na}', f'T{nn}_R0C6:7_n{ea}'],[(f'{BLANK}{na}',1), (f'T{nn}_R0C6:7_n{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'{BLANK}_p{nn}*{na}', f'T{nn}_R0C6:7_z{ea}'],[                    (f'T{nn}_R0C6:7_za{ea}',2)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'{BLANK}_p{nn}*{na}', f'T{nn}_R0C6:7_n{ea}'],[(f'{BLANK}{na}',1), (f'T{nn}_R0C6:7_na{ea}',2)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R0C6:7_na{ea}'],[(f'{B[7]}{na}',1),(f'{B[0]}{ea}',1),(f'T{nn}_R0:1C7{sa}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R0C6:7_za{ea}'],[                  (f'{B[0]}{ea}',1),(f'T{nn}_R0:1C7{sa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R0:1C7{sa}', f'{BLANK}_p{en}{ea}'],[(f'T{nn}_R0:1C7{sa}',1),(f'{BLANK}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R0:1C7{sa}', f'T{nn}_R1C6:7{ea}'],[(f'T{nn}_R1:2C7{sa}',1),(f'{B[1]}{ea}',1)], symbcol(B[0]), prefix))



	ttypes.append(createType([f'T{nn}_R1:2C7{sa}', f'{GS_PRE}_z{en}{ea}'],[(f'T{nn}_R1:2C7_z{sa}',1)], symbcol(B[0]), prefix))
	for w_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'T{nn}_R1:2C7_z{sa}', f'{w_in}_p{en}{ea}'],[(f'T{nn}_R1:2C7_z{sa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R1:2C7{sa}', f'{GS_PRE}_n{en}{ea}'],[(f'T{nn}_R1:2C7_t{nn}_n{sa}',1),(f'{GS_PRE}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R1:2C7_t{nn}_n{sa}', f'0_p{en}{ea}'],[(f'T{nn}_R1:2C7_t{nn}_n{sa}',1),(f'0{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R1:2C7_t{nn}_n{sa}', f'{GS_PRE}_p{en}{ea}'],[(f'T{nn}_R1:2C7_t{en}_n_msb{sa}',1),(f'{GS_PRE}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R1:2C7_t{en}_n_msb{sa}', f'0_p{en}{ea}'],[(f'T{nn}_R1:2C7_t{en}_n{sa}',1),(f'0{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R1:2C7_t{en}_n_msb{sa}', f'1_p{en}{ea}'],[(f'T{nn}_R1:2C7_t{en}_n{sa}',1),(f'1!{ea}',2)], symbcol(B[0]), prefix)) ### <- THIS ONE
	# ttypes.append(createType([f'T{nn}_R1:2C7_t{en}_n_msb{sa}', f'1_p{en}{ea}'],[(f'T{nn}_R1:2C7_t{en}_n{sa}',1),(f'1!{ea}',1)], symbcol(B[0]), prefix))

	for w_in in ['0','1']:
		ttypes.append(createType([f'T{nn}_R1:2C7_t{en}_n{sa}', f'{w_in}_p{en}{ea}'],[(f'T{nn}_R1:2C7_t{en}_n{sa}',1),(f'{w_in}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R1:2C7_t{en}_n{sa}', f'{GS_PRE}_p{en}{ea}'],[(f'T{nn}_R1:2C7_t{sn}_n{sa}',1),(f'{GS_PRE}{ea}',1)], symbcol(B[0]), prefix))
	for w_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'T{nn}_R1:2C7_t{sn}_n{sa}', f'{w_in}_p{en}{ea}'],[(f'T{nn}_R1:2C7_t{sn}_n{sa}',1),(f'{w_in}{ea}',1)], symbcol(B[0]), prefix))


	ttypes.append(createType([f'T{nn}_R1:2C7_t{sn}_n{sa}', f'T{nn}_R2C6:7{ea}'],[(f'T{nn}_R2:3C7_n{sa}',1),(f'{B[2]}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R1:2C7_z{sa}', f'T{nn}_R2C6:7{ea}'],[(f'T{nn}_R2:3C7_z{sa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R2:3C7_z{sa}', f'{BLANK}_p{en}{ea}'],[(f'T{nn}_R2:3C7_z{sa}',1)                   ], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R2:3C7_n{sa}', f'{BLANK}_p{en}{ea}'],[(f'T{nn}_R2:3C7_n{sa}',1),(f'{BLANK}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R2:3C7_z{sa}', f'T{nn}_R3C6:7{ea}'],[(f'T{nn}_R3:4C7_z{sa}',1)                  ], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R2:3C7_n{sa}', f'T{nn}_R3C6:7{ea}'],[(f'T{nn}_R3:4C7_n{sa}',1),(f'{B[3]}{ea}',1)], symbcol(B[0]), prefix))

	for w_in in ['0','1',GS_PRE,GTE_SEP1, GTE_SEP2, GT_PRE]:
		ttypes.append(createType([f'T{nn}_R3:4C7_z{sa}', f'{w_in}_p{en}{ea}'],[(f'T{nn}_R3:4C7_z{sa}',1)                  ], symbcol(B[0]), prefix))
		ttypes.append(createType([f'T{nn}_R3:4C7_n{sa}', f'{w_in}_p{en}{ea}'],[(f'T{nn}_R3:4C7_n{sa}',1),(f'{w_in}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R3:4C7_z{sa}', f'T{nn}_R4C6:7{ea}'],[(f'T{nn}_R4:5C7_z{sa}',1)                  ], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R3:4C7_n{sa}', f'T{nn}_R4C6:7{ea}'],[(f'T{nn}_R4:5C7_n{sa}',1),(f'{B[4]}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R4:5C7_z{sa}', f'{BLANK}_p{en}{ea}'],[(f'T{nn}_R4:5C7_z{sa}',1)                   ], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R4:5C7_n{sa}', f'{BLANK}_p{en}{ea}'],[(f'T{nn}_R4:5C7_n{sa}',1),(f'{BLANK}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R4:5C7_z{sa}', f'T{nn}_R5C6:7{ea}'],[(f'T{nn}_R5:6C7_z{sa}',1)                  ], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R4:5C7_n{sa}', f'T{nn}_R5C6:7{ea}'],[(f'T{nn}_R5:6C7_n{sa}',1),(f'{B[5]}{ea}',1)], symbcol(B[0]), prefix))

	for w_in in ['0','1',GS_PRE]:
		ttypes.append(createType([f'T{nn}_R5:6C7_z{sa}', f'{w_in}_p{en}{ea}'],[(f'T{nn}_R5:6C7_z{sa}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R5:6C7_n{sa}', f'{GS_PRE}_p{en}{ea}'],[(f'T{nn}_R5:6C7_t{nn}_n{sa}',1),(f'{GS_PRE}{ea}',1)], symbcol(B[0]), prefix))
	for w_in in ['0','1']:
		ttypes.append(createType([f'T{nn}_R5:6C7_t{nn}_n{sa}', f'{w_in}_p{en}{ea}'],[(f'T{nn}_R5:6C7_t{nn}_n{sa}',1),(f'0{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R5:6C7_t{nn}_n{sa}', f'{GS_PRE}_p{en}{ea}'],[(f'T{nn}_R5:6C7_t{en}_n{sa}',1),(f'{GS_PRE}{ea}',1)], symbcol(B[0]), prefix))
	for w_in in ['0','1']:
		ttypes.append(createType([f'T{nn}_R5:6C7_t{en}_n{sa}', f'{w_in}_p{en}{ea}'],[(f'T{nn}_R5:6C7_t{en}_n{sa}',1),(f'{w_in}{ea}',1)], symbcol(B[0]), prefix))
	
	ttypes.append(createType([f'T{nn}_R5:6C7_t{en}_n{sa}', f'{GS_PRE}_p{en}{ea}'],[(f'T{nn}_R5:6C7_t{sn}_n{sa}',1),(f'{GS_PRE}{ea}',1)], symbcol(B[0]), prefix))
	for w_in in ['0','1']:
		ttypes.append(createType([f'T{nn}_R5:6C7_t{sn}_n{sa}', f'{w_in}_p{en}{ea}'],[(f'T{nn}_R5:6C7_t{sn}_n{sa}',1),(f'0{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R5:6C7_t{sn}_n{sa}', f'{GS_PRE}_p{en}{ea}'],[(f'T{nn}_R5:6C7_t{sn}_n{sa}',1),(f'{GS_PRE}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R5:6C7_t{sn}_n{sa}', f'T{nn}_R6C6:7{ea}'],[(f'T{nn}_R6:7C7_n{sa}',1),(f'{B[6]}{ea}',1)], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R5:6C7_z{sa}', f'T{nn}_R6C6:7{ea}'],[(f'T{nn}_R6:7C7_z{sa}',1)], symbcol(B[0]), prefix))
	
	ttypes.append(createType([f'T{nn}_R6:7C7_z{sa}', f'{BLANK}_p{en}{ea}'],[(f'T{nn}_R6:7C7_z{sa}',1)                   ], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R6:7C7_n{sa}', f'{BLANK}_p{en}{ea}'],[(f'T{nn}_R6:7C7_n{sa}',1),(f'{BLANK}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R6:7C7_z{sa}', f'{BLANK}_pz{en}{ea}'],[(f'T{nn}_R6:7C7_z{sa}',1),                  ], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R6:7C7_n{sa}', f'{BLANK}_pz{en}{ea}'],[(f'T{nn}_R6:7C7_n{sa}',1),(f'{BLANK}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R6:7C7_z{sa}', f'C{nn}_R7C6:7{ea}'],[                 ], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R6:7C7_n{sa}', f'C{nn}_R7C6:7{ea}'],[(f'{B[7]}{ea}',1)], symbcol(B[0]), prefix))

	ttypes.append(createType([f'T{nn}_R6:7C7_z{sa}', f'S{nn}_R7C6:7{ea}'],[                 ], symbcol(B[0]), prefix))
	ttypes.append(createType([f'T{nn}_R6:7C7_n{sa}', f'S{nn}_R7C6:7{ea}'],[(f'{B[7]}{ea}',1)], symbcol(B[0]), prefix))
	
	'''